// Copyright (c) 2010  John Abbott
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "This program illustrates use of the pseudo-random number generator of CoCoALib.\n"
  "The numbers are independent and uniformly distributed in the given range; both \n"
  "ends of the range are reachable.                                               \n"
  "See RandomBitStream if you want random bits.                                   \n";

const string LongDescription =
  "CoCoALib offers a way to make uniform pseudo-random number generators.     \n"
  "When creating the generator you must specify the upper and lower bounds for\n"
  "the random numbers which wll be generated (both bounds may be generated).  \n"
  "A generator seeded when it is created; this allows different pseudo-random \n"
  "sequences to be produced, though the sequence is completely determined by  \n"
  "the initial seed value.                                                    \n";
//----------------------------------------------------------------------

void program()
{
  GlobalManager CoCoAFoundations;

  GlobalOutput() << ShortDescription << endl;

  GlobalOutput() << "Here are 20 random integers in the range [-10,+10]\n";
  RandomLongStream RLS(-10,10);
  for (int i=0; i < 20; ++i)
    GlobalOutput() << sample(RLS) << " ";
  GlobalOutput() << endl << endl;

  // If you prefer you can RLS as an (endless) input iterator
  GlobalOutput() << "Here are 20 more random integers in the range [0,99]\n";
  RandomLongStream RLS2(0,99);
  for (int i=0; i < 20; ++i)
  {
    ++RLS2;
    GlobalOutput() << *RLS2 << " ";
  }
  GlobalOutput() << endl;
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-RandomLongStream1.C,v 1.1 2010/02/16 10:19:29 abbott Exp $
// $Log: ex-RandomLongStream1.C,v $
// Revision 1.1  2010/02/16 10:19:29  abbott
// Added new class RandomLongStream with example and test.
//
// Revision 1.4  2008/10/07 12:12:54  abbott
// Removed useless commented out #include.
//
// Revision 1.3  2007/05/31 16:06:16  bigatti
// -- removed previous unwanted checked-in version
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.9  2007/03/07 11:51:40  bigatti
// -- improved test alignment
//
// Revision 1.8  2007/03/03 14:15:45  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.7  2007/03/02 17:46:40  bigatti
// -- unique RingZ and RingQ
// -- requires foundations.H ;  foundations blah;  (thik of a better name)
//
// Revision 1.6  2007/03/02 10:47:53  cocoa
// First stage of RingZ modifications -- tests do not compile currently, Anna will fix this.
//
// Revision 1.5  2007/03/01 13:52:59  bigatti
// -- minor: fixed typo
//
// Revision 1.4  2007/02/28 15:15:56  bigatti
// -- minor: removed quotes in description
//
// Revision 1.3  2007/02/12 16:27:43  bigatti
// -- added strings ShortDescription and LongDescription for indexing
//
// Revision 1.2  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.1  2006/03/12 21:28:34  cocoa
// Major check in after many changes
//
