// Copyright (c) 2003-2006  John Abbott, Anna Bigatti
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "An example about RingWeyl, the interface is not quite settled yet.\n";

const string LongDescription =
  "All these examples about RingWeyl will probably be merged into one.\n";
//----------------------------------------------------------------------

// Includes from the standard C++ library
// #include <iostream> // using std::endl;

void program()
{
  GlobalManager CoCoAFoundations;

  GlobalOutput() << ShortDescription << endl;

  ring Q = RingQ();
  ring Fp = NewZmod(3); // COMPUTE THE RESULT MODULO 3

  vector<symbol> names;
  names.push_back(symbol("u"));
  names.push_back(symbol("v"));
  names.push_back(symbol("x"));
  names.push_back(symbol("y"));
  names.push_back(symbol("z"));
  names.push_back(symbol("t"));  

  const size_t NumInds = 6;
  vector<size_t> ElimIndets;
  ElimIndets.push_back(0); // elim u
  ElimIndets.push_back(1); // elim v
  SparsePolyRing WA = NewWeylAlgebra(Q, names, ElimIndets);

  RingElem u = indet(WA, 0);
  RingElem v = indet(WA, 1);
  RingElem x = indet(WA, 2);
  RingElem y = indet(WA, 3);
  RingElem z = indet(WA, 4);
  RingElem t = indet(WA, 5);

  RingElem dx = indet(WA, 2+NumInds);
  RingElem dy = indet(WA, 3+NumInds);
  RingElem dz = indet(WA, 4+NumInds);
  RingElem dt = indet(WA, 5+NumInds);

  GlobalOutput() << "indets(WA) = " << indets(WA) << endl;
  GlobalOutput() << "x*dx = " << x*dx << "    dx*x = " << dx*x << endl;
  GlobalOutput() << "y*dy = " << y*dy << "    dy*y = " << dy*y << endl;
  GlobalOutput() << "z*dz = " << z*dz << "    dz*z = " << dz*z << endl;
  GlobalOutput() << "t*dt = " << t*dt << "    dt*t = " << dt*t << endl;
  GlobalOutput() << "wdeg(x) = " << wdeg(x) <<"  wdeg(dx) = " <<wdeg(dx) <<endl;

  vector<RingElem> g;
  g.push_back(x); 
  g.push_back(dx); 

  ideal K = ideal(WA, g);
  GlobalOutput() << "gens(K) = " << gens(K) << endl;
  GlobalOutput() << "TidyGens(K) = " << TidyGens(K) << endl;

  //-------
  g.clear();
  g.push_back(1-u*v);
  g.push_back(t*u - (x*z+y)*(power(x,6)-power(y,7)));
  g.push_back(v*(z*(power(x,6)-power(y,7)) + (x*z+y)*6*power(x,5))*dt + dx);
  g.push_back(v*((power(x,6)-power(y,7)) - 7*power(y,6)*(x*z+y) )*dt + dy);
  g.push_back(v*(x*(power(x,6)-power(y,7)))*dt + dz);
  
  ideal I = ideal(WA, g);
  GlobalOutput() << "gens(I) = " << gens(I) << endl;
  // ----CoCoA can't manage this example (yet)----
  //   GlobalOutput() << "TidyGens(I) = " << TidyGens(I) << endl;

  GlobalOutput() << endl;

  //-------
  g.clear();
//1-uv,tu-x,vdt+dx
  g.push_back(1-u*v);
  g.push_back(t*u-x*x*x-y*y*y-z*z*z);
  g.push_back(3*x*x*v*dt+dx);
  g.push_back(3*y*y*v*dt+dy);
  g.push_back(3*z*z*v*dt+dz); 

  // was in file WeylAlgebra9.C
  ideal J = ideal(WA, g);
  GlobalOutput() << "gens(J)=" << gens(J) << endl;
  GlobalOutput() << "TidyGens(J)=" << TidyGens(J) << endl;

}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//output
//gens(I)=[
//-x[0]*x[1] +(1),
//x[0]*x[5] +-x[2]^3 +-x[3]^3 +-x[4]^3,
//(3)*x[1]*x[2]^2*d[5] +d[2],
//(3)*x[1]*x[3]^2*d[5] +d[3],
//(3)*x[1]*x[4]^2*d[5] +d[4]]
 
//TidyGens(I)=[
//(3)*x[1]*x[4]^2*d[5] +d[4],
//(3)*x[1]*x[3]^2*d[5] +d[3],
//-x[4]^2*d[3] +x[3]^2*d[4],
//(3)*x[1]*x[2]^2*d[5] +d[2],
//-x[4]^2*d[2] +x[2]^2*d[4],
//x[3]^2*d[2]*d[4] +-x[2]^2*d[3]*d[4],
//(-2)*x[3]^2*x[4]*d[2] +(2)*x[2]^2*x[4]*d[3],
//-x[3]^2*d[2] +x[2]^2*d[3],
//x[0]*x[5] +-x[2]^3 +-x[3]^3 +-x[4]^3,
//(-3)*x[0]*x[1]*x[4]^2 +(3)*x[4]^2,
//x[0]*d[4] +(-3)/(-1) *x[4]^2*d[5],
//-x[2]^3*d[4] +-x[3]^3*d[4] +-x[4]^3*d[4] +(3)/(-1) *x[4]^2*x[5]*d[5] 
//+(-3)*x[4]^2,
//(-2)*x[2]^3*x[4]*d[3] +(-2)*x[3]^3*x[4]*d[3] +(-2)*x[3]^2*x[4]^2*d[4] 
//+(-6)*x[3]^2*x[4]*x[5]*d[5] +(6)/(-1) *x[3]^2*x[4],
//x[2]^3*d[3] +x[3]^3*d[3] +x[3]^2*x[4]*d[4] +(-3)/(-1) *x[3]^2*x[5]*d[5] +(3)*x[3]^2,
//(-2)*x[2]^3*x[4]*d[2] +(2)/(-1) *x[2]^2*x[3]*x[4]*d[3] +(-2)*x[2]^2*x[4]^2*d[4]
//+(-6)*x[2]^2*x[4]*x[5]*d[5] +(6)/(-1) *x[2]^2*x[4],
//x[2]^3*d[2] +x[2]^2*x[3]*d[3] +x[2]^2*x[4]*d[4] +(-3)/(-1) *x[2]^2*x[5]*d[5] 
//+(3)*x[2]^2,
//x[2]*d[2]*d[4] +x[3]*d[3]*d[4] +x[4]*d[4]^2 +(3)*x[5]*d[4]*d[5] +(-4)/(-1) *d[4],
//(-2)*x[2]*x[4]*d[2] +(-2)*x[3]*x[4]*d[3] +(-2)*x[4]^2*d[4] +(-6)*x[4]*x[5]*d[5]
//+(6)/(-1) *x[4],
//-x[2]*d[2] +-x[3]*d[3] +-x[4]*d[4] +(3)/(-1) *x[5]*d[5] +(-3),
//(3)/(-1) *x[1]*x[2]*x[3]*d[3]*d[5] +(3)/(-1) *x[1]*x[2]*x[4]*d[4]*d[5] 
//+(9)/(-1) *x[1]*x[2]*x[5]*d[5]^2 +(12)/(-1) *x[1]*x[2]*d[5] +d[2]^2,
//(-2)*x[1]*x[3]*x[4]*d[3]*d[4]*d[5] +(-6)*x[1]*x[3]*x[5]*d[3]*d[5]^2 
//+(6)/(-1) *x[1]*x[4]*x[5]*d[4]*d[5]^2 +(9)/(-1) *x[1]*x[5]^2*d[5]^3 
//+(-6)*x[1]*x[3]*d[3]*d[5] +(6)/(-1) *x[1]*x[4]*d[4]*d[5] +(-36)*x[1]*x[5]*d[5]^2 
//+(16)/(-1) *x[1]*d[5] +(1)/(-3) *d[2]^3 +(-1)/(-3) *d[3]^3 +(-1)/(-3) *d[4]^3,
//(3)/(-1) *x[1]*x[2]*x[3]*x[4]*d[4]*d[5] +(9)/(-1) *x[1]*x[2]*x[3]*x[5]*d[5]^2 
//+(6)/(-1) *x[1]*x[2]*x[3]*d[5] +x[3]*d[2]^2 +x[2]*d[3]^2,
//(9)/(-1) *x[1]*x[3]*x[4]*x[5]*d[3]*d[5]^2 +(-27)/(2) *x[1]*x[4]*x[5]^2*d[5]^3 
//+(3)/(-1) *x[1]*x[3]*x[4]*d[3]*d[5] +(36)/(-1) *x[1]*x[4]*x[5]*d[5]^2 
//+(-6)*x[1]*x[4]*d[5] +(-1)/(2) *x[4]*d[2]^3 +(1)/(2) *x[4]*d[3]^3 +x[3]*d[3]*d[4]^2 
//+(1)/(2) *x[4]*d[4]^3 +(3)*x[5]*d[4]^2*d[5] +(3)*d[4]^2,
//(-9)*x[1]*x[3]*x[4]*x[5]*d[4]*d[5]^2 +(-27)/(2) *x[1]*x[3]*x[5]^2*d[5]^3 
//+(-3)*x[1]*x[3]*x[4]*d[4]*d[5] +(-36)*x[1]*x[3]*x[5]*d[5]^2 +(6)/(-1) *x[1]*x[3]*d[5] 
//+(-1)/(2) *x[3]*d[2]^3 +(1)/(2) *x[3]*d[3]^3 +x[4]*d[3]^2*d[4] +(1)/(2) *x[3]*d[4]^3 
//+(-3)/(-1) *x[5]*d[3]^2*d[5] +(-3)/(-1) *d[3]^2,
//(9)/(-1) *x[1]*x[2]*x[4]*x[5]*d[4]*d[5]^2 +(-27)/(2) *x[1]*x[2]*x[5]^2*d[5]^3 
//+(3)/(-1) *x[1]*x[2]*x[4]*d[4]*d[5] +(36)/(-1) *x[1]*x[2]*x[5]*d[5]^2 
//+(6)/(-1) *x[1]*x[2]*d[5] +(-1)/(2) *x[3]*d[2]^2*d[3] +(1)/(-2) *x[2]*d[3]^3 
//+(1)/(2) *x[4]*d[2]^2*d[4] +(1)/(2) *x[2]*d[4]^3 +(3)/(2) *x[5]*d[2]^2*d[5] 
//+(1)/(2) *d[2]^2,
//(9)/(-1) *x[1]*x[2]*x[3]*x[4]*x[5]*d[5]^2 +x[3]*x[4]*d[2]^2 +x[2]*x[4]*d[3]^2 
//+x[2]*x[3]*d[4]^2,
//(-6)*x[1]*x[3]*x[5]^2*d[3]*d[5]^3 +(6)/(-1) *x[1]*x[5]^3*d[5]^4 
//+(12)/(-1) *x[1]*x[3]*x[5]*d[3]*d[5]^2 +(36)/(-1) *x[1]*x[5]^2*d[5]^3 
//+(4)/(-3) *x[1]*x[3]*d[3]*d[5] +(-112)/(3) *x[1]*x[5]*d[5]^2 +(8)/(-3) *x[1]*d[5] 
//+(2)/(-27) *x[3]*d[2]^3*d[3] +(-2)/(-27) *x[3]*d[3]^4 +(4)/(27) *x[4]*d[2]^3*d[4] 
//+(-2)/(9) *x[3]*d[3]*d[4]^3 +(-4)/(27) *x[4]*d[4]^4 +(2)/(-9) *x[5]*d[2]^3*d[5] 
//+(-2)/(-3) *x[5]*d[3]^3*d[5] +(2)/(-3) *x[5]*d[4]^3*d[5] +(-4)/(-9) *d[3]^3 +(-8)/(9) *d[4]^3,
//(-9)/(2) *x[1]*x[4]*x[5]^2*d[4]*d[5]^3 +(-9)/(2) *x[1]*x[5]^3*d[5]^4 
//+(-9)*x[1]*x[4]*x[5]*d[4]*d[5]^2 +(27)/(-1) *x[1]*x[5]^2*d[5]^3 +-x[1]*x[4]*d[4]*d[5] 
//+(-28)*x[1]*x[5]*d[5]^2 +(-2)*x[1]*d[5] +(-1)/(-9) *x[3]*d[2]^3*d[3] +(1)/(-9) *x[3]*d[3]^4 
//+(-1)/(18) *x[4]*d[2]^3*d[4] +(1)/(-6) *x[4]*d[3]^3*d[4] +(1)/(18) *x[4]*d[4]^4 
//+(-1)/(6) *x[5]*d[2]^3*d[5] +(-1)/(2) *x[5]*d[3]^3*d[5] +(1)/(2) *x[5]*d[4]^3*d[5] 
//+(2)/(-3) *d[3]^3 +(-1)/(-3) *d[4]^3,
//(9)/(-2) *x[1]*x[3]*x[4]*x[5]^2*d[5]^3 +(6)/(-1) *x[1]*x[3]*x[4]*x[5]*d[5]^2 
//+(1)/(-6) *x[3]*x[4]*d[2]^3 +(-1)/(-6) *x[3]*x[4]*d[3]^3 +(1)/(3) *x[3]^2*d[3]*d[4]^2 
//+(-1)/(-6) *x[3]*x[4]*d[4]^3 +x[4]*x[5]*d[3]^2*d[5] +x[3]*x[5]*d[4]^2*d[5] 
//+(1)/(3) *x[4]*d[3]^2 +x[3]*d[4]^2,
//(-9)/(2) *x[1]*x[2]*x[4]*x[5]^2*d[5]^3 +(6)/(-1) *x[1]*x[2]*x[4]*x[5]*d[5]^2 
//+(-1)/(6) *x[3]*x[4]*d[2]^2*d[3] +(1)/(-6) *x[2]*x[4]*d[3]^3 
//+(-1)/(6) *x[2]*x[3]*d[3]*d[4]^2 +(1)/(2) *x[4]*x[5]*d[2]^2*d[5] 
//+(-1)/(-2) *x[2]*x[5]*d[4]^2*d[5] +(1)/(-6) *x[4]*d[2]^2 +(1)/(-6) *x[2]*d[4]^2,
//(9)/(-2) *x[1]*x[2]*x[3]*x[5]^2*d[5]^3 +(6)/(-1) *x[1]*x[2]*x[3]*x[5]*d[5]^2 
//+(1)/(-6) *x[3]*x[4]*d[2]^2*d[4] +(1)/(-6) *x[2]*x[4]*d[3]^2*d[4] 
//+(-1)/(6) *x[2]*x[3]*d[4]^3 +(-1)/(-2) *x[3]*x[5]*d[2]^2*d[5] 
//+(-1)/(-2) *x[2]*x[5]*d[3]^2*d[5] +(1)/(-6) *x[3]*d[2]^2 +(-1)/(6) *x[2]*d[3]^2,
//(3)/(-1) *x[1]*x[4]*x[5]^3*d[5]^4 +(-12)*x[1]*x[4]*x[5]^2*d[5]^3 
//+(-20)/(3) *x[1]*x[4]*x[5]*d[5]^2 +(2)/(27) *x[3]*x[4]*d[2]^3*d[3] 
//+(-2)/(27) *x[3]*x[4]*d[3]^4 +(4)/(-27) *x[3]^2*d[3]^2*d[4]^2 
//+(-2)/(27) *x[3]*x[4]*d[3]*d[4]^3 +(1)/(-9) *x[4]*x[5]*d[2]^3*d[5] 
//+(1)/(-3) *x[4]*x[5]*d[3]^3*d[5] +(-2)/(9) *x[3]*x[5]*d[3]*d[4]^2*d[5] 
//+(-1)/(-9) *x[4]*x[5]*d[4]^3*d[5] +(2)/(3) *x[5]^2*d[4]^2*d[5]^2 +(2)/(27) *x[4]*d[2]^3 
//+(2)/(-9) *x[4]*d[3]^3 +(-20)/(27) *x[3]*d[3]*d[4]^2 +(2)/(-27) *x[4]*d[4]^3 
//+(-8)/(-9) *x[5]*d[4]^2*d[5] +(4)/(-9) *d[4]^2,
//(9)/(-2) *x[1]*x[3]*x[5]^3*d[5]^4 +(-18)*x[1]*x[3]*x[5]^2*d[5]^3 
//+(-10)*x[1]*x[3]*x[5]*d[5]^2 +(-1)/(-9) *x[3]*x[4]*d[2]^3*d[4] 
//+(-1)/(9) *x[3]*x[4]*d[3]^3*d[4] +(-2)/(9) *x[3]^2*d[3]*d[4]^3 
//+(1)/(-9) *x[3]*x[4]*d[4]^4 +(1)/(-6) *x[3]*x[5]*d[2]^3*d[5] 
//+(-1)/(-6) *x[3]*x[5]*d[3]^3*d[5] +(1)/(-3) *x[4]*x[5]*d[3]^2*d[4]*d[5] 
//+(1)/(-2) *x[3]*x[5]*d[4]^3*d[5] +x[5]^2*d[3]^2*d[5]^2 +(-1)/(-9) *x[3]*d[2]^3 
//+(-1)/(9) *x[3]*d[3]^3 +(2)/(-9) *x[4]*d[3]^2*d[4] +(-7)/(9) *x[3]*d[4]^3 
//+(4)/(3) *x[5]*d[3]^2*d[5] +(2)/(-9) *d[3]^2,
//(-9)/(2) *x[1]*x[2]*x[5]^3*d[5]^4 +(-18)*x[1]*x[2]*x[5]^2*d[5]^3 
//+(10)/(-1) *x[1]*x[2]*x[5]*d[5]^2 +(-1)/(-9) *x[3]*x[4]*d[2]^2*d[3]*d[4] 
//+(-1)/(-9) *x[2]*x[4]*d[3]^3*d[4] +(-1)/(-9) *x[2]*x[3]*d[3]*d[4]^3 
//+(1)/(-6) *x[3]*x[5]*d[2]^2*d[3]*d[5] +(1)/(-6) *x[2]*x[5]*d[3]^3*d[5] 
//+(1)/(-6) *x[4]*x[5]*d[2]^2*d[4]*d[5] +(-1)/(6) *x[2]*x[5]*d[4]^3*d[5] 
//+(-1)/(-2) *x[5]^2*d[2]^2*d[5]^2 +(1)/(9) *x[3]*d[2]^2*d[3] +(-1)/(-9) *x[2]*d[3]^3 
//+(-1)/(-9) *x[4]*d[2]^2*d[4] +(-1)/(-9) *x[2]*d[4]^3 +(1)/(3) *x[5]*d[2]^2*d[5] 
//+(-1)/(-9) *d[2]^2,
//(-9)/(2) *x[1]*x[5]^4*d[5]^5 +(-36)*x[1]*x[5]^3*d[5]^4 +(64)/(-1) *x[1]*x[5]^2*d[5]^3 
//+(-20)*x[1]*x[5]*d[5]^2 +(-1)/(9) *x[3]*x[4]*d[2]^3*d[3]*d[4] 
//+(1)/(9) *x[3]*x[4]*d[3]^4*d[4] +(-2)/(-9) *x[3]^2*d[3]^2*d[4]^3 
//+(1)/(9) *x[3]*x[4]*d[3]*d[4]^4 +(-1)/(-9) *x[3]*x[5]*d[2]^3*d[3]*d[5] 
//+(1)/(-9) *x[3]*x[5]*d[3]^4*d[5] +(-1)/(-9) *x[4]*x[5]*d[2]^3*d[4]*d[5] 
//+(1)/(3) *x[4]*x[5]*d[3]^3*d[4]*d[5] +(1)/(3) *x[3]*x[5]*d[3]*d[4]^3*d[5] 
//+(1)/(-9) *x[4]*x[5]*d[4]^4*d[5] +(-1)/(6) *x[5]^2*d[2]^3*d[5]^2 
//+(-1)/(2) *x[5]^2*d[3]^3*d[5]^2 +(-1)/(2) *x[5]^2*d[4]^3*d[5]^2 
//+(-1)/(9) *x[3]*d[2]^3*d[3] +(1)/(9) *x[3]*d[3]^4 +(-1)/(9) *x[4]*d[2]^3*d[4] 
//+(-1)/(-3) *x[4]*d[3]^3*d[4] +(11)/(9) *x[3]*d[3]*d[4]^3 +(-1)/(-9) *x[4]*d[4]^4 
//+(-2)/(3) *x[5]*d[3]^3*d[5] +(2)/(-3) *x[5]*d[4]^3*d[5] +(-1)/(9) *d[2]^3 
//+(-1)/(-3) *d[3]^3 +(7)/(9) *d[4]^3,
//(-2)*x[0]*x[4]*d[3] +(6)/(-1) *x[3]^2*x[4]*d[5],
//-x[0]*d[3] +(-3)*x[3]^2*d[5],
//(-2)*x[0]*x[4]*d[2] +(6)/(-1) *x[2]^2*x[4]*d[5],
//-x[0]*d[2] +(-3)*x[2]^2*d[5],
//(-6)*x[0]*x[1]*x[4] +(6)*x[4],
//-x[0]*x[1] +(-1)/(-1) ,
//-x[1]*x[2]^3 +-x[1]*x[3]^3 +-x[1]*x[4]^3 +x[5]]

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-RingWeyl1.C,v 1.2 2008/11/19 09:19:48 bigatti Exp $
// $Log: ex-RingWeyl1.C,v $
// Revision 1.2  2008/11/19 09:19:48  bigatti
// -- added easy example:  ideal(x, dx)
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.8  2007/03/03 14:15:45  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.7  2007/02/26 17:18:22  bigatti
// -- getting ready for unique ring Z: using NewZmod(N), NewRingQ()
//
// Revision 1.6  2007/02/12 16:15:37  bigatti
// -- added strings ShortDescription and LongDescription for indexing
//
// Revision 1.5  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.4  2006/08/30 15:22:12  cocoa
// -- added example from WeylAlgebra9.C
//
// Revision 1.3  2006/08/17 10:03:24  cocoa
// -- creation of RingWeyl with given list of symbols
// -- updated header
//
