// Copyright (c) 2003-2006  John Abbott, Anna Bigatti
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "An example about RingWeyl, the interface is not quite settled yet.\n";

const string LongDescription =
  "This shows a computation of a Groebner Basis.\n"
  "All these examples about RingWeyl will probably be merged into one.\n";
//----------------------------------------------------------------------

// Includes from the standard C++ library
// #include <iostream> // using std::endl;

void program()
{
  GlobalManager CoCoAFoundations;

  GlobalOutput() << ShortDescription << endl;

  ring Q = RingQ();
  ring F32003 = NewZmod(32003);

  const size_t NumInds = 4;
  vector<size_t> ElimIndets;
  SparsePolyRing WA = NewWeylAlgebra(Q, NumInds, ElimIndets);

  RingElem u = indet(WA, 0);
  RingElem v = indet(WA, 1);
  RingElem x = indet(WA, 2);
  RingElem y = indet(WA, 3);

  RingElem dx = indet(WA, 2+NumInds);
  RingElem dy = indet(WA, 3+NumInds);

  vector<RingElem> g;
  g.push_back(3*x*dx +2*y*dy +6);
  g.push_back(3*y*dx +2*x*dy);
  ideal I = ideal(WA, g);
  GlobalOutput() << "gens(I) = " << gens(I) << endl;
  GlobalOutput() << "TidyGens(I) = " << TidyGens(I) << endl;
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//output
// gens(I)=[3*x[2]*d[2] +2*x[3]*d[3] +6,  3*x[3]*d[2] +2*x[2]*d[3]]
// TidyGens(I)=[x[3]*d[2],  x[2]*d[2],  x[2]^2*d[3],  x[2]*d[3],  x[3]*d[3]^2,  x[3]^2*d[3],  d[2]*d[3],  d[2],  x[3]*d[3],  d[3],  1]

// ANNA: so is it [(1)] ???

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-RingWeyl2.C,v 1.2 2007/09/24 14:12:37 abbott Exp $
// $Log: ex-RingWeyl2.C,v $
// Revision 1.2  2007/09/24 14:12:37  abbott
// Added missing newline in a string.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.5  2007/03/03 14:15:45  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.4  2007/02/26 17:18:22  bigatti
// -- getting ready for unique ring Z: using NewZmod(N), NewRingQ()
//
// Revision 1.3  2007/02/12 16:15:37  bigatti
// -- added strings ShortDescription and LongDescription for indexing
//
// Revision 1.2  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.1  2006/08/17 10:08:18  cocoa
// -- modernized version of WeylAlgebraN example
//
