// Copyright (c) 2006-2007,2009  John Abbott
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

#include <cstdlib>
// using exit

//----------------------------------------------------------------------
const string ShortDescription =
  "Program to demonstrate changing the global i/o channels. \n";

const string LongDescription =
  "This example shows how to change the global output channel GlobalOutput. \n"
  "The most important point is that the variable  out  is at top-level      \n"
  "inside program().  If it were inside the then-block, the ofstream        \n"
  "would be destroyed as soon as control leaves the then-block; and at      \n"
  "that point the global output stream would contain a dangling reference.  \n";
//----------------------------------------------------------------------

// Includes from the standard C++ library
// #include <iostream> // using std::endl;
#include <fstream> // using std::ofstream;


void program()
{
  GlobalManager CoCoAFoundations;

  GlobalOutput() << ShortDescription << endl;

  GlobalOutput() << "Type in an integer: ";
  ZZ N;
  GlobalInput() >> N;
  if (!GlobalInput()) { GlobalErrput() << "ex-io: bad input" << endl; exit(1);}

  ofstream out; // MUST NOT be inside the then-block, o/w out is destroyed too early!
  if (IsEven(N))
  {
    GlobalOutput() << "Since the integer supplied was even, further output" << endl
                   << "will be sent to the file ex-io.out." << endl;
    out.open("ex-io.out");
    if (out)                 // if the file was opened successfully...
      SetGlobalOutput(out);  // ...use it for subsequent output.
    else
      GlobalOutput() << "Failed to open ex-io.out; continuing with previous ostream." << endl;
  }

  GlobalOutput() << "The integer read as input was: " << N << endl;
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-io.C,v 1.2 2009/10/29 18:25:40 abbott Exp $
// $Log: ex-io.C,v $
// Revision 1.2  2009/10/29 18:25:40  abbott
// Added missing include of cstdlib for exit.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.4  2007/03/03 14:15:45  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.3  2007/02/12 16:27:43  bigatti
// -- added strings ShortDescription and LongDescription for indexing
//
// Revision 1.2  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.1  2006/05/22 15:52:16  cocoa
// Added preprocess-disg algorithm to ApproxPts.
// Sundry minor improvements.
//
// Revision 1.1  2006/03/12 21:28:34  cocoa
// Major check in after many changes
//
