// Copyright (c) 2005  Anna Bigatti
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "Example program illustrating the creation of free modules, and \n"
  "some operations on them. \n";

const string LongDescription =
  "Please note that the module code is still rather young. \n";
//----------------------------------------------------------------------

// Includes from the standard C++ library
// #include <iostream> // using std::endl;


void trial(ring R)
{
  const size_t NumComps = 4;
  const FreeModule F = NewFreeModule(R, NumComps);
  const vector<ModuleElem>& e = gens(F);

  const ModuleElem u = e[0] + 2*e[1];
  const ModuleElem v = 4*e[0] + 3*e[3];
  const RingElem a = 2 * one(BaseRing(F));  // BaseRing(F) is R

  GlobalOutput() << "---- F = " << F << " ----" << endl;
  GlobalOutput() << "u[0] = " << u[0] << ";  u[1] = " << u[1] << endl;
  GlobalOutput() << "u + v = " << u + v << endl;
  GlobalOutput() << "u * a = " << u * a << endl;
  GlobalOutput() << "a * v = " << a * v << endl;
}


void program()
{
  GlobalManager CoCoAFoundations;

  GlobalOutput() << ShortDescription << endl;

  trial(RingZ());
  trial(RingQ());
  trial(NewRingTwinFloat(32));
  trial(NewZmod(2));
  trial(NewZmod(1048576)); // ring has zero divisors
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-module1.C,v 1.3 2008/04/21 12:33:45 abbott Exp $
// $Log: ex-module1.C,v $
// Revision 1.3  2008/04/21 12:33:45  abbott
// Made a very minor change.
//
// Revision 1.2  2007/06/21 21:29:47  abbott
// Changed name of RingFloat into RingTwinFloat.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.6  2007/03/03 14:15:45  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.5  2007/02/28 15:15:10  bigatti
// -- added sum and multiplication examples
//
// Revision 1.4  2007/02/26 17:40:34  bigatti
// -- getting ready for unique ring Z: using NewZmod(N), NewRingQ()
//
// Revision 1.3  2007/02/12 16:27:43  bigatti
// -- added strings ShortDescription and LongDescription for indexing
//
// Revision 1.2  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.1  2005/12/16 17:53:01  cocoa
// --- first import
//
