// Copyright (c) 2008  John Abbott
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "Creation of symbols, and some simple operations on them.\n";

const string LongDescription =
  "Symbols are used to give print names to indeterminates.  Their main   \n"
  "use is as an argument to a pseudo-ctor for a PPMonoid or a polynomial \n"
  "ring -- see examples for those types.                                 \n";
//----------------------------------------------------------------------

void program()
{
  GlobalManager CoCoAFoundations;

  GlobalOutput() << ShortDescription << endl;

  // Unindexed symbols:
  symbol x("x");
  symbol alpha("alpha");

  // Symbols with 1 index
  symbol y2("y",2);
  symbol zeta("zeta",0);

  // Symbols with 2 indexes
  symbol amn("a",4,7);
  symbol s("Y_beta", -3,8);

  // Symbols with many indexes -- flexible but cumbersome!
  vector<long> inds;
  inds.push_back(1);
  inds.push_back(2);
  inds.push_back(3);
  symbol triple("X", inds);

  // Querying a symbol to get its head, number of indexes and values of the indexes.
  GlobalOutput() << "Head of " << s << " is " << head(s) << endl;
  GlobalOutput() << "NumIndices of " << triple << " is " << NumIndices(triple) << endl;
  GlobalOutput() << "0-th index of " << y2 << " is " << index(y2, 0) << endl;

  // The convenience functions for making vectors of symbols
  GlobalOutput() << "vector of length 1: " << symbols("omega") << endl;
  GlobalOutput() << "vector of length 2: " << symbols("mu", "nu") << endl;
  GlobalOutput() << "vector of length 11: " << SymbolRange("B", -5,5) << endl;
  alpha = symbol("PP",0,1); zeta = symbol("PP",4,3);
  GlobalOutput() << "vector of length 15: " << SymbolRange(alpha, zeta) << endl;
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-symbol1.C,v 1.2 2008/12/12 16:32:09 abbott Exp $
// $Log: ex-symbol1.C,v $
// Revision 1.2  2008/12/12 16:32:09  abbott
// Improved comments.
//
// Revision 1.1  2008/12/12 11:29:48  abbott
// Fixed a bug in SymbolRange.  Added example and test for symbols.
//
// Revision 1.4  2008/10/07 12:12:54  abbott
// Removed useless commented out #include.
//
// Revision 1.3  2007/05/31 16:06:16  bigatti
// -- removed previous unwanted checked-in version
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.9  2007/03/07 11:51:40  bigatti
// -- improved test alignment
//
// Revision 1.8  2007/03/03 14:15:45  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.7  2007/03/02 17:46:40  bigatti
// -- unique RingZ and RingQ
// -- requires foundations.H ;  foundations blah;  (thik of a better name)
//
// Revision 1.6  2007/03/02 10:47:53  cocoa
// First stage of RingZ modifications -- tests do not compile currently, Anna will fix this.
//
// Revision 1.5  2007/03/01 13:52:59  bigatti
// -- minor: fixed typo
//
// Revision 1.4  2007/02/28 15:15:56  bigatti
// -- minor: removed quotes in description
//
// Revision 1.3  2007/02/12 16:27:43  bigatti
// -- added strings ShortDescription and LongDescription for indexing
//
// Revision 1.2  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.1  2006/03/12 21:28:34  cocoa
// Major check in after many changes
//
