#ifndef CoCoA_GMPAllocator_H
#define CoCoA_GMPAllocator_H

//   Copyright (c)  2005,2006  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/MemPool.H"

// #include <stddef.h> --- already included in MemPool.H
// using std::size_t;

extern "C"
{
  void* CoCoA_GMP_alloc(size_t sz);
  void* CoCoA_GMP_realloc(void* ptr, size_t oldsz, size_t newsz);
  void CoCoA_GMP_free(void* ptr, size_t sz);
}

namespace CoCoA
{

  class GMPAllocator
  {
  public:
    explicit GMPAllocator(std::size_t SliceSize=16); // default slice size is 16 bytes
    ~GMPAllocator();
  private: // disable copy ctor and assignment
    GMPAllocator(const GMPAllocator&);           // NEVER DEFINED -- disable default copy ctor
    GMPAllocator& operator=(const GMPAllocator&);// NEVER DEFINED -- disable assignment
  private:
    mutable MemPool myMemPool;
  };

} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/GMPAllocator.H,v 1.4 2009/10/28 16:49:33 bigatti Exp $
// $Log: GMPAllocator.H,v $
// Revision 1.4  2009/10/28 16:49:33  bigatti
// -- commented out include <cstddef> (including MemPool.H)
//
// Revision 1.3  2008/04/21 12:32:54  abbott
// Corrected size_t into std::size_t in several header files; in some cases,
// replaced size_t with MachineInteger (with consequent changes to impl files).
//
// Revision 1.2  2007/10/30 17:14:12  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.5  2006/12/06 17:11:16  cocoa
// -- removed #include "config.H"
//
// Revision 1.4  2006/10/06 14:04:16  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.3  2006/10/06 10:08:17  cocoa
// Added changelog.
//


#endif
