#ifndef CoCoA_GlobalManager_H
#define CoCoA_GlobalManager_H

//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include <memory>
// using std::auto_ptr

namespace CoCoA
{

  class ring;          // fwd decl -- defined in ring.H
  class FractionField; // fwd decl -- defined in FractionField.H

  // At most one instance of this class may exist at any given time.
  class GlobalManager
  {
  public:
    GlobalManager(bool ResiduesAreSymmByDefault=true);
    ~GlobalManager();
  private: // disable copy ctor and assignment
    GlobalManager(const GlobalManager&);            ///< NEVER DEFINED -- disable copy ctor
    GlobalManager& operator=(const GlobalManager&); ///< NEVER DEFINED -- disable assignment
  private:
    static const GlobalManager* ourGlobalDataPtr; ///< NULL or addr of unique existing GlobalManager
    friend const ring& RingZ(); // defined in RingZ.C
    friend const FractionField& RingQ(); // defined in RingQ.C
    friend bool DefaultResiduesAreSymm(); // used when creating rings of the form Z/(nZ)
  private: // data members
    const bool myResiduesAreSymm; // default value used for creating rings Z/(nZ)
    std::auto_ptr<ring> myRingZPtr;
    std::auto_ptr<FractionField> myRingQPtr;
  };

  // The following names are handy as args to ctor for GlobalManager
  enum { UseNonNegResidues=false, UseSymmResidues=true };
  bool DefaultResiduesAreSymm();

} // end of namespace CoCoA



// RCS header/log in the next few lines.
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/GlobalManager.H,v 1.3 2009/05/14 09:39:29 abbott Exp $
// $Log: GlobalManager.H,v $
// Revision 1.3  2009/05/14 09:39:29  abbott
// Added possibility to specify "symmetric" or "non-negative" residues
// in quotients of ZZ.  Affects printing of elements in quotients of ZZ
// (also changed printing of elements in general quotient rings).
// Consequent changes in several tests.
//
// Revision 1.2  2007/10/30 17:14:12  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.2  2007/03/05 21:33:13  cocoa
// Improved/cleaned GlobalManager; added doc too.
//
// Revision 1.1  2007/03/03 14:02:11  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.1  2007/03/02 16:46:28  cocoa
// New foundations object which calls ctors and dtors of global objects.
//

#endif
