#ifndef CoCoA_headerfile_H
#define CoCoA_headerfile_H

//   Copyright (c)  2008  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.
//   -------------------------------------------------------------------------

#include "CoCoA/config.H"

#include <cstddef>
// using std::size_t;
#include <iosfwd>
//using std::ostream;
#include <vector>
// using std::vector;

namespace CoCoA
{

  class ring; // fwd decl -- defined in ring.H
  class ConstMatrixView; // fwd decl -- defined in matrix.H
  class matrix; // fwd decl -- defined in matrix.H

/**
    expects a matrix with entries in an ordered ring
*/
  bool IsTermOrdering(ConstMatrixView M);
  matrix NewPositiveMat(ConstMatrixView M); ///< returns a matrix with positive entries which defines an equivalent term-ordering
  bool IsPositiveGrading(ConstMatrixView M, std::size_t GradingDim);  ///< are the first GradingDim rows of M a positive grading?


  matrix NewMatMinimize(ConstMatrixView M);///< the (ordering) matrix obtained by removing linearly dependent rows
  matrix NewDenseMatRevLex(std::size_t n);
  matrix NewMatCompleteOrd(ConstMatrixView M);
  matrix NewMatElim(std::size_t NumIndets, const std::vector<std::size_t>& IndetsToElim);
  matrix NewMatElim(ConstMatrixView GradingM, const std::vector<std::size_t>& IndetsToElim, bool IsHomog);

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/MatrixForOrdering.H,v 1.3 2009/09/22 13:35:55 bigatti Exp $
// $Log: MatrixForOrdering.H,v $
// Revision 1.3  2009/09/22 13:35:55  bigatti
// -- following coding conventions in function names Matrix --> Mat
// -- forced all matrices to be over RingZ
//
// Revision 1.2  2008/12/17 12:11:52  abbott
// Changed type from long to MachineInteger in operations which use a machine integer
// in place of a RingElem.  The change is "superficial" but affects many files.
//
// Revision 1.1  2008/04/21 11:23:11  abbott
// Separated functions dealing with matrices and PPOrderings into a new file.
// Added matrix norms, and completed adjoint.
//
//

#endif
