#ifndef CoCoA_MatrixViews_H
#define CoCoA_MatrixViews_H

//   Copyright (c)  2005,2008  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include <cstddef>
//using std::size_t;
#include <vector>
// using std::vector;


namespace CoCoA
{

  class MatrixView;     // forward declaration -- defined in matrix.H
  class ConstMatrixView;// forward declaration -- defined in matrix.H
  class ring;           // forward declaration -- defined in ring.H
  class RingElem;       // forward declaration -- defined in ring.H

  /*-----------------------------------------------------------------*/
  /** \include MatrixViews.txt  */
  /*-----------------------------------------------------------------*/

  MatrixView ZeroMat(const ring& R, std::size_t NumRows, std::size_t NumCols);
  MatrixView IdentityMat(const ring& R, std::size_t dim);

  MatrixView transpose(MatrixView M);
  ConstMatrixView transpose(ConstMatrixView M);

  MatrixView submat(MatrixView M, const std::vector<std::size_t>& rows, const std::vector<std::size_t>& cols);
  ConstMatrixView submat(ConstMatrixView M, const std::vector<std::size_t>& rows, const std::vector<std::size_t>& cols);

  MatrixView ColMat(std::vector<RingElem>& v);
  ConstMatrixView ColMat(const std::vector<RingElem>& v);
  MatrixView RowMat(std::vector<RingElem>& v);
  ConstMatrixView RowMat(const std::vector<RingElem>& v);
  MatrixView DiagMat(std::vector<RingElem>& DiagEntries);
  ConstMatrixView DiagMat(const std::vector<RingElem>& DiagEntries);

  /***************************************************************************/
  // BlockMat -- block matrices  ( A  B )
  //                             ( C  D )
  MatrixView BlockMat(MatrixView A, MatrixView B, MatrixView C, MatrixView D);
  ConstMatrixView BlockMat(ConstMatrixView A, ConstMatrixView B, ConstMatrixView C, ConstMatrixView D);

  MatrixView ConcatVer(MatrixView M1, MatrixView M2);
  ConstMatrixView ConcatVer(ConstMatrixView M1, ConstMatrixView M2);
  MatrixView ConcatHor(MatrixView M1, MatrixView M2);
  ConstMatrixView ConcatHor(ConstMatrixView M1, ConstMatrixView M2);
  MatrixView ConcatDiag(MatrixView M1, MatrixView M2);
  ConstMatrixView ConcatDiag(ConstMatrixView M1, ConstMatrixView M2);
  MatrixView ConcatAntiDiag(MatrixView M1, MatrixView M2);
  ConstMatrixView ConcatAntiDiag(ConstMatrixView M1, ConstMatrixView M2);

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/MatrixViews.H,v 1.2 2009/02/13 15:20:33 bigatti Exp $
// $Log: MatrixViews.H,v $
// Revision 1.2  2009/02/13 15:20:33  bigatti
// -- comment fix: BlockMatrix --> BlockMat
//
// Revision 1.1  2008/04/18 15:35:57  abbott
// (long overdue) Major revision to matrices
//
//

#endif
