#ifndef CoCoA_NumTheoryQQ_H
#define CoCoA_NumTheoryQQ_H

//   Copyright (c)  2010  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.
//   -------------------------------------------------------------------------

#include "CoCoA/QQ.H"

#include <iosfwd>
//using std::ostream;

namespace CoCoA
{

  class ContFracIter
  {
  public:
    explicit ContFracIter(const QQ& Q);
    const ZZ& operator*() const;
    ContFracIter& operator++();
    ContFracIter operator++(int);
    friend bool IsEnded(const ContFracIter& CFIter) { return IsZero(CFIter.myQuot) && IsZero(CFIter.myFrac); }
  private: // data members
    QQ myFrac;
    ZZ myQuot;
  };

  class CFApproximantsIter
  {
  public:
    explicit CFApproximantsIter(const QQ& Q);
    explicit CFApproximantsIter(const ContFracIter& CFIter);
    const QQ& operator*() const { return myApproximant; }
    CFApproximantsIter& operator++();
    CFApproximantsIter operator++(int);
    friend bool IsEnded(const CFApproximantsIter& CFAIter) { return IsEnded(CFAIter.myCFIter); }
  private: // data members
    ContFracIter myCFIter;
    QQ myApproximant;
    ZZ myPrevNum; // cannot use a QQ here because initial value is 1/0
    ZZ myPrevDen;
  };

  std::ostream& operator<<(std::ostream& out, const ContFracIter& CFIter);
  std::ostream& operator<<(std::ostream& out, const CFApproximantsIter& CFAIter);

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/NumTheoryQQ.H,v 1.2 2010/03/22 18:09:33 abbott Exp $
// $Log: NumTheoryQQ.H,v $
// Revision 1.2  2010/03/22 18:09:33  abbott
// Corrected copyright year.
//
// Revision 1.1  2010/03/18 16:35:08  abbott
// Added NumTheoryQQ files for number theory operations involving QQs.
//

#endif
