#ifndef CoCoA_RandomLongStream_H
#define CoCoA_RandomLongStream_H

//   Copyright (c)  2010  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/MachineInteger.H"

#include <cstddef>
//using std::size_t;
#include <iosfwd>
//using std::ostream;
#include "gmp.h"

namespace CoCoA
{

  class RandomLongStream
  {
  public:
    RandomLongStream(const MachineInteger& lwb, const MachineInteger& upb, const MachineInteger& seed = 1);
    RandomLongStream(const RandomLongStream& copy);
    RandomLongStream& operator=(const RandomLongStream& rhs);
    ~RandomLongStream();
    long operator*() const;
    RandomLongStream& operator++();
    RandomLongStream operator++(int);
    std::size_t myIndex() const;
  private:
    void myGenValue(); // fills myValue with a newly generated random value.
  private: // data members
    long myLwb;
    long myUpb;
    unsigned long myRange;
    long myValue;
    std::size_t myCounter;
    unsigned long mySeed;
    gmp_randstate_t myState;
    friend std::ostream& operator<<(std::ostream& out, const RandomLongStream& RLS);
  };

  long sample(RandomLongStream& RLS);
  std::ostream& operator<<(std::ostream& out, const RandomLongStream& RLS);

  //---------------------------------------------------------------------------
  // Inline functions.

  inline long RandomLongStream::operator*() const
  {
    return myValue;
  }


  inline RandomLongStream& RandomLongStream::operator++()
  {
    ++myCounter;
    myGenValue();
    return *this;
  }


  inline long sample(RandomLongStream& RLS)
  {
    return *++RLS; // advance then get value
  }

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/RandomLongStream.H,v 1.1 2010/02/16 10:19:29 abbott Exp $
// $Log: RandomLongStream.H,v $
// Revision 1.1  2010/02/16 10:19:29  abbott
// Added new class RandomLongStream with example and test.
//
//

#endif
