#ifndef CoCoA_GPoly_H
#define CoCoA_GPoly_H

//   Copyright (c)  2005-2010  Massimo Caboara

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/FreeModule.H"
#include "CoCoA/GBEnv.H"
#include "CoCoA/PPWithMask.H"
#include "CoCoA/RingHom.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/SugarDegree.H"
#include "CoCoA/TmpGTypes.H"

#include<list>
// using std::list; // for GPolyList; GPolyPtrList;
#include<vector>
// using std::vector; // for GPolyPtrVector;


namespace CoCoA
{

  class GPair;     // fwd decl
  class Reductors; // fwd decl


enum ClearType {clear};

class GPoly;


typedef std::list<GPoly> GPolyList;


/*-- class GPoly --------------------------------------------------*/
/**

This class is only a wrapper around the polys.
Some other data useful for GBasis computation is added.

*/
/*-----------------------------------------------------------------*/

  class GPoly
  {
  public:
    enum ReductionMarker { Full, OnlyTail };

  public:
    GPoly(ConstRefRingElem,
          const GRingInfo&,
	  unsigned int age=0);
    GPoly(ConstRefRingElem,
          ConstRefPPMonoidElem theLPP,//          const PPWithMask&,  ANNA
	  ConstRefRingElem,
	  const GRingInfo&,
	  unsigned int age=0);///< this ctor takes LPP and LC from input for Dynamic Algorithm
    GPoly(RefRingElem,
          const GRingInfo&,
	  const ClearType,
	  unsigned int age=0);  ///< this ctor resets RefRingElem
    GPoly(RefRingElem,
          ConstRefPPMonoidElem theLPP,//          const PPWithMask&,  ANNA
	  ConstRefRingElem,
	  const GRingInfo&,
	  const ClearType,
	  unsigned int age=0);///< this ctor resets RefRingElem and takes LPP and LC from input for Dynamic Algorithm
     explicit  GPoly(const GRingInfo&);
    GPoly(const GPoly&);
    ~GPoly(){};
    inline bool operator<(const GPoly& f) const {return myAge<f.myAge;};
    GPoly& operator=(const GPoly& rhs);
    void AssignClear(GPoly&);
    bool operator==(const GPoly&)const;
    void myAppendClear(RefRingElem);  ///< Dangerous
    void myAppendClear(GPoly&); ///< Dangerous
    const GRingInfo& myGRingInfo()const{return myGRingInfoValue;};
    CoeffEncoding::type myCoeffRingType()const{return myGRingInfoValue.myCoeffRingType();};
    bool operator!=(const GPoly&)const;
    inline bool IsActive() const {return myIsActive;}
    inline void Deactivate(){myIsActive=false;};
    void myAssignSPoly(const GPair&, const unsigned int age);
    ConstRefPPMonoidElem myLPPForDiv() const { return PP(myLPPForDivwMask);}
    friend unsigned int Component(const GPoly& g){return g.myComponent;};
    void MultiplyByPP(ConstRefPPMonoidElem);
    friend ConstRefRingElem LC(const GPoly&);
    friend bool  IsZero(const GPoly&);
    friend std::ostream& operator<<(std::ostream&, const GPoly&);
    //    friend ConstRefPPMonoidElem LPP(const GPoly& f);
    //    friend ConstRefPPMonoidElem LPPForDiv(const GPoly& f);
    friend ConstRefPPMonoidElem LPPForOrd(const GPoly& f);
    friend const PPWithMask& LPPForDivwMask(const GPoly& f);

    inline friend unsigned int len(const GPoly& gp) {return gp.myLen;};
    inline friend const degree& wdeg(const GPoly& gp) {return gp.myWDeg;};
    inline friend const SugarDegree& sugar(const GPoly& gp) {return gp.mySugar;};
    inline friend ConstRefRingElem poly(const GPoly& gp) {return gp.myPolyValue;};
    inline friend unsigned int Age(const GPoly& gp) {return gp.myAge;};
    inline friend unsigned int MakeKey(const GPoly&);
    //  friend unsigned int nterms(const GPoly& gp);

    friend const SparsePolyRing& owner(const GPoly& gp);
    friend const PPMonoid& PPM(const GPoly& gp);
    friend const ring& CoeffRing(const GPoly& gp);

    /**@name Pseudo-saturation functions */
    //@{
    // real body should be in SparsePolyRing
    void smart_dehomog_DRL(std::size_t DHVar);
    void smart_dehomog(std::size_t DHVar);
    friend std::size_t max_common_wdeg(GPoly&,std::size_t Var);
    friend void GPolyList2PolyListClear(GPolyList&,PolyList&);
    friend void GPolyList2PolyList(const GPolyList&,PolyList&);

    //@}

    /**@name Public reduction functions */
    //@{
    //  void reduce(const std::vector<GPoly*>& G); // is this useful??
    void myReduce(const Reductors& theReductors);
    void myReduceTail(const GPoly& g);
    void myReduceTail(const Reductors& theReductors);
    //@}

    /**@name member fields of GPoly  */
    //@{
  private:
    friend class GReductor;

    PPWithMask myLPPForDivwMask; ///< precomputed LPP of the polynomial, stored with DivMask, for divisibility tests
    PPMonoidElem myLPPForOrd; ///< precomputed LPP of the polynomial, for sorting
    RingElem myLCValue;// use iterator, can be big
    RingElem myPolyValue; ///< the polynomial proper
    //const SparsePolyRing& myPolyRing; ///< owner(myPoly) as SparsePolyRing
    const GRingInfo& myGRingInfoValue;
    bool myIsActive; ///< mutable, the rest const ... a cosa serve?
    unsigned int myLen; ///< precomputed length of the polynomial: for geobuckets
    unsigned int myComponent; /// <the module component of the LPP
    degree myWDeg; ///< precomputed degree of the polynomial
    SugarDegree mySugar; ///< "sugar" of the polynomial: a sort of degree
    unsigned int myAge; ///< age of the polynomial: for ordering reducers
    //@}

    //----------  GPoly private member functions  ----------
    void myPolySetSPoly(const GPoly& f, const GPoly& g); ///< set only s-polynomial in myPoly
    inline ConstRefRingElem myPoly() const {return myPolyValue;}; // DANGER DANGER DANGER: the LPP, LC, wdeg, StdDeg, sugar, wsugar of this polynomial DIFFER form the corresponding fields for the
                                                                   // GPoly if we are using the Dynamic Algorithm. NEVER use it if you can avoid it

    void myUpdateLenLPPLCDegComp();
    friend void monic(std::list<GPoly>&);
    //    friend void ChooseReduction(char r='g');   ///< 'g' = geobucket; 'b' = basic;

    /**@name Private functions defined in reduce.C*/
    //@{
    friend GPoly* FindReducer(const GPoly&, const Reductors& theReductors);
    //@}

    // these functions need to access myPoly(). See if they can be pared down/put here
    // they are located in GReductor.C
    friend void DeEmbedPoly(ModuleElem&,const GPoly&,std::size_t);
    friend void IntEmbedGPolyList(GPolyList&, GPolyList&);
    friend void DeEmbedPoly(RefRingElem, const GPoly&);
  };//class GPoly

  // These are only for internal use
  typedef std::list<GPoly*> GPolyPtrList;
  typedef std::vector<GPoly*> GPolyPtrVector;


  GPoly* FindReducer(const PPWithMask& pm,
                     const unsigned int component,
                     const Reductors& theReductors);
  GPoly* FindReducer(const GPoly&, const Reductors& theReductors);

  //  std::vector<GPoly*>::const_iterator FindReducer(const geobucket&, const std::vector<GPoly*>& G);


   void monic(PolyList&);
   void monic(GPolyList&);
   void power(RefRingElem theResult,
               const std::vector<RingElem>& theV,
               const degree& the_d);

  // -------------------- inline functions -------------------- //

  inline ConstRefPPMonoidElem LPPForDiv(const GPoly& f)
  {
    return f.myLPPForDiv();
  }

  inline ConstRefPPMonoidElem LPPForOrd(const GPoly& f)
  {
    return f.myLPPForOrd;
  }

  inline ConstRefRingElem LC(const GPoly& f)
  {
    return f.myLCValue;
  }

  inline const PPWithMask& LPPForDivwMask(const GPoly& f)
  {
    return f.myLPPForDivwMask;
  }


// inline unsigned int nterms(const GPoly& gp)
// {
//   //  const SparsePolyRing& P = dynamic_cast<const SparsePolyRing&>(owner(gp.myPoly));
//   //  return AsSparsePolyRing(owner(gp.myPoly))->NumTerms(raw(gp.myPoly));
//   return AsSparsePolyRing(owner(gp))->NumTerms(raw(gp.myPoly));
// }


  inline const SparsePolyRing& owner(const GPoly& gp)
  {
    return gp.myGRingInfoValue.myNewSPR();
  }


  inline const PPMonoid& PPM(const GPoly& gp)
  {
    return PPM(owner(gp));
  }


  inline const ring& CoeffRing(const GPoly& gp)
  {
    return CoeffRing(owner(gp));
  }


  // -------------------- ReductorData -------------------- //
  class ReductorData

  {
  public:
    ReductorData(GPoly* p, unsigned int p_component, unsigned int count=0);
    ReductorData(const ReductorData&);
    ~ReductorData(){};
    bool operator<(const ReductorData& RD) const {return myKey<RD.myKey;};
    friend std::ostream& operator<<(std::ostream&, const ReductorData&);
    //inline friend unsigned int Deg(const ReductorData& RD) {return Deg(*myGPoly);};
    friend const SparsePolyRing& owner(const Reductors&);
    friend const PPMonoid& PPM(const Reductors&);
    friend unsigned int Component(const ReductorData& RD){return RD.myComponent;};
    bool IamNotToBeUsed()const{return myIamNotToBeUsedValue;};
    void mySetIamNotToBeUsed(const bool b){myIamNotToBeUsedValue=b;};
    GPoly* myGetGPolyPtr()const{return myGPolyPtr;};



  private:
    friend class Reductors;
    GPoly* myGPolyPtr;
    unsigned int myKey; ///< integer "key" used to sort the list of reducers
    unsigned int myComponent;
    mutable PPWithMask myLPPForDivwMask;
    mutable unsigned int myCount; ///< how many times it has been used
    mutable bool IamBorelUpdated; ///< true if its degree is the current degree (Homogeneous algorithm)
    bool myIamNotToBeUsedValue;///<true if I don't want to use this element for reduction, i.e. interreducing

    friend GPoly* FindReducer(const PPWithMask& pp,const unsigned int pp_component,
                                const Reductors& theReductors);
    bool myBorelUpdate(ConstRefPPMonoidElem pp, const Reductors& theReductors);
  };


  class Reductors{

  public:
    enum UseBorelMarker { UseBorel, DontUseBorel };

    Reductors(const GRingInfo& P);
    Reductors(const GRingInfo& P, const UseBorelMarker);
    ~Reductors(){};
    void Insert(GPoly*, const unsigned int count=0);
    UseBorelMarker IhaveBorelReductors()const
    {if(IhaveBorelReductorsFlag) return UseBorel; else return DontUseBorel;};

    unsigned int size() const {return myReductors.size();};
    void myStampaReductors(std::ostream&) const;
    const GRingInfo& myGRingInfo()const{return myGRingInfoValue;};
    //    GPoly* LastGPolyPtr(){return myReductors.back().myGPolyPtr;};//copia ptr  // never used
    //    PPMonoidElem LastGPolyLPP(){return myReductors.back().myLPP;};//copia PP  // never used
    void interreduce(const GPoly&);
    void OrderedInterreduce(const GPoly&);
    void SuperInterreduce(const GPoly&);

    friend const SparsePolyRing& owner(const Reductors&);
    friend const PPMonoid& PPM(const Reductors&);
    friend void interreduce(Reductors&, GPoly);
    friend void OrderedInterreduce(Reductors&, GPoly);
    friend void reduce(const Reductors&);
    friend void ReduceTail(const Reductors&);
    friend GPoly* FindReducer(const PPWithMask& pp,
                              const unsigned int pp_component,
                              const Reductors& theReductors);
    // HERE TO BE NOTICED friend const SparsePolyRing& owner(const Reductors&);
    // HERE TO BE NOTICED friend const PPMonoid& PPM(const Reductors&);
			
    // This function takes params that are part of GReductor class.
    // In the future, a new interface class will be built that have this funct. as a member
    // This new class would insulate Reductors from end user, which will see only
    // this new interface class which would rule all reductors functions.
    // data in a GReductor will be pruned accordingly.
    void myBorelReductorsUpdateInNextDegree();
    void clear();
    std::vector<ReductorData>::iterator find(GPoly*);// Find the (unique) Reductor data which ptr is equal to GPolyPtr. Returns end() if nothing is found.

    //    friend void swap(Reductors&,Reductors&); // ANNA: no longer needed?

    //@name member fields for Reductors
    //@{
  private:
    /// the poly used for reductions NB polys are ptrs to myPolys
    std::vector<ReductorData> myReductors;
    mutable std::vector<ReductorData> myBorelReductors;
    GPolyList myBorelGPolys;
    bool IhaveBorelReductorsFlag;
    const GRingInfo& myGRingInfoValue;
    //@}

  };// End class Reductors


  SparsePolyRing owner(const PolyList& thePL);
  SparsePolyRing owner(const std::vector<RingElem>& thePV);
  SparsePolyRing owner(const GPolyList& theGPL);

  void PolyList2PolyVectorClear(PolyList&,std::vector<RingElem>&);
  void PolyVector2PolyListClear(std::vector<RingElem>&,PolyList&);

}// end namespace cocoa



// RCS header/log on the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/TmpGPoly.H,v 1.12 2010/03/23 14:43:07 bigatti Exp $
// $Log: TmpGPoly.H,v $
// Revision 1.12  2010/03/23 14:43:07  bigatti
// -- class GRingInfo estracted from TmpGPoly
//
// Revision 1.11  2009/10/27 17:15:14  bigatti
// -- fixed: using sugar(g)->myWSugar() insted of wsugar(g)
//
// Revision 1.10  2009/04/27 13:12:44  bigatti
// -- added some temporary functions (DegLess...)
//
// Revision 1.9  2009/04/27 12:23:36  bigatti
// -- added SaturatingAlgNoDRL  to enum
//
// Revision 1.8  2008/09/19 13:33:42  bigatti
// -- added: Sat algorithm (M.Caboara)
//
// Revision 1.7  2008/09/16 15:03:42  bigatti
// -- added LPPForDiv
// -- changed LPP into LPPForOrd
//
// Revision 1.6  2008/07/09 16:08:04  abbott
// Cosmetic tidying.
//
// Revision 1.5  2008/04/21 12:32:54  abbott
// Corrected size_t into std::size_t in several header files; in some cases,
// replaced size_t with MachineInteger (with consequent changes to impl files).
//
// Revision 1.4  2007/12/05 11:06:24  bigatti
// -- changed "size_t StdDeg/myStdDeg(f)" into "long"  (and related functions)
// -- changed "log/myLog(f, i)" into "MaxExponent/myMaxExponent(f, i)"
// -- fixed bug in "IsOne(ideal)" in SparsePolyRing.C
//
// Revision 1.3  2007/12/04 14:27:07  bigatti
// -- changed "log(pp, i)" into "exponent(pp, i)"
//
// Revision 1.2  2007/10/30 17:14:11  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1  2007/03/09 18:56:56  bigatti
// -- added Tmp prefix to Groebner related files
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.13  2007/03/08 18:42:06  cocoa
// Cleaned up whitespace.
//
// Revision 1.12  2007/03/07 22:24:03  bigatti
// -- reintroduced TmpGTypes.H (waiting for a better solution)
//
// Revision 1.11  2007/03/07 17:04:31  cocoa
// -- several changes by M.Caboara: more operations on ideals,
//    exception cleaner, coding conventions, WSugar, dynamic
//
// Revision 1.10  2006/12/22 11:37:40  cocoa
// Removed operator<< for PolyList and GPolyList.  A few consequential changes.
//
// Revision 1.9  2006/12/06 17:11:16  cocoa
// -- removed #include "config.H"
//
// Revision 1.8  2006/12/04 13:55:54  cocoa
// -- added: sugar for GradingDim > 0  (called wsugar)
//
// Revision 1.7  2006/11/24 17:17:58  cocoa
// -- reorganized includes of header files
//
// Revision 1.6  2006/11/20 14:57:17  cocoa
// -- added: (standard) sugar for modules
// -- fixed: non-homogeneous sysygies
// -- minor fixes     [M.Caboara]
//
// Revision 1.5  2006/11/02 13:25:44  cocoa
// Simplification of header files: the OpenMath classes have been renamed.
// Many minor consequential changes.
//
// Revision 1.4  2006/10/06 16:32:06  cocoa
// -- changed: GPoly::SPoly --> GPoly::myAssignSPoly
// -- changed: Len(const GPoly&) --> len(const GPoly&)
// -- added:   poly(const GPoly&)
// -- added:   GPoly::myUpdateLenLPPDegComp()
// -- in reduce.C added functions for computing sugar during reduction
//
// Revision 1.3  2006/10/06 14:04:16  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.2  2006/08/17 09:28:56  cocoa
// -- added: sugar
// -- added: controls on homogeneity
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files

#endif
