#ifndef CoCoA_GReductor_H
#define CoCoA_GReductor_H

//   Copyright (c)  2005  Massimo Caboara

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/GBEnv.H"
#include "CoCoA/TmpGPoly.H"
#include "CoCoA/TmpGPair.H"
#include "CoCoA/TmpGRStats.H"
//#include "CoCoA/CoCoA4io.H"


namespace CoCoA
{

  //  void ReadInt(std::istream&,int&,SkipTagType);

  class FreeModule;     // forward declaration -- defined in FreeModule.H
  class SparsePolyRing; // forward declaration -- defined in SparsePolyRing.H

  SparsePolyRing NewSparsePolyRing(const ring& CoeffRing,
                                   const std::vector<symbol>& IndetNames,
                                   const PPOrdering& ord);


  class GReductor
  {
  public:

    enum UseCoprimeCritMarker   { UseCoprimeCrit, DontUseCoprimeCrit };
    enum UseGMCritMarker        { UseGMCrit, DontUseGMCrit };
    enum UseBackCritMarker      { UseBackCrit, DontUseBackCrit };
    enum UseDivCritMarker       { UseDivCrit, DontUseDivCrit };
    enum UseDynamicAlgMarker    { UseDynamicAlg, DontUseDynamicAlg };
    enum BuchbergerOpTypeMarker { HomogeneousAlg, SaturatingAlg, AffineAlg };


    GReductor(const GRingInfo&,
              const PolyList&,
              const int StatLevel=-1,
              const Reductors::UseBorelMarker=Reductors::DontUseBorel,
	      const UseDynamicAlgMarker IsDynamic=DontUseDynamicAlg,
              const UseCoprimeCritMarker CopCriterion=UseCoprimeCrit,
              const UseGMCritMarker GMCriteria=UseGMCrit,
              const UseBackCritMarker BCriterion=UseBackCrit,
              const UseDivCritMarker DivCriterion=UseDivCrit,
	      const BuchbergerOpTypeMarker theBuchbergerOpType=HomogeneousAlg);
    GReductor(const GRingInfo&,
              const GPolyList&,
              const int StatLevel=-1,
              const Reductors::UseBorelMarker=Reductors::DontUseBorel,
	      const UseDynamicAlgMarker IsDynamic=DontUseDynamicAlg,
              const UseCoprimeCritMarker CopCriterion=UseCoprimeCrit,
              const UseGMCritMarker GMCriteria=UseGMCrit,
              const UseBackCritMarker BCriterion=UseBackCrit,
              const UseDivCritMarker DivCriterion=UseDivCrit,
	      const BuchbergerOpTypeMarker theBuchbergerOpType=HomogeneousAlg);
    GReductor(const GReductor&);// copy ctor not working, to be fixed
    ~GReductor(){};
    void myCtorAux(const BuchbergerOpTypeMarker theBuchbergerOpType,
                   const UseDynamicAlgMarker IsDynamic,
                   const UseCoprimeCritMarker CopCriterion,
                   const UseGMCritMarker GMCriteria,
                   const UseBackCritMarker BCriterion,
                   const UseDivCritMarker DivCriterion);
    GReductor(const GRingInfo&,
              GPolyList&,
              const ClearType,
              const int StatLevel=-1,
              const Reductors::UseBorelMarker=Reductors::DontUseBorel,
	      const UseDynamicAlgMarker IsDynamic=DontUseDynamicAlg,
              const UseCoprimeCritMarker CopCriterion=UseCoprimeCrit,
              const UseGMCritMarker GMCriteria=UseGMCrit,
              const UseBackCritMarker BCriterion=UseBackCrit,
              const UseDivCritMarker DivCriterion=UseDivCrit,
	      const BuchbergerOpTypeMarker theBuchbergerOpType=HomogeneousAlg);
    unsigned int myReductorsLen()const {return myTrueReductors.size();};
    unsigned int myGBasisLen()const {return myGB.size();};
    unsigned int myPairsLen() const{return myPairs.size();};
    bool myPreparationDone() const{return myPrepared;};
    bool IsDynamic() const{return IsDynamicAlgorithm;};
    BuchbergerOpTypeMarker myGetBuchbergerOpType() const{return myBuchbergerOpType;};
    void mySetBuchbergerOpType(const BuchbergerOpTypeMarker theBuchbergerOpType) 
        {myBuchbergerOpType=theBuchbergerOpType;};
    bool IsCopCriterionActive() const{return myCopCriterion;};
    bool WrongLPPFound() const{return myWrongLPPFoundValue;};
    void myGBasis(PolyList&);// GB output
    void myGBasis(GPolyList&);// GB output in GPoly form
    void myGBasisClear(GPolyList&); ///< GB output in GPoly form
    void myGBasis(VectorList&);
    void myDoAFFGBasis(); // I am working on sugar selection strategies
    void myDoGBasis(); ///< flag may be set for using Borel reductors
    void myDoGBasisTEST(); ///< flag may be set for using Borel reductors
    void myReduceCurrentSPoly();
    void myPrepareGBasis(); ///< flag may be set for using Borel reductors
    void myFinalizeGBasis();//Last operations (stats at the moment) immediately before shutting down a GB computation
    void myDoGBasis(const int ReductionNumber); // Performs ReductionNumber reductions, -1 means unlimited
    void myReduceUntilNonZeroRedSP(); // Reduces until an SPoly does not reduces to zero. Stop immediately when that happens (no updates)
     // DYNAMIC ALG: reduces until a non zero SPoly has a LPP with is different from the best possibile LPP w.r.t. HPoly.
     // In that case, settles the WrongLPP filed in the reductor.
    void myReduceUntilWrongLPPFound(RefPPMonoidElem,
                                    std::vector<RingElem>& );
    GPoly GetSPoly()const{return mySPoly;};// Reads the SPoly
    void GetCandidateGBasis(PolyList&)const;// the polys computed up to now and the non processed generators
    unsigned int GetAge()const{return myAge;};
    unsigned int GetNReductions()const{return myStat.myNReductions;};
    void SetSPoly(GPoly& p){mySPoly=p;};// Sets the SPoly
    void myDoSATGBasis(); ///< dehomog algorithm
    void _myDoSATGBasis(); ///< dehomog algorithm
    void myDoSATMixGBasis(); ///< dehomog mix algorithm
    void myStampaPPGB(std::ostream&)const; ///< print ?
    void myStampaGB(std::ostream&)const; ///< print ?
    void myStampaPairs(std::ostream&)const; ///< print ?
    void myStampaReductors(std::ostream&)const; ///< print ?
    void myStampaStats(std::ostream& out) const{myStat.myStampa(out);}; ///< print ?
    friend std::ostream& operator<<(std::ostream& out, const GReductor& GR);
    const SparsePolyRing& myPRing()const{return myGRingInfoValue.myNewSPR();};
    const GRingInfo& myGRingInfo()const{return myGRingInfoValue;};
    void Rebuild(const PolyList&);///< rebuild the GReductor initliazinig it with the PL
    void myUpdateBasisOnly();///<Updates the Basis only.
    void myCreatePairs();// this should be const, but this requires
                                   //the GPair ctor to be const, and I have to think about that
    void myCreateSpecialPairs(GPolyList&);
    void myDoGBasisByBatch();//Tmp, DYNAMIC
    void myPrepareGBasisPairsExcluded();//Tmp, DYNAMIC
    void myBuildNewPairsAll(GPairList&);//Tmp, DYNAMIC


  public:
    static int ourDefaultStatLevel; ///< default verbosity level for statistics
  private:
    const GRingInfo myGRingInfoValue;
    GPairList myPairs;// here the polys are ptrs to myPolys
    Reductors myTrueReductors;// the true reductors.
    GPolyPtrList myGB;// the candidate Gbasis - NB polys are ptrs to myPolys
    GPolyList myPolys;// the REAL Polys, the other are ptrs
    GPoly mySPoly;
    degree myOldDeg;// used for flow control and stats. The degree it refers to is the degree of the pair
    degree myCurrentPairDeg;// used for flow control and stats. The degree itrefers is that of the pair
    Stats myStat;// the statistics
    bool myPrepared; // Default false. True after a myPrepareGBasis has been performed
    unsigned int myAge;
    bool myCopCriterion;
    bool IsDynamicAlgorithm;
    bool myWrongLPPFoundValue; // DYNAMIC ALGORITHM
    bool myGMCriteria;
    bool myBCriterion;
    bool myDivCriterion; // This allows the elimination of poly (in the candidate GB) if its LPP is divided by the LPP of the new poly
                         // true always except for ring weyl
    BuchbergerOpTypeMarker myBuchbergerOpType; // Type of operation performed on the reductor

    unsigned int myGMInsert(GPairList&,GPair);
    void         myBuildNewPairs(GPairList&);
    void         myUpdateBasisAndPairs();
    void         myApplyBCriterion();

// These two should go in GPoly (interface) with the
// real body in PolyRing
//void smart_dehomog(GPoly&,std::size_t);
//void smart_dehomog_DRL(GPoly&,std::size_t);

  };// End class GReductor

   void monic(PolyList&);

   const GRingInfo& GetGRI(const GPolyList& theGPL);
   FreeModule owner(const VectorList& theVL);

    enum ModOrdTypeForcing {NoForcing,PosWDegTO,WDegTOPos,WDegPosTO};
    SparsePolyRing MakeNewPRingFromModule(const FreeModule& FM);
    SparsePolyRing MakeNewPRingFromModule(const FreeModule& FM,ModOrdTypeForcing MOType);
    SparsePolyRing MakeNewPRingForSimpleEmbedding(const SparsePolyRing& theOldP);
    SparsePolyRing MakeNewPRingForSimpleEmbedding(const SparsePolyRing& theOldP,ModOrdTypeForcing MOType);
    FreeModule MakeNewFreeModuleForSyz(const GPolyList& GPL);
    FreeModule MakeNewFreeModuleForSyz(const VectorList& VL);
    FreeModule MakeNewFreeModuleForSyz(const PolyList& PL);
    SparsePolyRing MakeElimRingFromOld(const SparsePolyRing& theOldP,
                                       const std::vector<long>& theElimVars,
				       const bool IsHomog);
    SparsePolyRing MakeNewPRingFromModulePosFirst(const FreeModule& FM,
					          bool HomogInput);

    SparsePolyRing MakeNewPRingForSimpleEmbeddingPosFirst(const SparsePolyRing& OldP,
                                                          bool HomogInput);
    // Embed p in the CompIndex component, grading given by the embedding
    GPoly EmbedPoly(ConstRefRingElem p,
    		    const GRingInfo& theGRI,
		    const std::size_t CompIndex);
    // Embed p in the CompIndex component, grading given by the_d
    GPoly EmbedPoly(ConstRefRingElem p,
    		    const GRingInfo& theGRI,
                    const degree& the_d,
		    const std::size_t CompIndex);

   // Embed v
    GPoly EmbedVector(const ModuleElem& v,
    		      const GRingInfo& theGRI);

   // Embed v starting from the StartingFromCompIndex component
    GPoly EmbedVector(const ModuleElem& v,
    		      const GRingInfo& theGRI,
		      const std::size_t StartingFromCompIndex);

   // Embed theVL
    GPolyList EmbedVectorList(const VectorList& theVL,
                              const GRingInfo& theGRI);
    // Embed theVL starting from the StartingFromCompIndex component
    GPolyList EmbedVectorList(const VectorList& theVL,
                              const GRingInfo& theGRI,
			      const std::size_t StartingFromCompIndex);

    // Just transform the PolyList in a GPolyList
    GPolyList EmbedPolyList(PolyList& thePL,
                            const GRingInfo& theGRI);


    GPolyList EmbedPolyList(const PolyList& thePL,
                            const GRingInfo& theGRI,
                            const degree& the_d,
			    const std::size_t CompIndex);

    // Embed with the shifts Shifts
    GPolyList EmbedPolyList(const PolyList& InputPolyList,
                            const GRingInfo& theGRI,
			    const std::size_t CompIndex);
			
    // The embedding used in syzygy computations
    GPolyList SyzEmbedVectorList(const VectorList& InputVectorList,
                                 const GRingInfo& theGRI);
  // The embedding used in syzygy computations
    GPolyList SyzEmbedPolyList(const PolyList& InputPolyList,
                                 const GRingInfo& theGRI);
				
    // The embedding used in intersection computations
    GPolyList IntEmbedVectorLists(const VectorList& VL1,
                                  const VectorList& VL2,
                                  const GRingInfo& theGRI);
				
    // The embedding used in intersection computations
    GPolyList IntEmbedPolyLists(const PolyList& PL1,
                                const PolyList& PL2,
                                const GRingInfo& theGRI);
 			
  // The special poly embedding used in colon computations
    GPolyList ColonEmbedVectorLists(const VectorList& VL1,
                                    const VectorList& VL2,
                                    const GRingInfo& theGRI);
				
    GPolyList ColonEmbedPolyLists(const PolyList& PL1,
                                  const PolyList& PL2,
                                  const GRingInfo& theGRI);

    void SyzEmbedGPolyList(GPolyList& theGPL);

    void IntEmbedGPolyList(GPolyList& theGPL1,
                            GPolyList& theGPL2);

    void ColonEmbedGPolyList(GPolyList& theGPL,
                              GPoly& the_gp);

    ModuleElem DeEmbedPoly(ConstRefRingElem p,											
			   const GRingInfo& theGRI,
                           const std::size_t ComponentsLimit); // the component in p that goes to the 0 component of the output vector v. Lesser components of p go to higher component of v
		
    ModuleElem DeEmbedPoly(ConstRefRingElem p,
                           const GRingInfo& theGRI);

    VectorList DeEmbedPolyList(const PolyList& PL,
                               const GRingInfo& theGRI);

    VectorList DeEmbedPolyList(const PolyList& PL,
                               const GRingInfo& theGRI,
			       const std::size_t ComponentsLimit); // Poly whose LPP has last var degree bigger than this number disappear on DeEmbedding



    void DeEmbedPoly(ModuleElem& theOutputP,
                      const GPoly& the_gp,
                      const std::size_t ComponentsLimit); // the component in p that goes to the 0 component of the output vector v. Lesser components of p go to higher component of v
		
    void DeEmbedPoly(ModuleElem& theOutputP,
                      GPoly& the_gp);

    void DeEmbedPolyList(VectorList& theOutputVL,
                          GPolyList& theGPL);

    void DeEmbedPolyList(VectorList& theOutputVL,
                          const GPolyList& theGPL,
			  const std::size_t ComponentsLimit); // Poly whose LPP has last var degree bigger than this number disappear on DeEmbedding

    void DeEmbedPoly(RefRingElem theOutputP,
                      GPoly& the_gp); // Poly whose LPP has last var degree bigger than this number disappear on DeEmbedding

    void DeEmbedPolyList(PolyList& theOutputPL,
                          GPolyList& theGPL,
			  const std::size_t ComponentsLimit); // Poly whose LPP has last var degree bigger than this number disappear on DeEmbedding

    // input are embedded polys, output true (OldP) Polys
    // this is done directly and not passing through a VectorList to avoid copying
    RingElem DeEmbedPolyToP(ConstRefRingElem the_p,
                            const GRingInfo& theGRI);

    PolyList DeEmbedPolyListToPL(const PolyList& PL,
                                 const GRingInfo& theGRI,
			         const std::size_t ComponentsLimit); // Poly whose LPP has last var degree bigger than this number disappear on DeEmbedding


    void PolyList2GPolyList(const PolyList&,GPolyList&,const GRingInfo&);
    void GPolyList2PolyList(const GPolyList&,PolyList&);
    void PolyList2GPolyListClear(PolyList&,GPolyList&,const GRingInfo&);
    void GPolyList2PolyListClear(GPolyList&,PolyList&);
    void homogenized(RefRingElem the_hp,
                     ConstRefRingElem the_p,
                     const std::vector<RingElem>& the_Y); ///< hp = hom(p,y) in the ring with the y's
    void homogenized(ModuleElem& the_hv,
                     const ModuleElem& the_v,
                     const GRingInfo& theGRI);
    std::vector<long> PolyList2IndexList(const PolyList&);
    PPMonoidElem IndexList2PPMonoidElem(const PPMonoid&,
                                        const std::vector<long>&);
    std::vector<long> PPMonoidElem2IndexList(ConstRefPPMonoidElem);
    std::size_t FirstNonZeroPos(const ModuleElem&);
    bool IsHomog(const PolyList&);
    bool IsHomog(const VectorList&);
    std::vector<degree> DegStructure(ConstRefRingElem);
    std::vector<std::vector<degree> > DegStructure(const ModuleElem&);
    std::vector<std::vector<degree> > DegStructure(const PolyList&);
    std::vector<std::vector<std::vector<degree> > > DegStructure(const VectorList&);
    PolyList MakePolyList(ConstRefRingElem);
    VectorList MakeVectorList(const ModuleElem&);
    RingElem CoeffCommonDenominator(ConstRefRingElem f);
    PolyList WithoutDenominators(const PolyList& PL, SparsePolyRing Rx);
    PolyList WithDenominator1Hom(const PolyList& PL, SparsePolyRing P);


}// end namespace cocoa



// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/TmpGReductor.H,v 1.11 2010/03/23 14:43:07 bigatti Exp $
// $Log: TmpGReductor.H,v $
// Revision 1.11  2010/03/23 14:43:07  bigatti
// -- class GRingInfo estracted from TmpGPoly
//
// Revision 1.10  2009/04/27 13:16:13  bigatti
// -- added BuchbergerOpTypeMarker
//
// Revision 1.9  2009/03/20 17:37:18  bigatti
// -- added IsCopCriterionActive
//
// Revision 1.8  2009/01/30 13:41:50  bigatti
// -- enum instead of bool arguments
//
// Revision 1.7  2008/09/19 14:08:16  bigatti
// -- modified GRStats (M.Caboara)
//
// Revision 1.6  2008/07/09 16:08:40  abbott
// Added missing ampersand (for C++ const reference).
//
// Revision 1.5  2008/04/21 12:32:54  abbott
// Corrected size_t into std::size_t in several header files; in some cases,
// replaced size_t with MachineInteger (with consequent changes to impl files).
//
// Revision 1.4  2008/03/12 16:35:18  bigatti
// -- changed: IsHomogeneous --> IsHomog
// -- changed: ERR:ZeroPoly --> ERR::ZeroRingElem
//
// Revision 1.3  2007/11/09 10:45:52  bigatti
// -- [caboara] preparation for self-saturating algorithm
//
// Revision 1.2  2007/10/30 17:14:11  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1  2007/03/09 18:56:56  bigatti
// -- added Tmp prefix to Groebner related files
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.13  2007/03/08 18:42:06  cocoa
// Cleaned up whitespace.
//
// Revision 1.12  2007/03/07 22:24:03  bigatti
// -- reintroduced TmpGTypes.H (waiting for a better solution)
//
// Revision 1.11  2007/03/07 17:04:31  cocoa
// -- several changes by M.Caboara: more operations on ideals,
//    exception cleaner, coding conventions, WSugar, dynamic
//
// Revision 1.10  2006/12/06 17:11:16  cocoa
// -- removed #include "config.H"
//
// Revision 1.9  2006/11/24 17:20:07  cocoa
// -- reorganized includes of header files
//
// Revision 1.8  2006/11/22 15:33:39  cocoa
// -- changed handling of timings and number of reductions (M.Caboara)
//
// Revision 1.7  2006/11/20 14:57:17  cocoa
// -- added: (standard) sugar for modules
// -- fixed: non-homogeneous sysygies
// -- minor fixes     [M.Caboara]
//
// Revision 1.6  2006/10/06 16:46:17  cocoa
// -- syzygies for non-homogenous polynomials (Max)
// -- wip: evolution of Groebner Framework (Max)
//
// Revision 1.5  2006/10/06 14:04:16  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.4  2006/08/17 09:32:26  cocoa
// -- added: flags for homogeneous input
//
// Revision 1.3  2006/08/07 21:23:25  cocoa
// Removed almost all publicly visible references to SmallExponent_t;
// changed to long in all PPMonoid functions and SparsePolyRing functions.
// DivMask remains to sorted out.
//
// Revision 1.2  2006/07/19 07:13:00  cocoa
// -- moved "IsHomogeneous" to SparsePolyRing/FreeModule
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.16  2006/05/16 09:01:26  cocoa
// -- added "const" to arg of homogenized
// -- changed ordering matrix for modules (compatibility with GradedFreeModule)
//
// Revision 1.15  2006/05/12 17:03:16  cocoa
// -- swapped arguments in homogenized
//
// Revision 1.14  2006/05/11 16:00:22  cocoa
// -- fixed spelling of "homogenize"
//
// Revision 1.13  2006/05/04 14:25:16  cocoa
// -- major cleaning of FreeModule: created GradedFreeModule and moved
//    some code around
//
// Revision 1.12  2006/04/27 14:03:27  cocoa
// -- tidied up include files
//
// Revision 1.11  2006/04/26 09:50:40  cocoa
// -- added GReductor::ourDefaultStatLevel variable to allow CoCoAServer
//    to set statistics level
//
// Revision 1.10  2006/04/21 16:45:17  cocoa
// -- new functions by Max
//
// Revision 1.9  2006/04/11 16:42:31  cocoa
// -- added functions for Elim
//
// Revision 1.8  2006/03/13 16:56:53  cocoa
// -- changed: member data  myGRingInfoValue  is no longer a reference
//
// Revision 1.7  2006/03/02 13:45:57  cocoa
// -- changes by Max
//
// Revision 1.6  2006/02/13 14:46:45  cocoa
// -- changes by Max
//
// Revision 1.5  2006/02/13 13:45:04  cocoa
// -- changes by Max (GRingInfo)
//
// Revision 1.4  2006/01/18 15:58:20  cocoa
// -- new changes my Max
//
// Revision 1.3  2006/01/17 15:44:56  cocoa
// -- chamges by Max for operations with modules
//
// Revision 1.2  2006/01/17 10:23:08  cocoa
// Updated DivMask; many consequential changes.
// A few other minor fixes.
//
// Revision 1.1.1.1  2005/10/17 10:46:54  cocoa
// Imported files
//
// Revision 1.2  2005/07/01 16:08:16  cocoa
// Friday check-in.  Major change to structure under PolyRing:
// now SparsePolyRing and DUPolyRing are separated (in preparation
// for implementing iterators).
//
// A number of other relatively minor changes had to be chased through
// (e.g. IndetPower).
//
// Revision 1.1.1.1  2005/05/03 15:47:30  cocoa
// Imported files
//
// Revision 1.2  2005/04/19 14:06:04  cocoa
// Added GPL and GFDL licence stuff.
//
// Revision 1.1.1.1  2005/01/27 15:12:13  cocoa
// Imported files
//
// Revision 1.5  2004/10/29 16:02:52  cocoa
// -- added field myDivMaskImplPtr for creating GPolys with the same
//    DivMask::base
// -- calls to LCMwMask (instead of LCM) for fast divisibility tests
// -- function IsDivisible had wrong semantics --> swapped arguments everywhere
//
// Revision 1.4  2004/06/16 16:13:41  cocoa
// Improved I/O facilities with knock-on changes
//
// Revision 1.3  2004/03/04 11:37:17  cocoa
// -- updated code for Borel reductors:
//    ReductorData fields: myGPoly->myGPolyPtr;  new: myCount, IamBorelUpdated
//    myBorelReductors is now in Reductors (was in GReductor)
//    Reductors: field: IhaveBorelReductors;  type: enum UseBorelMarker
//
// Revision 1.2  2003/10/09 12:16:38  cocoa
// New coding convention for rings.
//
// Revision 1.11  2003/06/23 17:09:54  abbott
// Minor cleaning prior to public release.
// Improved the include directives,
//
// Revision 1.10  2003/05/29 16:49:58  bigatti
// - change: myRingSpecialIndex is now an int
// - added:  flag for criterion (to be disables for non-comm Groebner)
//
// Revision 1.9  2003/05/28 14:23:09  bigatti
// - new code for modules
//
// Revision 1.8  2003/05/14 17:06:07  bigatti
// - new functions for "BorelReductorsPolys" and saturating algorithm
//
// Revision 1.7  2002/09/19 17:25:28  bigatti
// - Cleaner code based on PolyRing
//
// Revision 1.6  2002/05/13 11:47:34  bigatti
// - new data structure for "Reductors"
//
// Revision 1.5  2002/04/15 17:17:30  bigatti
// - Max's new code
//
// Revision 1.4  2002/04/09 14:21:57  bigatti
// - CKR criterion
//
// Revision 1.3  2002/01/31 17:29:47  bigatti
// - new function: Stampa_Reductors_Stats
//
// Revision 1.2  2001/12/12 18:19:55  bigatti
// - new structure of reduction
//
// Revision 1.1  2001/12/05 13:27:48  bigatti
// Initial revision
//

#endif
