#ifndef TmpPPVECTOR_H
#define TmpPPVECTOR_H

//   Copyright (c)  2008  Anna M. Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "CoCoA/PPWithMask.H"
#include "CoCoA/MachineInteger.H" // for inline def of operator[]

#include <cstddef>
// using std::size_t;
#include <iosfwd>
// using std::ostream;
#include <vector>
// using std::vector;

namespace CoCoA
{
  class ConstRefPPMonoidElem;   // fwd declaration -- defined in PPMonoid.H
  //  class MachineInteger;   // fwd declaration -- defined in MachineInteger.H
  class PPWithMask;   // fwd declaration -- defined in PPWithMask.H
  class PPMonoid;   // fwd declaration -- defined in PPMonoid.H
  class RingElem;   // fwd declaration -- defined in ring.H
  class SparsePolyRing;   // fwd declaration -- defined in SparsePolyRing.H
  class ring;   // fwd declaration -- defined in ring.H


  class PPVector
  {
  public:
    PPVector(PPMonoid PPM, DivMaskRule DMR);
    ~PPVector() {};
    PPVector& operator=(const PPVector& rhs);
    PPVector(const PPVector& copy);
  private:
  public:
    //      void myOutputSelf(std::ostream& out) const;
    //      void myOutputSelf(OpenMathOutput& OMOut) const;
    std::size_t mySize() const { return myVec.size(); } ///< size of the vector
    const PPWithMask& operator[](MachineInteger i) const { return myVec[AsUnsignedLong(i)]; }
    //    PPVector* myZeroClone() const; ///< creates a PPVector of the same kind
    void myReserve(long n); ///< calls "reserve"
    bool IamEmpty() const; ///< return true if vector is empty
    void myPushBackPopBack(PPVector& FromPPs); ///< push_back(pop_back(FromPPs))
    void myPushBack(ConstRefPPMonoidElem pp); ///< push_back(pp), if owner(pp)!=PPM(PPs) it maps it
    void myPushBack(const PPWithMask& pp); ///< push_back(pp)
    void mySort(); ///< sort
    bool myDivides(const PPWithMask& pp) const; ///< true is pp is divisible by an element of *this
    bool myDivides(ConstRefPPMonoidElem pp) const; ///< true is pp is divisible by an element of *this
    void myInterreduce(); ///< interreduce PPs (NOT exception clean)
    void myInterreduceSort(); ///< interreduce and sort PPs (NOT exception clean)
    void myLcms(const PPVector& PPs1, const PPVector& PPs2); ///< all the lcm between elements of PPs1 and PPs2, effectively the generators of the intersection ideal
    void myConvert(std::vector<RingElem>& v, SparsePolyRing P) const; ///< converts PP's into RingElem's

    friend void swap(PPVector& PPs1, PPVector& PPs2);
    friend PPMonoid PPM(const PPVector& PPs);
    friend DivMaskRule DMR(const PPVector& PPs);
    friend std::ostream& operator<<(std::ostream&, PPVector);
//     friend void PushBack(PPVector& PPs, ConstRefPPMonoidElem pp);
//     friend void PushBackPopBack(PPVector& ToPPs, PPVector& FromPPs);
//     friend bool IsDivisible(const PPWithMask& pp, const PPVector& ByL);
//     friend bool IsDivisible(const PPWithMask& pp, const PPVector& ByL);
//     friend void interreduce(PPVector& PPs);
//     friend void InterreduceSort(PPVector& PPs);
//     friend void lcms(PPVector& PPs, const PPVector& PPs1, const PPVector& PPs2);
  private: // member fields
    PPMonoid myPPM;
    DivMaskRule myDMR;
    std::vector<PPWithMask> myVec;
  };

  void PushBack(PPVector& PPs, ConstRefPPMonoidElem pp);
  void PushBackPopBack(PPVector& ToPPs, PPVector& FromPPs); ///< append(ToPPs, last(FromPPs)), and remove the element from FromPPs
  inline bool IsEmpty(const PPVector& PPs) { return PPs.IamEmpty(); } ///< true if vector is empty
  inline std::size_t len(const PPVector& PPs) { return PPs.mySize(); } ///< the lenght of PPs
  bool IsDivisible(const PPWithMask& pp, const PPVector& ByL); ///< true is pp is divisible by an element of L
  bool IsDivisible(ConstRefPPMonoidElem pp, const PPVector& ByL); ///< true is pp is divisible by an element of L
  void interreduce(PPVector& PPs); ///< interreduce PPs (NOT exception clean)
  void InterreduceSort(PPVector& PPs); ///< interreduce and sort PPs (NOT exception clean)
  void lcms(PPVector& PPs, const PPVector& PPs1, const PPVector& PPs2); ///< all the lcm between elements of PPs1 and PPs2, effectively the generators of the intersection ideal
  void convert(std::vector<RingElem>& v, ring P, const PPVector& PPs); ///< converts PP's into RingElem's
  void convert(PPVector& PPs, const std::vector<RingElem>& v); ///< converts PP's into RingElem's
  void swap(PPVector& PPs1, PPVector& PPs2); ///< swap PPs1 and PPs1
  PPMonoid PPM(const PPVector& PPs); ///< the PPMonoid
  DivMaskRule DMR(const PPVector& PPs); ///< the DivMaskRule

  // void pivot(std::vector<PPWithMask>& PPs); // anna: for Hilbert not in PPs (pp or sp), for mvt in PPs and needs the position for deletion
  // void sort(std::vector<PPWithMask>& PPs);
  // void colon(std::vector<PPWithMask>& PPs, const std::vector<PPWithMask>& PPs1, const std::vector<PPWithMask>& PPs2);
  // void IndetsFrequencies(std::vector<int>& freq, const std::vector<PPWithMask>& PPs);

  //----

// PPVector SplitIndets(PPVector PPs);
// void ReduceAndDivideBySimplePower( PPVector PPs, PPVector *DivTList,
//                                    size_t PIndex, unsigned PExp);
// void ReduceAndDivideByPivot(PPVector PPs, PPVector *DivTList, PPWithMask Pivot);
// void myBigPivot(int *PivotIndex, int *PivotDeg);
// void myGCD3Pivot(PPWithMask pp);

// void MoveNotCoprimeSP(PPWithMask FromSPList, PPWithMask ToSPList, PPWithMask theTerm);

//void InsInTList(PPVector PPs, PPWithMask pp, int *NewLen);

// inline void PPVector::myEraseNth(int N)
// {
//   swap(myPPs[N], myPPs.last());
//   pop_back();
// }


// /********************    list of simple powers    ********************/
// /*
//     SPList(PPs) is represented with a PPWithMask
//     which is the product of the simple powers.
// */
// void InsInSPList(size_t Index, unsigned Exp, PPWithMask SPL);


// /**********************************************************************/
}

// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/TmpPPVector.H,v 1.6 2010/02/03 11:58:03 bigatti Exp $
// $Log: TmpPPVector.H,v $
// Revision 1.6  2010/02/03 11:58:03  bigatti
// -- more flexible convert and PushBack functions
// -- added len(PPs), IsEmpty(PPs)
//
// Revision 1.5  2009/01/26 16:01:50  bigatti
// -- just comments
//
// Revision 1.4  2008/07/04 12:12:15  bigatti
// -- added operator<<
//
// Revision 1.3  2008/07/04 09:11:04  bigatti
// -- new PPVector class
//

#endif /* PPVECTOR_H */
