#ifndef CoCoA_convert_H
#define CoCoA_convert_H

//   Copyright (c)  2007,2009  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.
//   -------------------------------------------------------------------------

#include "CoCoA/error.H"
#include "CoCoA/MachineInteger.H"

#include <limits>
// using std::numeric_limits;  <--- needed only for NumericCast (obsolescent)
#include <string>
// using std::string;

namespace CoCoA
{

  class ZZ; // fwd decl, defined in ZZ.H
  class QQ; // fwd decl, defined in QQ.H

  bool IsConvertible(long& lhs, const ZZ& src);
  bool IsConvertible(int& lhs, const ZZ& src);
  bool IsConvertible(unsigned long& lhs, const ZZ& src);
  bool IsConvertible(unsigned int& lhs, const ZZ& src);
  bool IsConvertible(long& lhs, const QQ& src);
  bool IsConvertible(int& lhs, const QQ& src);
  bool IsConvertible(unsigned long& lhs, const QQ& src);
  bool IsConvertible(unsigned int& lhs, const QQ& src);
  bool IsConvertible(std::string& lhs, const ZZ& src, int base=10);
  bool IsConvertible(ZZ& lhs, const std::string& src, int base=0);
  bool IsConvertible(std::string& lhs, const QQ& src, int base=10);
  bool IsConvertible(QQ& lhs, const std::string& src, int base=0);

///???  bool IsConvertible(ZZ& num, ZZ& den, double z);               ///< !!!OBSOLESCENT!!!  convert double z into an exact rational ==> den will be a power of 2.
  bool IsConvertible(QQ& Q, double z);                          ///< convert double z into an exact rational ==> den will be a power of 2.
  bool IsConvertible(ZZ& N, double z);                          ///< convert double z into an integer, fails if the the value is not an integer
  bool IsConvertible(double& z, const ZZ& N);                   ///< conversion may discard information!
///???  bool IsConvertible(double& z, const ZZ& num, const ZZ& den);  ///< !!!OBSOLESCENT!!! conversion may discard information!
  bool IsConvertible(double& z, const QQ& Q);                   ///< conversion may discard information!


  bool IsInRange(const MachineInteger& lwb, const MachineInteger& val, const MachineInteger& upb);



  // Conversion as a procedure; throws error ERR::BadConvert if it cannot convert.
  template<typename ToType, typename FromType>
  void convert(ToType& dest, const FromType& src)
  {
    if (!IsConvertible(dest, src))
      CoCoA_ERROR(ERR::BadConvert, "convert(ToType&, const FromType&)");
  }

//   // Have to do these separately as they require three args:
//   void convert(std::string& lhs, const ZZ& src, int base=10);
//   void convert(ZZ& lhs, const std::string& src, int base=0);
//   void convert(std::string& lhs, const QQ& src, int base=10);
//   void convert(QQ& lhs, const std::string& src, int base=0);


///??? Is ConvertTo<type>(value) actually useful fo anyone???
///??? Cannot use it to convert to RingElems because type must have default ctor.
//   // Write it this way to allow user to specify just one of the template args when calling.
//   template<typename ToType>
//   class ConvertTo
//   {
//   public:
//     template<typename FromType>
//     ConvertTo(const FromType& val) { if (!IsConvertible(myVal, val)) CoCoA_ERROR(ERR::BadConvert, "ConvertTo"); }
//     template<typename FromType>
//     ConvertTo(const FromType& val, const char* const FnName) { if (!IsConvertible(myVal, val)) CoCoA_ERROR(ERR::BadConvert, FnName); }
//     operator ToType() const { return myVal; }
//   private: // disable copy ctor and assignment
//     ConvertTo(const ConvertTo<ToType>&);                     // NEVER DEFINED -- copy ctor disabled
//     ConvertTo<ToType>& operator=(const ConvertTo<ToType>&);  // NEVER DEFINED -- assignment disabled
//   private: // data member
//     ToType myVal;
//   };


  // (OBSOLESCENT!!!) PLACEHOLDER for BOOST::numeric_cast until we actually require the BOOST libraries
  template<typename ToType>
  class NumericCast
  {
  public:
    template<typename FromType>
    NumericCast(const FromType& val) { if (!IsInRange(std::numeric_limits<ToType>::min(), val, std::numeric_limits<ToType>::max())) CoCoA_ERROR(ERR::BadConvert, "NumericCast"); myVal = static_cast<ToType>(val); /* static_cast will always succeed */}
    operator ToType() const { return myVal; }
  private: // disable copy ctor and assignment
    NumericCast(const NumericCast<ToType>&);                     // NEVER DEFINED -- copy ctor disabled
    NumericCast<ToType>& operator=(const NumericCast<ToType>&);  // NEVER DEFINED -- assignment disabled
  private: // data member
    ToType myVal;
  };


} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/convert.H,v 1.7 2009/12/23 22:27:28 abbott Exp $
// $Log: convert.H,v $
// Revision 1.7  2009/12/23 22:27:28  abbott
// Added conversions from QQ to machine integers.
//
// Revision 1.6  2009/12/23 18:53:52  abbott
// Major change to conversion functions:
//   convert(..) is now a procedure instead of a function
//   IsConvertible(..) replaces the former convert(..) function
//   Added new NumericCast conversion function (placeholder for BOOST feature)
//   Consequent changes in code which uses these features.
//
// Revision 1.5  2009/12/11 11:46:32  abbott
// Changed fn  convert  into  IsConvertible.
// Added template procedure  convert.
// New version because change is not backward compatible.
//
// Revision 1.4  2009/07/02 16:32:11  abbott
// Consequential changes stemming from new class QQ, and modified interface to the member
// function RingBase::myIsRational.  Also some new conversion functions.
//
// Revision 1.3  2008/07/21 07:59:11  abbott
// Improved two comments.
//
// Revision 1.2  2007/10/30 17:14:11  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1  2007/03/23 18:38:43  abbott
// Separated the "convert" functions (and CheckedCast) into their own files.
// Many consequential changes.  Also corrected conversion to doubles.
//
//

#endif
