//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/GlobalManager.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/error.H"

namespace CoCoA
{

  const GlobalManager* GlobalManager::ourGlobalDataPtr = 0;

  // Pseudo-ctors for RingZ and RingQ.
  ring MakeUniqueInstanceOfRingZ(); // Defined in RingZ.C.
  FractionField MakeUniqueInstanceOfRingQ(); // Defined in RingQ.C.

  GlobalManager::GlobalManager(bool ResiduesAreSymmByDefault):
      myResiduesAreSymm(ResiduesAreSymmByDefault)
  {
    // Check whether a GlobalManager object has already been created, complain if so.
    if (ourGlobalDataPtr != 0)
      CoCoA_ERROR(ERR::GlobalManager2, "GlobalManager ctor");
    ourGlobalDataPtr = this; // Must happen before calling MakeUniqueInstanceOfRingQ.
    myRingZPtr.reset(new ring(MakeUniqueInstanceOfRingZ()));
    myRingQPtr.reset(new FractionField(MakeUniqueInstanceOfRingQ()));
  }


  GlobalManager::~GlobalManager()
  {
    if (ourGlobalDataPtr == 0)
      CoCoA_ERROR(ERR::SERIOUS, "GlobalManager dtor");
    ourGlobalDataPtr = 0; // "deregister" the global data
  }

  bool DefaultResiduesAreSymm()
  {
    if (GlobalManager::ourGlobalDataPtr == 0)
      CoCoA_ERROR(ERR::NoGlobalMgr, "DefaultResiduesAreSymm()");
    return GlobalManager::ourGlobalDataPtr->myResiduesAreSymm;
  }

} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/GlobalManager.C,v 1.4 2009/12/03 17:40:36 abbott Exp $
// $Log: GlobalManager.C,v $
// Revision 1.4  2009/12/03 17:40:36  abbott
// Added include directives for ZZ.H (as a consequence of removing
// the directive from ring.H).
//
// Revision 1.3  2009/05/14 09:39:29  abbott
// Added possibility to specify "symmetric" or "non-negative" residues
// in quotients of ZZ.  Affects printing of elements in quotients of ZZ
// (also changed printing of elements in general quotient rings).
// Consequent changes in several tests.
//
// Revision 1.2  2007/10/30 17:14:08  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.2  2007/03/05 21:33:13  cocoa
// Improved/cleaned GlobalManager; added doc too.
//
// Revision 1.1  2007/03/03 14:02:11  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.1  2007/03/02 16:46:28  cocoa
// New foundations object which calls ctors and dtors of global objects.
//
