//   Copyright (c)  1997-2006  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "mpz_log.h"
#include <math.h>

static int MPZ_LOG_SHIFT_MIN = 52;
static double log_2 = 0.0;
static mpz_t MPZ_LOG_AUX;
static int first_time = 1;

/* this should work by a more direct access to the limbs */

/* it returns a close approximation to log(abs(x)) */

double mpz_log(const mpz_t x)
{
  int shift, bitsize;
  double tmp, tmp2; /* to avoid mac68k bug */

  if (first_time) { mpz_init(MPZ_LOG_AUX); first_time = 0; log_2 = log(2.0); }
  bitsize = mpz_sizeinbase(x, 2);
  shift = (bitsize < MPZ_LOG_SHIFT_MIN)? 0 : bitsize - MPZ_LOG_SHIFT_MIN;
  mpz_tdiv_q_2exp(MPZ_LOG_AUX, x, shift);
  mpz_abs(MPZ_LOG_AUX, MPZ_LOG_AUX);
/* Line below replaced by the following lines to avoid a mac68k bug */
/*  return shift*log_2 + log(mpz_get_d(MPZ_LOG_AUX));                */
  tmp = mpz_get_d(MPZ_LOG_AUX);
  if (tmp == 1) tmp2 = 0;
  else tmp2 = log(tmp);
  return shift*log_2 + tmp2;
}


double mpq_log(const mpq_t q)
{
  return mpz_log(mpq_numref(q)) - mpz_log(mpq_denref(q));
}
