//   Copyright (c)  2006  Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


// Source code for Hilbert-Poincare Series

#include "CoCoA/PPMonoid.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/DenseUPolyRing.H"
#include "CoCoA/TmpHilbert.H"
#include "CoCoA/ZZ.H"
#include "CoCoA/ideal.H"
#include "CoCoA/ring.H"
#include "TmpHilbertDir/AnnaUtils.h"
#include "TmpHilbertDir/IVectors.h"
#include "TmpHilbertDir/eterms.h"
#include "TmpHilbertDir/poincare.h"
#include "TmpHilbertDir/TmpPoincareCPP.H"
#include "TmpHilbertDir/unipoly.h"

#include <vector>
using std::vector;
#include <memory>
using std::auto_ptr;

namespace CoCoA
{

  //----------------------------------------------------------------------
  eterm NewEterm(ConstRefPPMonoidElem pp);
  TermList NewLPPTList(const vector<RingElem>& GB);
  RingElem NewPoly(PolyRing HSRing, unipoly f);
  //----------------------------------------------------------------------


  RingElem HilbertNumeratorMod_C(PolyRing HSRing, ideal I)
  {
    const vector<RingElem>& GB = TidyGens(I);
    for (vector<RingElem>::const_iterator it=GB.begin(); it!=GB.end() ; ++it)
      if ( !IsHomog(*it) )
        CoCoA_ERROR("GBasis not homogeneous", "HilbertNumeratorMod");
    ::StartPoincare((int)NumIndets(AsPolyRing(AmbientRing(I))));
    TermList TL = NewLPPTList(GB);
    unipoly PN = TLPoincareNumerator(TL); // TL is freed (?)
    RingElem ans = NewPoly(HSRing, PN);
    FreeUnipoly(PN);
    return ans;
  }


  RingElem HilbertNumeratorMod(DenseUPolyRing HSRing, ideal I)
  {
    const vector<RingElem>& GB = TidyGens(I);
    for (vector<RingElem>::const_iterator it=GB.begin(); it!=GB.end() ; ++it)
      if ( !IsHomog(*it) )
        CoCoA_ERROR("GBasis not homogeneous", "HilbertNumeratorMod");
    StartPoincare(HSRing, (int)NumIndets(AsPolyRing(AmbientRing(I))));
    TermList TL = NewLPPTList(GB);
    RingElem PN = TLPoincareNumeratorCPP(AsDenseUPolyRing(HSRing), TL); // TL is freed (?)
    return PN;
  }


  eterm NewEterm(ConstRefPPMonoidElem pp)
  {
    PPMonoid PPM = owner(pp);
    eterm aux = eterm_init(NumIndets(PPM));
    ints OccInd = Indets(aux);
    vector<long> exps;
    exponents(exps, pp);
    for ( size_t i=0 ; i<NumIndets(PPM) ; ++i )
      if ( exps[i] != 0 )
      {
        eterm_put_nth(aux, i+1, exps[i]); // range from 1
        IntsPutLast(OccInd, i+1);
      }
    return aux;
  }


  TermList NewLPPTList(const vector<RingElem>& GB)
  {
    SparsePolyRing P = AsSparsePolyRing(owner(GB[0]));
    TermList aux = NewTList(GB.size(), NumIndets(P));;
    int NewLen =0;

    for ( size_t i=0 ; i<GB.size() ; ++i )
    {
      eterm t = NewEterm(LPP(GB[i]));
      InsInTList(aux, t, &NewLen);
    }
    TListReduceSize (aux, NewLen);

    return aux;
  }


  RingElem NewPoly(PolyRing HSRing, unipoly f)
  {
    RingElem ans(HSRing);

    if ( HasMPZCoeffs(f) )
      for ( size_t d=0 ; d<=(size_t)UPDeg(f) ; ++d )
        ans += ZZ(CopyFromMPZ, f[d].z) * IndetPower(HSRing,0,d);
    else
      for ( size_t d=0 ; d<=(size_t)UPDeg(f) ; ++d )
        ans += f[d].i * IndetPower(HSRing,0,d);
    return ans;
  }


// the following code is for using a Hilbert-server

//   void SendPPListToHilbert(std::ostream& out, const vector<RingElem>& GB)
//   {
//     SparsePolyRing P = AsSparsePolyRing(owner(GB[0]));

//     // "1 P" stands for unused truncation,
//     out << GB.size() << NumIndets(P) << "1 P" << std::endl << std::endl;
//     for ( size_t i=0 ; i<GB.size() ; ++i )
//       out << LPP(GB[i]) << "," << std::endl;
//   }


//   RingElem ReceivePoincareNumerator(std::istream& in)
//   {
//     PolyRing P = NewPolyRing(NewRingZ(), 1);
//     RingElem ans(P);

//     size_t d;
//     in >> d;
//     ZZ tmpcoeff;
//     for ( size_t i=0 ; i<=d ; ++i )
//     {
//       in >> tmpcoeff;
//       ans += tmpcoeff * IndetPower(P,0,i);
//     }
//     return ans;
//   }


//   RingElem TmpUniPoincareNumeratorMod(ideal I)
//   {
//     // (UsingSocket)
//     const unsigned short HilbertPort = 123456;
//     auto_ptr<SocketStream> SockPtr;
//     SockPtr.reset(new SocketStream(HilbertPort));
//     SetGlobalInput(*SockPtr);
//     SetGlobalOutput(*SockPtr);

//     const vector<RingElem>& GB = TidyGens(I);
//     SendPPListToHilbert(*SockPtr, GB);
//     RingElem ans = ReceivePoincareNumerator(*SockPtr);
//     GlobalLogput() << "poincare numerator = " << ans << std::endl;
//     return ans;
//   }


} // end of namespace CoCoA

// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/TmpHilbert.C,v 1.6 2009/12/03 17:40:36 abbott Exp $
// $Log: TmpHilbert.C,v $
// Revision 1.6  2009/12/03 17:40:36  abbott
// Added include directives for ZZ.H (as a consequence of removing
// the directive from ring.H).
//
// Revision 1.5  2009/10/26 15:45:10  bigatti
// -- added CopyFromMPZ in ZZ ctor
//
// Revision 1.4  2008/03/12 16:35:18  bigatti
// -- changed: IsHomogeneous --> IsHomog
// -- changed: ERR:ZeroPoly --> ERR::ZeroRingElem
//
// Revision 1.3  2007/10/30 17:14:07  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.2  2007/10/10 14:36:34  bigatti
// new: TmpHilbertDir/TmpPoincareCPP.[CH]: poincare code using
// 	C++ univariate polynomials (DenseUPolyRing)
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.3  2007/03/08 18:22:28  cocoa
// Just whitespace cleaning.
//
// Revision 1.2  2007/01/17 17:54:24  bigatti
// -- fixed upper case in includes names
//
// Revision 1.1  2007/01/17 17:38:11  bigatti
// -- moved all cocoa-4 code for hilbert into src/TmpHilbertDir
//
// Revision 1.6  2006/11/27 15:30:04  cocoa
// -- fixed: minor memory leak in Hilbert
//
// Revision 1.5  2006/11/24 17:12:05  cocoa
// -- reorganized includes of header files
//
// Revision 1.4  2006/11/20 15:17:23  cocoa
// -- added check for homogeneity
//
// Revision 1.3  2006/11/17 18:14:01  cocoa
// -- fixed: now Hilbert computes TidyGens instead of cheating
// -- myGBasis is much more efficient on monomial input
//
// Revision 1.2  2006/11/16 17:39:51  cocoa
// -- fixed some calls to rum: lots of code could be deleted, but this is
//    all much more fragile than I would like
//
// Revision 1.1  2006/10/09 16:48:58  cocoa
// -- first import
//
