//   Copyright (c)  2008-2009  Anna Bigatti and Eduardo Saenz-de-Cabezon

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "CoCoA/TmpMonomialFunctions.H"

#include "CoCoA/PPMonoid.H"  // for MultidegreeMap
#include "CoCoA/TmpPPVector.H"
#include "CoCoA/ZZ.H"  // for position_t
#include "CoCoA/io.H"
#include "CoCoA/tmp.H"

using std::size_t;
#include <iostream>

// using std::list;
// #include <utility>
// using std::pair;
 using std::vector;
// using std::endl;

/**********************************************************/

namespace CoCoA
{
void support(std::vector<size_t>& sup, const PPMonoidElem& monomial)
{
  std::vector<long> expv;
  exponents(expv,monomial); 
  sup.clear();
  const size_t size=expv.size();
  for (size_t i=0; i<size; ++i)
    if (expv[i]!=0)
      sup.push_back(i);
}

bool IsIrreducible(const PPVector& ideal)
{
  size_t numgens=len(ideal);
  size_t var;
  ZZ expo;
  for (size_t cont=0; cont<numgens; ++cont)
    if (!IsIndetPosPower(var, expo, PP(ideal[cont])))
      return false;
  return true;
}

bool IsPrime(const PPVector& ideal)
{
  size_t numgens=len(ideal);
  size_t var;
  for (size_t cont=0; cont<numgens; ++cont)
    if (!IsIndet(var, PP(ideal[cont])))
      return false;
  return true;
}

bool IsPrimary(const PPVector& ideal)
{
  size_t numgens=len(ideal);
  size_t var;
  ZZ expo;
  size_t N=NumIndets(PPM(ideal));
  vector<bool> pures(N,false);
  vector<bool> other(N,false);
  for(size_t cont=0; cont<numgens; ++cont)
  {
    if (IsIndetPosPower(var, expo, PP(ideal[cont])))
      pures[var] = true;
    else //I copied part of "support"
    {
      std::vector<long> expv;
      exponents(expv, PP(ideal[cont]));
      for (size_t i=0; i<N; ++i)
        if (expv[i]!=0)
          other[i]=true;
    }
  }
  for (size_t conta=0; conta<N; ++conta)
    if (pures[conta]==false and other[conta]==true)
      return false;
  return true;
}

void ColonIdeal(PPVector& PPs, const PPVector& PPs1, const PPVector& PPs2)
{
  //is it relevant which one has more generators?
  PushBack(PPs,one(PPM(PPs)));
  for(size_t i=0; i<len(PPs2); ++i)
  {
    PPVector current(PPM(PPs),DMR(PPs));
    for (size_t j=0; j<len(PPs1); ++j)
      PushBack(current, colon(PP(PPs1[j]),PP(PPs2[i])));
    interreduce(current); //is it better to just add and then interreduce at the end?
    lcms(PPs, PPs, current);
  }
  interreduce(PPs);
}
  


}  // end of namespace CoCoA

