//   Copyright (c)  2005  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/error.H"
#include "CoCoA/io.H"

#include <iostream>
using std::istream;
using std::ostream;
using std::cin;
using std::cout;
using std::cerr;
using std::clog;


namespace CoCoA
{

  namespace // anonymous namespace
  {
    istream* InPtr = &std::cin;
    ostream* OutPtr = &std::cout;
    ostream* ErrPtr = &std::cerr;
    ostream* LogPtr = &std::clog;
  } // end of anonymous namespace


  istream& GlobalInput()
  {
    return *InPtr;
  }

  ostream& GlobalOutput()
  {
    return *OutPtr;
  }

  ostream& GlobalErrput()
  {
    return *ErrPtr;
  }

  ostream& GlobalLogput()
  {
    return *LogPtr;
  }


  istream& SetGlobalInput(std::istream& in)
  {
    istream& prev = GlobalInput();
    InPtr = &in;
    return prev;
  }

  ostream& SetGlobalOutput(std::ostream& out)
  {
    ostream& prev = GlobalOutput();
    OutPtr = &out;
    return prev;
  }

  ostream& SetGlobalErrput(std::ostream& err)
  {
    ostream& prev = GlobalErrput();
    ErrPtr = &err;
    return prev;
  }

  ostream& SetGlobalLogput(std::ostream& log)
  {
    ostream& prev = GlobalLogput();
    LogPtr = &log;
    return prev;
  }


  void InputFailCheck(const std::istream& in, const char* const FnName)
  {
    if (!in) CoCoA_ERROR(ERR::InputFail, FnName);
  }


} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/io.C,v 1.3 2009/05/20 14:23:48 abbott Exp $
// $Log: io.C,v $
// Revision 1.3  2009/05/20 14:23:48  abbott
// Added InputFailCheck function.
//
// Revision 1.2  2007/10/30 17:14:06  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.1.1.1  2006/05/30 11:39:37  cocoa
// Imported files
//
// Revision 1.1.1.1  2005/10/17 10:46:54  cocoa
// Imported files
//
// Revision 1.1.1.1  2005/05/03 15:47:31  cocoa
// Imported files
//
// Revision 1.2  2005/04/19 14:06:03  cocoa
// Added GPL and GFDL licence stuff.
//
// Revision 1.1.1.1  2005/01/27 15:12:13  cocoa
// Imported files
//
// Revision 1.3  2004/11/12 15:49:29  cocoa
// Tidying prior to 0.90 release.
// (a) documentation improved (or marked as poor)
// (b) sundry minor improvements to the code
//
// Revision 1.2  2004/06/29 17:10:22  cocoa
// Partially tidied use of "protected" and "private" in various
// base classes.  Checking in at the end of the day -- it works,
// and I wouldn't want it to be lost next time point's disk
// misbehaves.
//
// Revision 1.1  2004/06/16 16:13:41  cocoa
// Improved I/O facilities with knock-on changes
//
//

