//   Copyright (c) 2009 Giovanni Lagorio (lagorio@disi.unige.it)
//
//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <boost/tr1/memory.hpp>
#include <readline/readline.h>
#include <readline/history.h>
#include <cstring>
#include <cctype>
#include <boost/iostreams/filter/newline.hpp>

#include "LineProviders.H"

namespace CoCoA {
namespace Lexer {

using namespace std;
using namespace boost;
using namespace boost::iostreams;

ReferenceCountedObject::~ReferenceCountedObject() {}

FileLineProvider::FileLineProvider(const std::string &_filename,  bool _echoToCout) :
	echoToCout(_echoToCout),
	filename(_filename)
{
	input.push(newline_filter(newline::posix));
	file_source fs(_filename.c_str());
	if (!fs.is_open())
		throw IOException("Cannot open file \""+filename+"\" for reading.");
	input.push(fs);
}

const char *InteractiveLineProvider::prompt(const LexerStatus &ls, const CoCoA::Parser::ParserStatus &ps) {
	if (ps.isInRecoveryMode())
		throw CoCoA::Parser::AskingForNewInteractiveInputDuringRecoveryException();
	string s = ps.promptHeader();
	if (ps.isInTheMiddleOfAStatement())
		s.append("... ; # ");
	else if (ls.isInMultiLineComment())
		s.append("/* ... # ");
	else if (ls.isInStringLiteral())
		s.append("\" ... # ");
	else
		s.append("# ");
	return s.c_str();
}

bool ReadlineLineProvider::doReadNextLine(const LexerStatus &ls, const CoCoA::Parser::ParserStatus &ps, string &chars) {
	char *c = ::readline(prompt(ls, ps));
	if (c) {
		::add_history(c);
		chars = string(c);
	}
	return c;
}

bool GetlineLineProvider::doReadNextLine(const LexerStatus &ls, const CoCoA::Parser::ParserStatus &ps, string &chars) {
	if (cin.eof())
		return false;
	cout << prompt(ls, ps) << flush;
	getline(cin, chars);
	return true;
}

bool FileLineProvider::doReadNextLine(const LexerStatus &, const CoCoA::Parser::ParserStatus &, string &chars) {
	if (input.eof())
		return false;
	getline(input, chars);
	if (input.bad())
		throw IOException("Cannot read from \""+filename+"\".");
	if (echoToCout)
		cout << chars << endl;
	return true;
}

} // namespace Lexer
} // namespace CoCoA
