//   Copyright (c) 2009 Giovanni Lagorio (lagorio@disi.unige.it)
//
//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#ifndef PARSER_H_
#define PARSER_H_

#include <vector>
#include "AST.H"

namespace CoCoA {
namespace Parser {

class Parser : public CoCoA::Lexer::ReferenceCountedObject {
	ParserStatus status;
	int nNestingLoops;
	bool insidePackage;
	bool insideDefine;
	CoCoA::Lexer::CharPointer lastRecoveredPosition;
	boost::intrusive_ptr<CoCoA::Lexer::Lexer> lexer;
	CoCoA::Lexer::Token expecting(CoCoA::Lexer::TokenType);
	boost::intrusive_ptr<const CoCoA::AST::RingDefinition> parseRingDefinition();
	boost::intrusive_ptr<const CoCoA::AST::IndeterminateDeclaration> parseIndeterminateDecl(bool isInsideElim);
	std::vector<boost::intrusive_ptr<const CoCoA::AST::Expression> > parseExpressionList();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseConditionalOrExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseConditionalAndExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseEqualityExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseRelationalExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseCartesianProductExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseListExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseAdditiveExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseMultiplicativeExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parsePowerExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseLimitedPowerExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseUnaryExpression();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parsePrimary(CoCoA::Lexer::Token t);
	boost::intrusive_ptr<const CoCoA::AST::RecordExpression> parseRecord(const CoCoA::Lexer::Token &tokRecord);
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseSelectors(boost::intrusive_ptr<const CoCoA::AST::Expression> targetExp);
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseListPrimary(const CoCoA::Lexer::Token &tokOpenSquare);
	boost::intrusive_ptr<const CoCoA::AST::UseStatement> parseUseStatement(const CoCoA::Lexer::Token &tokUse);
	boost::intrusive_ptr<const CoCoA::AST::IdOrVar> parseIdOrVar();
	boost::intrusive_ptr<const CoCoA::AST::IdOrVar> parseIdOrVar(const CoCoA::Lexer::Token &tokIdOrVar);
	boost::intrusive_ptr<const CoCoA::AST::SetStatement> parseSetStatement(const CoCoA::Lexer::Token &tokSet);
	boost::intrusive_ptr<const CoCoA::AST::UnsetStatement> parseUnsetStatement(const CoCoA::Lexer::Token &tokUnset);
	boost::intrusive_ptr<const CoCoA::AST::ForStatement> parseForStatement(const CoCoA::Lexer::Token &tokFor);
	boost::intrusive_ptr<const CoCoA::AST::ForeachStatement> parseForeachStatement(const CoCoA::Lexer::Token &tokForeach);
	boost::intrusive_ptr<const CoCoA::AST::IfStatement> parseIfStatement(const CoCoA::Lexer::Token &tokIf);
	boost::intrusive_ptr<const CoCoA::AST::ReturnStatement> parseReturnStatement(const CoCoA::Lexer::Token &tokReturn);
	boost::intrusive_ptr<const CoCoA::AST::UsingStatement> parseUsingStatement(const CoCoA::Lexer::Token &tokUsing);
	boost::intrusive_ptr<const CoCoA::AST::PrintStatement> parsePrintStatement(const CoCoA::Lexer::Token &tokPrintOrPrintln);
	boost::intrusive_ptr<const CoCoA::AST::BreakStatement> parseBreakStatement(const CoCoA::Lexer::Token &tokBreak);
	boost::intrusive_ptr<const CoCoA::AST::ContinueStatement> parseContinueStatement(const CoCoA::Lexer::Token &tokContinue);
	boost::intrusive_ptr<const CoCoA::AST::SkipStatement> parseSkipStatement(const CoCoA::Lexer::Token &tokSkip);
	boost::intrusive_ptr<const CoCoA::AST::CiaoOrQuitStatement> parseCiaoOrQuitStatement(const CoCoA::Lexer::Token &tokCiaoOrQuit);
	boost::intrusive_ptr<const CoCoA::AST::DescribeStatement> parseDescribeStatement(const CoCoA::Lexer::Token &tokDescribe);
	boost::intrusive_ptr<const CoCoA::AST::RepeatUntilStatement> parseRepeatUntilStatement(const CoCoA::Lexer::Token &tokRepeat);
	boost::intrusive_ptr<const CoCoA::AST::WhileStatement> parseWhileStatement(const CoCoA::Lexer::Token &tokWhile);
	boost::intrusive_ptr<const CoCoA::AST::ClearStatement> parseClearStatement(const CoCoA::Lexer::Token &tokClear);
	boost::intrusive_ptr<const CoCoA::AST::DeleteStatement> parseDeleteStatement(const CoCoA::Lexer::Token &tokDelete);
	boost::intrusive_ptr<const CoCoA::AST::DestroyStatement> parseDestroyStatement(const CoCoA::Lexer::Token &tokDestroy);
	boost::intrusive_ptr<const CoCoA::AST::AliasStatement> parseAliasStatement(const CoCoA::Lexer::Token &tokAlias);
	boost::intrusive_ptr<const CoCoA::AST::BlockStatement> parseBlockStatement(const CoCoA::Lexer::Token &tokBlock);
	boost::intrusive_ptr<const CoCoA::AST::CatchStatement> parseCatchStatement(const CoCoA::Lexer::Token &tokCatch);
	boost::intrusive_ptr<const CoCoA::AST::PackageStatement> parsePackageStatement(const CoCoA::Lexer::Token &tokPackage);
	boost::intrusive_ptr<const CoCoA::AST::SourceStatement> parseSourceStatement(const CoCoA::Lexer::Token &tokSource);
	boost::intrusive_ptr<const CoCoA::AST::DefineStatement> parseDefineStatement(const CoCoA::Lexer::Token &tokDefine);
	void checkThereIsntAnotherUnaryPlusOrMinus(const CoCoA::Lexer::Token &tokFirstOp);
	std::vector<CoCoA::AST::Param> parseParamList();
	boost::intrusive_ptr<const CoCoA::AST::Expression> parseExpOrNewLine();
	CoCoA::Lexer::Token getTokenWithVirtualSemicolon(const CoCoA::Lexer::CharPointer &position);
	std::vector<CoCoA::AST::Binding> parseBindingList(CoCoA::Lexer::CharPointer &position);
	std::vector<boost::intrusive_ptr<const CoCoA::AST::IdOrVar> > parseIdOrVarList(CoCoA::Lexer::CharPointer &position);
	CoCoA::Lexer::Token expectingSemicolon(const CoCoA::Lexer::CharPointer &position, const char * const errorMsg);
	CoCoA::Lexer::Token expectingSemicolon(const CoCoA::Lexer::CharPointer &position) { return expectingSemicolon(position, 0); }
	void enteringBlock(std::string blockName);
	void exitingBlock();
	void reportSkippedInput(const CoCoA::Lexer::CharPointer &from, const CoCoA::Lexer::CharPointer &to);
	boost::intrusive_ptr<const CoCoA::AST::Statement> tryParseStatement();
public:
	explicit Parser(boost::intrusive_ptr<CoCoA::Lexer::Lexer> _lexer) :
		nNestingLoops(0),
		insidePackage(false),
		insideDefine(false),
		lastRecoveredPosition(CoCoA::Lexer::CharPointer::Null),
		lexer(_lexer)
	{
		assert(lexer);
	}
	inline const ParserStatus & getStatus() const { return status; }
	void tryToRecover(const CoCoA::Lexer::CharPointer &beginPosition);
	boost::intrusive_ptr<const CoCoA::AST::Statement> parseStatement();
};

class UnexpectedTokenException : public CoCoA::Lexer::ExceptionWithSourcePosition {
	static std::string humanReadable(CoCoA::Lexer::TokenType tt);
public:
	const CoCoA::Lexer::Token found;
	UnexpectedTokenException(const CoCoA::Lexer::TokenType expected, const CoCoA::Lexer::Token & _found) :
		ExceptionWithSourcePosition("I was expecting "+humanReadable(expected)+" but I've found "+_found.lexeme(), _found, true),
		found(_found)
	{}
	UnexpectedTokenException(const std::string msg, const CoCoA::Lexer::Token & _found) :
		ExceptionWithSourcePosition(msg, _found, true),
		found(_found)
	{}
	~UnexpectedTokenException() throw () {}
};

} // namespace Parser
} // namespace CoCoA

#endif /* PARSER_H_ */
