;; cocoa5.el --- major mode for editing and running CoCoA

;; Copyright (c) 2005-2009  Anna Bigatti
;; This file is part of the CoCoA distribution.
;; You are free to use any part of this file in your own programs.
;; Many thanks to Giovanna Roda and Giorgio Vecchiocattivi 
;;   for their substantial early contributions.
;; cocoa-comint-mode introduced by Burkhard Zimmermann (May 2005)

;; Jun 2005: 
;; tab does both indentation and dabbrev completion -- B.Zimmermann
;; C-c C-e for EndDefine also prints comment with function name
;; F2 goes to error line after sourcing a file -- B.Zimmermann

;; Oct 2005: 
;; "CoCoA not responding?" (panic button) in the menu
;; "Source File Parse Error Line" in the menu

;; Nov 2005: 
;; toggle "TAB for Completion" in the menu

(setq cocoa5-mode-version "20 Aug 2009")

;;==============================================================>
;;     CUSTOMIZATION in cocoa5.emacs
;;==============================================================>

;;==============================================================>
;  \M-x cocoa5
;; will open a buffer called *cocoa5* with a running CoCoA-5

;; you can edit a CoCoA file and send commands from it to *cocoa5*:
;   "\C-c\C-l" 'cocoa5-send-line
;   "\C-c\C-b" 'cocoa5-send-buffer
;   "\C-c\C-r" 'cocoa5-send-region
;   "\C-c\C-f" 'cocoa5-source-file
;   "\C-c\C-c" 'comment-region
;   "\C-c\C-e" 'cocoa5-close-block  ;; writes the appropriate "End"
;   "\C-c\C-w" 'cocoa5-open-wordlist-file
;   "\C-c\C-q" 'cocoa5-shell-quit
;   "\C-c?"    'cocoa5-word-man
;   "\C-cm"    'cocoa5-word-man
;   "\C-c\C-m" 'cocoa5-word-man
;;==============================================================>


(defgroup cocoa5 nil
  "Options for cocoa mode.  'Font Lock' to change syntax colours"
  :prefix "cocoa5-"
  :link '(custom-group-link :tag "Font Lock Faces group" font-lock-faces)
  :group 'languages)

(defcustom cocoa5-alt-executable nil
  "alternative executable"
  :type 'string
  :group 'cocoa5)

(defcustom cocoa5-tab-is-completion nil
  "if 'on' TAB runs auto-completion in addition to  M-/  which runs auto-completion anyway"
  :type 'boolean
  :group 'cocoa5)

;;; End customizable variables.


;;-------------------------------------------------------------;;
;;                    EDITING CoCoA files                      ;;
;;-------------------------------------------------------------;;

;;-------------------------------------------------------------->
;; abbrev-table
;;-------------------------------------------------------------->
;; to use this, in cocoa5-mode, type Esc-x abbrev-mode.
;; it capitalizes the CoCoA-5 keywords 
(defvar cocoa5-abbrev-table nil
  "Abbrev table in use in CoCoA-5 buffers.")

(define-abbrev-table 'cocoa5-abbrev-table '(
    ("alias"   		"Alias"   	nil 0)
    ("and"    		"And"   	nil 0)
    ("break"   		"Break"   	nil 0)
    ("ciao"    		"Ciao;"   	nil 0)
;;obsol    ("cond"    		"Cond"    	nil 0)
    ("define"  		"Define"  	nil 0)
    ("deglex"		"DegLex"     	nil 0)
    ("degrevlex"	"DegRevLex"     nil 0)
    ("do"      		"Do"      	nil 0)
    ("elif"    		"Elif"    	nil 0)
    ("else"    		"Else"    	nil 0)
;;obsol    ("elsif"   		"ElsIf"   	nil 0)
    ("end"     		"End;"    	nil 0)
;;obsol    ("endalias"         "EndAlias;"   	nil 0)
    ("endblock"         "EndBlock;"   	nil 0)
    ("endcatch"         "EndCatch;"   	nil 0)
;;obsol    ("endcond"          "EndCond;"   	nil 0)
    ("enddefine"        "EndDefine;"   	nil 0)
    ("endfor"           "EndFor;"   	nil 0)
    ("endforeach"       "EndForeach;"  	nil 0)
    ("endif"            "EndIf;"   	nil 0)
    ("endpackage"       "EndPackage;"  	nil 0)
    ("endrepeat"        "EndRepeat;"  	nil 0)
    ("endusing"         "EndUsing;"   	nil 0)
    ("endwhile"         "EndWhile;"   	nil 0)
    ("error"   		"Error"    	nil 0)
    ("false"     	"False"     	nil 0)
    ("for"     		"For"     	nil 0)
    ("foreach" 		"Foreach" 	nil 0)
    ("ideal"      	"Ideal"      	nil 0)
    ("if"      		"If"      	nil 0)
    ("in"      		"In"      	nil 0)
    ("isin"            	"IsIn"      	nil 0)
    ("indentation"	"Indentation;"	nil 0)
    ("lex"    		"Lex"  	        nil 0)
    ("mat"    		"Mat"  	        nil 0)
    ("module"    	"Module"        nil 0)
    ("not"	        "Not"           nil 0)
    ("on"    		"On"   	        nil 0)
    ("or"    		"Or"   	        nil 0)
    ("posto"   		"PosTo"   	nil 0)
    ("print"   		"Print"   	nil 0)
    ("println" 		"PrintLn" 	nil 0)
    ("record"  		"Record"  	nil 0)
    ("return"  		"Return"  	nil 0)
    ("set"     		"Set"   	nil 0)
    ("step"    		"Step"   	nil 0)
    ("quit"    		"Quit;"   	nil 0)
    ("then"    		"Then"    	nil 0)
    ("time"    		"Time"    	nil 0)
    ("true"    		"True"    	nil 0)
    ("to"      		"To"      	nil 0)
    ("topos"   		"ToPos"      	nil 0)
    ("unset"     	"Unset"     	nil 0)
    ("use"     		"Use"     	nil 0)
    ("using"   		"Using"   	nil 0)
    ("var"   		"Var"   	nil 0)
    ("vector"  		"Vector"   	nil 0)
    ("while"   		"While"   	nil 0)
    ("xel"    		"Xel"  	        nil 0)
    ))

(defvar cocoa5-imenu-generic-expression
  '("^[ \t]*\\(Define\\)[ \t\n]+\\([a-zA-Z0-9_.:]+\\)" . (2))
  "Imenu expression for Cocoa5-mode.  See `imenu-generic-expression'.")

(defconst cocoa5-startkeywords-list
(concat "\\(Block\\|Catch\\|Define\\|For\\|Foreach\\|"
	"If\\|Package\\|Repeat\\|Using\\|While\\)"
	))  ;; JAA removed Alias & Cond  20090512
(defconst cocoa5-endkeywords-list
(concat "End" 
	"\\(\\|Block\\|Catch\\|Define\\|For\\|Foreach\\|"
	"If\\|Package\\|Repeat\\|Using\\|While\\)"
	))  ;; JAA removed Alias & Cond  20090512

;;-------------------------------------------------------------->
;; Regular expressions used to calculate indent, etc.
;;-------------------------------------------------------------->

(defconst cocoa5-symbol-re      "\\<[a-zA-Z_][a-zA-Z_0-9.]*\\>")
(defconst cocoa5-beg-block-re ;; JAA removed Alias & Cond  20090512
"\\<\\(Block\\|Catch\\|Define\\|If\\|Foreach\\|For\\|Repeat\\|Using\\|While\\)\\>")
(defconst cocoa5-end-block-re
(concat "\\<\\("
	cocoa5-endkeywords-list "\\|"
	"Until"
	"\\)\\>"))
(defconst cocoa5-declaration-re "\\<\\(NIENTE_DA_FISSARE\\)\\>")
(defconst cocoa5-defun-re       "\\<\\(Define\\)\\>")
(defconst cocoa5-sub-block-re   ;; JAA removed Alias & Cond  20090512
"\\<\\(Block\\|Catch\\|If\\|Foreach\\|For\\|Repeat\\|Using\\|While\\)\\>")
(defconst cocoa5-noindent-re
(concat "\\<\\("
	cocoa5-endkeywords-list "\\|"
	"Else\\|Elif\\|Until" ;; JAA replaced Elsif by Elif  20090512
	"\\)\\>"))
(defconst cocoa5-nosemi-re      "\\<\\(Then\\|Do\\|Else\\)\\>")
(defconst cocoa5-autoindent-lines-re
(concat "\\<\\("
	cocoa5-endkeywords-list "\\|"
	"Define\\|While\\|Else"
	"\\)\\>"))

;;; Strings used to mark beginning and end of excluded text
(defconst cocoa5-exclude-str-start "{-----\\/----- EXCLUDED -----\\/-----")
(defconst cocoa5-exclude-str-end " -----/\\----- EXCLUDED -----/\\-----}")

;;-------------------------------------------------------------->
;; cocoa5-mode-syntax-table
;;-------------------------------------------------------------->

(defvar cocoa5-mode-syntax-table nil
  "Syntax table in use in cocoa5-mode buffers.")

(if cocoa5-mode-syntax-table
    ()
  (setq cocoa5-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\\ "."   cocoa5-mode-syntax-table)
  (modify-syntax-entry ?( "()"  cocoa5-mode-syntax-table)
  (modify-syntax-entry ?) ")("  cocoa5-mode-syntax-table)
;  (modify-syntax-entry ?* "." cocoa5-mode-syntax-table)
  (modify-syntax-entry ?+ "."    cocoa5-mode-syntax-table)
  (modify-syntax-entry ?= "."    cocoa5-mode-syntax-table)
  (modify-syntax-entry ?% "."    cocoa5-mode-syntax-table)
  (modify-syntax-entry ?< "."    cocoa5-mode-syntax-table)
  (modify-syntax-entry ?> "."    cocoa5-mode-syntax-table)
  (modify-syntax-entry ?& "."    cocoa5-mode-syntax-table)
  (modify-syntax-entry ?| "."    cocoa5-mode-syntax-table)
  (modify-syntax-entry ?_ "w"    cocoa5-mode-syntax-table)
  ; string
  (modify-syntax-entry ?\' "\""  cocoa5-mode-syntax-table)
  ; -- comments 
  (modify-syntax-entry ?- ". 12"    cocoa5-mode-syntax-table)
  ; // and /* */ comments 
  (modify-syntax-entry ?/  ". 124" cocoa5-mode-syntax-table)
  (modify-syntax-entry ?*  ". 23b" cocoa5-mode-syntax-table)
  (modify-syntax-entry ?\n ">"   cocoa5-mode-syntax-table)
)

;;-------------------------------------------------------------->
;; font-lock-keywords
;;-------------------------------------------------------------->

(defvar cocoa5-font-lock-keywords
  (list
   ; function definition
   '("^[ \t]*\\(Define\\)\\>[ \t]*\\(\\sw+\\)?"
     (1 font-lock-keyword-face) (2 font-lock-function-name-face nil t))
   ; types
   (cons    (concat "\\<\\("
	   "NULL\\|BOOL\\|STRING\\|TYPE\\|ERROR\\|RECORD\\|DEVICE\\|"
	   "INT\\|RAT\\|ZMOD\\|POLY\\|RATFUN\\|VECTOR\\|IDEAL\\|MODULE\\|"
	   "MAT\\|LIST\\|RING\\|TAGGED\\|FUNCTION"
	   "\\)\\>")
	 'font-lock-type-face)
   ; constants
   (cons   (concat "\\<\\("
		   "TRUE\\|FALSE\\|True\\|False\\|"
		   "Lex\\|Xel\\|DegLex\\|DegRevLex\\|"
		   "PosTo\\|ToPos\\|Null\\)\\>") 
	   'font-lock-constant-face)
   ; Ideal, Module...
   (cons   (concat "\\<\\("
		   "Ideal\\|Module\\|Mat\\|Record\\|Vector\\|Error"
		   "\\)\\>") 
	   'font-lock-builtin-face)
   ; keywords
   (concat "\\<\\("
	   "And\\|Or\\|Not\\|Do\\|"
	   "El\\(se\\|if\\)\\|" ;; JAA replaced Elsif by Elif 20090512
	   "Break\\|"
	   cocoa5-startkeywords-list "\\|"
	   cocoa5-endkeywords-list "\\|"
	   "In\\|IsIn\\|"
	   "On\\|Print\\(\\|Ln\\)\\|"
	   "Repeat\\|Until\\|Return\\|Set\\|Step\\|"
	   "T\\(hen\\|ime\\|o\\)\\|"
	   "Unset\\|Use\\|Var"
	   "\\)\\>")
   '("\\<\\(NULLA_DA_FISSARE\\)\\>[ \t]*\\([0-9]+\\)?"
     (1 font-lock-keyword-face) (2 font-lock-reference-face nil t)))
  "Additional expressions to highlight in CoCoA-5 mode.")

;;-------------------------------------------------------------->

(defvar cocoa5-indent-level 2
  "*Indentation of CoCoA-5 statements with respect to containing block.")

(defvar cocoa5-case-indent 2
  "*Indentation for case statements.")

(defvar cocoa5-auto-newline nil
  "*Non-nil means automatically newline after semicolons and the punctuation
mark after an end.")

(defvar cocoa5-tab-always-indent t
  "*Non-nil means TAB in CoCoA-5 mode should always reindent the current line,
regardless of where in the line point is when the TAB command is used.")

(defvar cocoa5-auto-endcomments t
  "*Non-nil means a comment { ... } is set after the ends which ends cases and
functions. The name of the function or case will be set between the braces.")

(defvar cocoa5-auto-lineup '(all)
  "*List of contexts where auto lineup of :'s or ='s should be done.
Elements can be of type: 'paramlist', 'declaration' or 'case', which will
do auto lineup in parameterlist, declarations or case-statements
respectively. The word 'all' will do all lineups. '(case paramlist) for
instance will do lineup in case-statements and parameterlist, while '(all)
will do all lineups.")

;; (defvar cocoa5-toggle-completions nil
;;   "*Non-nil means \\<cocoa5-mode-map>\\[cocoa5-complete-word] should try all
;; possible completions one by one.
;; Repeated use of \\[cocoa5-complete-word] will show you all of them.
;; Normally, when there is more than one possible completion,
;; it displays a list of all possible completions.")

;; (defvar cocoa5-type-keywords
;;   '("NULLA_DA_FISSARE_COCOA5_TYPE_KEYWORDS")
;;   "*Keywords for types used when completing a word in a declaration or
;; parmlist.
;; \(eg. integer, real, char.)  The types defined within the CoCoA program
;; will be completed runtime, and should not be added to this list.")

;; (defvar cocoa5-start-keywords
;;   '("Define" "End" "While")
;;   "*Keywords to complete when standing at the first word of a statement.
;; \(eg. begin, repeat, until, readln.)
;; The procedures and variables defined within the CoCoA program
;; will be completed runtime and should not be added to this list.")

;; (defvar cocoa5-separator-keywords
;;   '("To" "Else" "Mod" "Div" "Then")
;;   "*Keywords to complete when NOT standing at the first word of a statement.
;; \(eg. downto, else, mod, then.)
;; Variables and function names defined within the
;; CoCoA program are completed runtime and should not be added to this list.")


;;;
;;;  Macros
;;;

(defsubst cocoa5-get-beg-of-line (&optional arg)
  (save-excursion
    (beginning-of-line arg)
    (point)))

(defsubst cocoa5-get-end-of-line (&optional arg)
  (save-excursion
    (end-of-line arg)
    (point)))

(defun cocoa5-declaration-end ()
  (let ((nest 1))
    (while (and (> nest 0)
		(re-search-forward
		 "[:=]\\|\\(\\<record\\>\\)\\|\\(\\<end\\>\\)"
		 (save-excursion (end-of-line 2) (point)) t))
      (cond ((match-beginning 1) (setq nest (1+ nest)))
	    ((match-beginning 2) (setq nest (1- nest)))
	    ((looking-at "[^(\n]+)") (setq nest 0))))))


(defun cocoa5-declaration-beg ()
  (let ((nest 1))
    (while (and (> nest 0)
		(re-search-backward
"[:=]\\|\\<\\(NULLA_DECLARATION_BEG\\)\\>\\|\\(\\<record\\>\\)\\|\\(\\<end\\>\\)
" (cocoa5-get-beg-of-line 0) t))
      (cond ((match-beginning 1) (setq nest 0))
	    ((match-beginning 2) (setq nest (1- nest)))
	    ((match-beginning 3) (setq nest (1+ nest)))))
    (= nest 0)))


(defsubst cocoa5-within-string ()
  (save-excursion
    (nth 3 (parse-partial-sexp (cocoa5-get-beg-of-line) (point)))))




;;======================================================================
;;;  Electric functions
;;======================================================================
(defun electric-cocoa5-terminate-line ()
  "Terminate line and indent next line."
  (interactive)
  ;; First, check if current line should be indented
  (save-excursion
    (beginning-of-line)
    (skip-chars-forward " \t")
    (if (looking-at cocoa5-autoindent-lines-re)
	(cocoa5-indent-line)))
  (delete-horizontal-space) ; Removes trailing whitespaces
  (newline)
  ;; Indent next line
  (cocoa5-indent-line)
  ;; Maybe we should set some endcomments
  (if cocoa5-auto-endcomments
      (cocoa5-set-auto-comments))
  ;; Check if we shall indent inside comment
  (let ((setstar nil))
    (save-excursion
      (forward-line -1)
      (skip-chars-forward " \t")
      (cond ((looking-at "\\*[ \t]+/")
	     ;; Delete region between `*' and `/' if there is only whitespaces.
	     (forward-char 1)
	     (delete-horizontal-space))
	    ((and (looking-at "/\\*\\|\\*[^/]")
		  (not (save-excursion
			 (search-forward "\\*/" (cocoa5-get-end-of-line) t))))
	     (setq setstar t))))
    ;; If last line was a star comment line then this one shall be too.
;    (if (null setstar)
;	(cocoa5-indent-line)
;      (insert "*  "))
))

(defun electric-cocoa5-semi-or-dot ()
  "Insert `;' or `.' character and reindent the line."
  (interactive)
  (insert last-command-char)
  ;; Do nothing if within string.
  (if (cocoa5-within-string)
      ()
    (save-excursion
      (beginning-of-line)
      (cocoa5-indent-line))
    (if cocoa5-auto-newline
	(electric-cocoa5-terminate-line))))

;; (defun electric-cocoa5-colon ()
;;   "Insert `:' and do all indentions except line indent on this line."
;;   (interactive)
;;   (insert last-command-char)
;;   ;; Do nothing if within string.
;;   (if (cocoa5-within-string)
;;       ()
;;     (save-excursion
;;       (beginning-of-line)
;;       (cocoa5-indent-line))
;;     (let ((cocoa5-tab-always-indent nil))
;;       (cocoa5-indent-command))))

(defun electric-cocoa5-equal ()
  "Insert `=', and do indention if within type declaration."
  (interactive)
  (insert last-command-char)
  (if (eq (car (cocoa5-calculate-indent)) 'declaration)
      (let ((cocoa5-tab-always-indent nil))
	(cocoa5-indent-command))))

;; (defun electric-cocoa5-hash ()
;;   "Insert `#', and indent to column 0 if this is a CPP directive."
;;   (interactive)
;;   (insert last-command-char)
;;   (if (save-excursion (beginning-of-line) (looking-at "^[ \t]*#"))
;;       (save-excursion (beginning-of-line)
;; 		      (delete-horizontal-space))))

(defun electric-cocoa5-tab-no-completion ()
  "Function called when TAB is pressed in CoCoA-5 mode."
  (interactive)
  ;; Do nothing if within a string or in a CPP directive.
  (if (or (cocoa5-within-string)
	  (and (not (bolp))
	       (save-excursion (beginning-of-line) (eq (following-char) ?#))))
      (insert "\t")
    ;; If cocoa5-tab-always-indent, indent the beginning of the line.
    (if cocoa5-tab-always-indent
	(save-excursion
	  (beginning-of-line)
	  (cocoa5-indent-line))
      (if (save-excursion
	    (skip-chars-backward " \t")
	    (bolp))
	  (cocoa5-indent-line)
	(insert "\t")))
    (cocoa5-indent-command))
)


(defun electric-cocoa5-tab ()
  (interactive)
  (if cocoa5-tab-is-completion
      (progn (electric-cocoa5-tab-no-completion) (dabbrev-expand()))
    (electric-cocoa5-tab-no-completion)
    )
  )



(defun cocoa5-beg-of-defun ()
  "Move backward to the beginning of the current function or procedure."
  (interactive)
  (re-search-backward cocoa5-defun-re nil 'move)
)

(defun cocoa5-end-of-statement ()
  "Move forward to end of current statement."
  (interactive)
  (let ((parse-sexp-ignore-comments t)
	(nest 0) pos
	(regexp (concat "\\(" cocoa5-beg-block-re "\\)\\|\\("
			cocoa5-end-block-re "\\)")))
    (if (not (looking-at "[ \t\n]")) (forward-sexp -1))
    (or (looking-at cocoa5-beg-block-re)
	;; Skip to end of statement
	(setq pos (catch 'found
		    (while t
		      (forward-sexp 1)
		      (cond ((looking-at "[ \t]*;")
			     (skip-chars-forward "^;")
			     (forward-char 1)
			     (throw 'found (point)))
			    ((save-excursion
			       (forward-sexp -1)
			       (looking-at cocoa5-beg-block-re))
			     (goto-char (match-beginning 0))
			     (throw 'found nil))
			    ((eobp)
			     (throw 'found (point))))))))
    (if (not pos)
	;; Skip a whole block
	(catch 'found
	  (while t
	    (re-search-forward regexp nil 'move)
	    (setq nest (if (match-end 1)
			   (1+ nest)
			 (1- nest)))
	    (cond ((eobp)
		   (throw 'found (point)))
		  ((= 0 nest)
		   (throw 'found (cocoa5-end-of-statement))))))
      pos)))

;;;
;;; Other functions
;;;
(defun cocoa5-set-auto-comments ()
  "Insert `{ case }' or `{ NAME }' on this line if appropriate.
Insert `{ case }' if there is an `end' on the line which
ends a case block.  Insert `{ NAME }' if there is an `end'
on the line which ends a function or procedure named NAME."
  (save-excursion
    (forward-line -1)
    (skip-chars-forward " \t")
    (if (and (or (looking-at "\\<EndDefine;") (looking-at "\\<End;") )
	     (not (save-excursion
		    (end-of-line)
		    (search-backward "--" (cocoa5-get-beg-of-line) t))))
	(let ((type (car (cocoa5-calculate-indent))))
	  (if (eq type 'declaration)
	      ()
	    (if (eq type 'case)
		;; This is a case block
		(progn
		  (end-of-line)
		  (delete-horizontal-space)
		  (insert " -- case "))
	      (let ((found-define nil))
		;; Check if this is the end of a function
		(save-excursion
		  (re-search-backward cocoa5-defun-re)
		  (setq found-define (looking-at cocoa5-defun-re)))
		(if found-define
		    (progn
		      (end-of-line)
		      (delete-horizontal-space)
		      (insert " -- ")
		      (let (b e)
			(save-excursion
			  (setq b (progn (cocoa5-beg-of-defun)
					 (skip-chars-forward "^ \t")
					 (skip-chars-forward " \t")
					 (point))
				e (progn (skip-chars-forward "a-zA-Z0-9_")
					 (point))))
			(insert-buffer-substring (current-buffer) b e))
		      (insert ""))))))))))



;;;
;;; Indentation
;;;
(defconst cocoa5-indent-alist
  '((block . (+ ind cocoa5-indent-level))
    (case . (+ ind cocoa5-case-indent))
    (caseblock . ind) (cpp . 0)
    (declaration . (+ ind cocoa5-indent-level))
    (paramlist . (cocoa5-indent-paramlist t))
    (comment . (cocoa5-indent-comment t))
    (defun . ind) (contexp . ind)
    (unknown . 0) (string . 0)))

(defun cocoa5-indent-command ()
  "Indent for special part of code."
  (let* ((indent-str (cocoa5-calculate-indent))
	 (type (car indent-str))
	 (ind (car (cdr indent-str))))
    (cond ((and (eq type 'paramlist)
		(or (memq 'all cocoa5-auto-lineup)
		    (memq 'paramlist cocoa5-auto-lineup)))
	   (cocoa5-indent-paramlist)
	   (cocoa5-indent-paramlist))
	  ((and (eq type 'declaration)
		(or (memq 'all cocoa5-auto-lineup)
		    (memq 'declaration  cocoa5-auto-lineup)))
	   (cocoa5-indent-declaration))
	  ((and (eq type 'case) (not (looking-at "^[ \t]*$"))
		(or (memq 'all cocoa5-auto-lineup)
		    (memq 'case cocoa5-auto-lineup)))
	   (cocoa5-indent-case)))
    (if (looking-at "[ \t]+$")
	(skip-chars-forward " \t"))))

(defun cocoa5-indent-line ()
  "Indent current line as a CoCoA-5 statement."
  (let* ((indent-str (cocoa5-calculate-indent))
	 (type (car indent-str))
	 (ind (car (cdr indent-str))))
    (if (looking-at "^[0-9a-zA-Z]+[ \t]*:[^:=]")
	(search-forward ":" nil t))
    (delete-horizontal-space)
    ;; Some things should not be indented
    (if (or (and (eq type 'declaration) (looking-at cocoa5-declaration-re))
	    (eq type 'cpp)
	    (looking-at cocoa5-defun-re))
	()
      ;; Other things should have no extra indent
      (if (looking-at cocoa5-noindent-re)
	  (indent-to ind)
	;; But most lines are treated this way:
	(indent-to (eval (cdr (assoc type cocoa5-indent-alist))))
	))))

(defun cocoa5-calculate-indent ()
  "Calculate the indent of the current CoCoA-5 line.
Return a list of two elements: (INDENT-TYPE INDENT-LEVEL)."
  (save-excursion
    (let* ((parse-sexp-ignore-comments t)
	   (oldpos (point))
	   (state (save-excursion (parse-partial-sexp (point-min) (point))))
	   (nest 0) (par 0) (complete (looking-at "[ \t]*End\\>"))
	   (elsed (looking-at "[ \t]*Else\\>"))
	   (type (catch 'nesting
		   ;; Check if inside a string, comment or parenthesis
		   (cond ((nth 3 state) (throw 'nesting 'string))
			 ((nth 4 state) (throw 'nesting 'comment))
			 ((> (car state) 0)
			  (goto-char (scan-lists (point) -1 (car state)))
			  (setq par (1+ (current-column))))
			 ((save-excursion (beginning-of-line)
					  (eq (following-char) ?#))
			  (throw 'nesting 'cpp)))
		   ;; Loop until correct indent is found
		   (while t
		     (if (bobp) (throw 'nesting 'unknown))
		     (backward-sexp 1)
		     (cond (;--Escape from case statements
			    (and (looking-at "[A-Za-z0-9]+[ \t]*:[^:=]")
				 (not complete)
				 (save-excursion (skip-chars-backward " \t")
						 (bolp))
				 (= (save-excursion
				      (end-of-line) (backward-sexp) (point))
				    (point))
				 (> (save-excursion (goto-char oldpos)
						    (beginning-of-line)
						    (point))
				    (point)))
			    (throw 'nesting 'caseblock))
			   (;--Nest block outwards
			    (looking-at cocoa5-beg-block-re)
			    (if (= nest 0)
				(cond ((looking-at "NULLA_DA_FISSARE\\>")
				       (throw 'nesting 'case))
				      ((looking-at "NULLA_DA_FISSARE\\>")
				       (throw 'nesting 'declaration))
				      (t (throw 'nesting 'block)))
			      (setq nest (1- nest))))
			   (;--Nest block inwards
			    (looking-at cocoa5-end-block-re)
			    (if (and (looking-at "End\\s ")
				     elsed (not complete))
				(throw 'nesting 'block))
			    (setq complete t
				  nest (1+ nest)))
			   (;--Defun (or parameter list)
			    (looking-at cocoa5-defun-re)
			    (if (= 0 par)
				(throw 'nesting 'defun)
			      (setq par 0)
			      (let ((n 0))
				(while (re-search-forward

	"\\(\\<NULLA_DA_FISSARE\\>\\)\\|\\<NULLA_DA_FISSARE\\>"
					oldpos t)
				  (if (match-end 1)
				      (setq n (1+ n)) (setq n (1- n))))
				(if (> n 0)
				    (throw 'nesting 'declaration)
				  (throw 'nesting 'paramlist)))))
			   (;--Declaration part
			    (looking-at cocoa5-declaration-re)
			    (if (save-excursion
				  (goto-char oldpos)
				  (forward-line -1)
				  (looking-at "^[ \t]*$"))
				(throw 'nesting 'unknown)
			      (throw 'nesting 'declaration)))
			   (;--If, else or while statement
			    (and (not complete)
				 (looking-at cocoa5-sub-block-re))
			    (throw 'nesting 'block))
			   (;--Found complete statement
			    (save-excursion (forward-sexp 1)
					    (= (following-char) ?\;))
			    (setq complete t))
			   (;--No known statements
			    (bobp)
			    (throw 'nesting 'unknown))
			   )))))

      ;; Return type of block and indent level.
      (if (> par 0)                               ; Unclosed Parenthesis
	  (list 'contexp par)
	(list type (cocoa5-indent-level))))))

(defun cocoa5-indent-level ()
  "Return the indent-level the current statement has.
Do not count labels, case-statements or records."
  (save-excursion
    (beginning-of-line)
    (if (looking-at "[ \t]*[0-9a-zA-Z]+[ \t]*:[^:=]")
	(search-forward ":" nil t)
      (if (looking-at ".*=[ \t]*NULLA_DA_FISSARE\\>")
	  (search-forward "=" nil t)))
    (skip-chars-forward " \t")
    (current-column)))

(defun cocoa5-indent-comment (&optional arg)
  "Indent current line as comment.
If optional arg is non-nil, just return the
column number the line should be indented to."
    (let* ((stcol (save-excursion
		    (re-search-backward "--" nil t)
		    (1+ (current-column)))))
      (if arg stcol
	(delete-horizontal-space)
	(indent-to stcol))))


;from b to e nicely. The lineup string is str."

(defun cocoa5-get-lineup-indent (b e str)
  (save-excursion
    (let ((ind 0)
	  (reg (concat str "\\|\\(\\<NULLA_DA_FISSARE\\>\\)"))
	  nest)
      (goto-char b)
      ;; Get rightmost position
      (while (< (point) e)
	(setq nest 1)
	(if (re-search-forward reg (min e (cocoa5-get-end-of-line 2)) 'move)
	    (progn
	      ;; Skip record blocks
	      (if (match-beginning 1)
		  (cocoa5-declaration-end)
		(progn
		  (goto-char (match-beginning 0))
		  (skip-chars-backward " \t")
		  (if (> (current-column) ind)
		      (setq ind (current-column)))
		  (goto-char (match-end 0)))))))
      ;; In case no lineup was found
      (if (> ind 0)
	  (1+ ind)
	;; No lineup-string found
	(goto-char b)
	(end-of-line)
	(skip-chars-backward " \t")
	(1+ (current-column))))))

;;-------------------------------------------------------------->
;; cocoa5-close-block  {like tex-close-latex-block} C-c C-e
;;-------------------------------------------------------------->

(defun cocoa5-last-unended-begin ()
  "Leave point at the beginning of the last command that is unended."
;  (while (and (re-search-backward "\\(\\\\begin\\s *{\\)\\|\\(\\\\end\\s *{\\)")
;              (looking-at "\\\\end{"))
  (while (and (re-search-backward 
	       (concat 
;JAA "^\\(?:[^\"]*\"[^\"]*\"\\)*[^\"]*" ;; something like this should restrict matches to keywords outside strings
		"\\<\\(" cocoa5-startkeywords-list "\\|"
		cocoa5-endkeywords-list "\\)\\>")
	       )
              (looking-at cocoa5-endkeywords-list))
    (cocoa5-last-unended-begin)))

(defun cocoa5-close-block ()
  "Creates an End... to match the last unclosed command."
  (interactive "*")
  (let ((new-line-needed (bolp))
	text indentation)
    (save-excursion
      (condition-case nil
          (cocoa5-last-unended-begin)
        (error (error "Couldn't find unended command")))
      (setq indentation (current-column))
;;      (re-search-forward "[ \n]*\\(\\s *[^)\n ]*\\)")
      (re-search-forward "\\([A-Za-z]+\\)")
      (setq text (buffer-substring (match-beginning 1) (match-end 1))))
    (indent-to indentation)
    (insert "End" text ";")
    (electric-cocoa5-terminate-line)))
;    (if new-line-needed (insert ?\n))))

;;-------------------------------------------------------------;;
;;                        RUNNING CoCoA                        ;;
;;-------------------------------------------------------------;;

(defun cocoa5 ()
  "CoCoA-5 interface.
The function splits the current window in two parts, one containing
a CoCoA-5 shell and the other containing a file chosen by the user
in cocoa5 mode.

The file in cocoa mode can be modified as any text file; in addition
to that, there is a special key binding (C-c C-l) that allows to send
the current line as input to CoCoA. This saves the effort of cutting
and pasting lines of input.

Type C-h C-f cocoa-mode to know about the other functions
available in the cocoa mode.

Customization:
- change the value of the global variable cocoa5-dir to the 
  directory where your CoCoA-5 input files are;
- change the value of cocoa5-shell-launch-dir to the directory where the 
  CoCoA-5 executable is.
"
;  (setq default-directory cocoa5-shell-launch-dir)
  (interactive)
  (require 'comint)
  (require 'easymenu)  ;; MENU

;;;; uncomment the following lines if you have problems with trailing ^M
;;;; you can also add them to your .emacs file
;;(if (null system-uses-terminfo) nil
;;    (autoload 'shell-strip-ctrl-m "shell" nil t)
;;    (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m)
;;    (add-hook 'comint-input-filter-functions 'shell-ctrl-m)
;;)
  (cocoa5-make-shell)
;;  (delete-other-windows)
  (save-excursion
    (switch-to-buffer (process-buffer (get-process "cocoa5")))
;;;; opens a cocoa5 file from cocoa5-dir
;;    (split-window-vertically)
;;    (other-window 1)
  )
)

(defun cocoa5server ()
;  (setq default-directory cocoa5-shell-launch-dir)
  (interactive)
  (require 'comint)
  (require 'easymenu)  ;; MENU

;;;; uncomment the following lines if you have problems with trailing ^M
;;;; you can also add them to your .emacs file
;;(if (null system-uses-terminfo) nil
;;    (autoload 'shell-strip-ctrl-m "shell" nil t)
;;    (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m)
;;    (add-hook 'comint-input-filter-functions 'shell-ctrl-m)
;;)
  (cocoa5-make-server-shell)
  (switch-to-buffer-other-frame "*Cocoa5Server*")
  (previous-multiframe-window) ;possibly (other-frame -1)
)

;;----------------------------------------------------------------------
;; cocoa5-mode-map
;;----------------------------------------------------------------------
(message "cocoa5-mode-map")

(defvar cocoa5-mode-map ()  "Keymap used in CoCoA-5 mode.")
(if cocoa5-mode-map
    ()
;; syntax
  (setq cocoa5-mode-map (make-sparse-keymap))
  (define-key cocoa5-mode-map ";"       'electric-cocoa5-semi-or-dot)
;;  (define-key cocoa5-mode-map "."       'electric-cocoa5-semi-or-dot)
;;  (define-key cocoa5-mode-map ":"       'electric-cocoa5-colon)
  (define-key cocoa5-mode-map "="       'electric-cocoa5-equal)
;;  (define-key cocoa5-mode-map "#"       'electric-cocoa5-hash)
  (define-key cocoa5-mode-map "\r"      'electric-cocoa5-terminate-line)
  (define-key cocoa5-mode-map "\t"      'electric-cocoa5-tab)
  (define-key cocoa5-mode-map "\177"    'backward-delete-char-untabify)
  (define-key cocoa5-mode-map "\M-\C-a" 'cocoa5-beg-of-defun)
;; to send commands to Cocoa5
  (define-key cocoa5-mode-map "\C-c\C-l" 'cocoa5-send-line)
  (define-key cocoa5-mode-map "\C-c\C-r" 'cocoa5-send-region)
  (define-key cocoa5-mode-map "\C-c\C-f" 'cocoa5-source-file)
  (define-key cocoa5-mode-map "\C-c\C-p" 'pop-to-buffer-cocoa5-source-find-error)
  (define-key cocoa5-mode-map "\C-c\C-b" 'cocoa5-send-buffer)
;; other useful things
  (define-key cocoa5-mode-map "\C-c\C-c" 'comment-region)
  (define-key cocoa5-mode-map "\C-c\C-u" 'uncomment-region)
  (define-key cocoa5-mode-map "\C-c\C-e" 'cocoa5-close-block)
  (define-key cocoa5-mode-map "\C-c\C-s" 'cocoa5-restart-cocoa5server)
    (define-key cocoa5-mode-map "\C-c\C-m" 'cocoa5-word-man)
  (define-key cocoa5-mode-map "\C-cm"    'cocoa5-word-man)
  (define-key cocoa5-mode-map "\C-c?"    'cocoa5-word-man)
;  (define-key cocoa5-mode-map "\C-c\C-n" 'cocoa5-open-file)
  (define-key cocoa5-mode-map "\C-c\C-o" 'cocoa5-shell-panic-string)
  (define-key cocoa5-mode-map "\C-c\C-q" 'cocoa5-shell-quit)
  (define-key cocoa5-mode-map "\C-c\C-k" 'cocoa5-restart-shell)
;  (define-key cocoa5-mode-map "\C-c\C-w" 'cocoa5-open-wordlist-file)
)

;;-------------------------------------------------------------->
;; cocoa5-menu
;;-------------------------------------------------------------->
(message "cocoa5-menu")

(defvar cocoa5-mode-menu nil)
(defvar cocoa5-menu nil  "Menu for `cocoa5-mode'.")
(setq cocoa5-menu
      '("CoCoA-5"
	["Send Line to CoCoA-5"             cocoa5-send-line   t]
	["Send Region to CoCoA-5"           cocoa5-send-region t]
;	["Send Buffer to CoCoA-5"           cocoa5-send-buffer t]
	["Source File into CoCoA-5"         cocoa5-source-file t]
	["Source File Parse Error Line"   pop-to-buffer-cocoa5-source-find-error t]
;; 	"---"
;; 	["Use TAB for Completion"         (setq cocoa5-tab-is-completion
;; 					    (not cocoa5-tab-is-completion))
;; 	 :style toggle
;; 	 :selected cocoa5-tab-is-completion
;; 	 :active 't]
	"---"
	["File Functions"                 cocoa5-outline]
	["Manual Word"                    cocoa5-word-man]
	["Complete Word"                  dabbrev-expand]
	["Indent Region"                  indent-region]
	["Comment Region"                 comment-region]
	["Uncomment Region"               uncomment-region]
	["Print End***;"                  cocoa5-close-block]
	"---"
	["Preferences"                    cocoa5-customize]
	["Show cocoa5-mode version"        cocoa5-show-mode-version t]
	["Show cocoa5 version"             cocoa5-show-version t]
	"---"
	["CoCoA-5 not responding?"          cocoa5-shell-panic-string t]
	["(Re)start Cocoa5Server"          cocoa5-restart-cocoa5server t]
	["(Re)start Cocoa54"               cocoa5-restart-shell t]
	["Quit CoCoA-5 and Emacs"           cocoa5-quit-emacs t]
	))

(message "cocoa5-comint-menu")

(defvar cocoa5-comint-mode-menu nil)
(defvar cocoa5-comint-menu nil  "Menu for `cocoa5-comint-mode'.")
(setq cocoa5-comint-menu
      '("CoCoA-5"
	["Source File Parse Error Line"   pop-to-buffer-cocoa5-source-find-error t]
	"---"
	["(Re)start Cocoa5Server"          cocoa5-restart-cocoa5server t]
	["(Re)start Cocoa54"               cocoa5-comint-restart-shell t]
	["Quit CoCoA-5 and Emacs"           cocoa5-quit-emacs t]
	))

;; (defvar cocoa5-statements-menu nil  "Menu for cocoa5 statements.")
;; (setq cocoa5-statements-menu
;;       '("Statements"
;; 	["If"   cocoa5-if-statement]
;; 	["For"   cocoa5-for-statement]
;; 	["Foreach"   cocoa5-foreach-statement]
;; 	["While"   cocoa5-while-statement]
;; ))


;; (defun cocoa5-while-statement ()
;; ""
;; (insert
;; "While . Do
;;   .
;; EndWhile"
;; )
;; )

(defun cocoa5-mode-setup-menubar ()
  "Initial setup of cocoa5 and insertions menus."
  (progn
    (easy-menu-define			; set up cocoa5 menu
      cocoa5-mode-menu cocoa5-mode-map "Menu used in cocoa5-mode"
      cocoa5-menu)
    (easy-menu-add cocoa5-mode-menu cocoa5-mode-map) )
)

(defun cocoa5-comint-setup-menubar ()
  "Initial setup of cocoa5 and insertions menus."
  (progn
    (easy-menu-define			; set up menu
      cocoa5-comint-mode-menu cocoa5-comint-mode-map "Menu used in cocoa5-comint-mode"
      cocoa5-comint-menu)
    (easy-menu-add cocoa5-comint-mode-menu cocoa5-comint-mode-map) )
)



(defconst cocoa5-xemacs-p (string-match "XEmacs" (emacs-version)))

;;-------------------------------------------------------------->
;; cocoa-outline
;;-------------------------------------------------------------->

(defun cocoa5-outline ()
  "The function opens a file outline"
  (interactive)
  (setq cocoa-source-file buffer-file-name)
  (grep  (concat "grep -nH -e \"\\<Define\\>\" " cocoa5-source-file))
)

;;-------------------------------------------------------------->
;; cocoa-customize
;;-------------------------------------------------------------->

(defun cocoa5-customize ()
  "The function opens customize-group cocoa5"
  (interactive)
  (customize-group 'cocoa5)
)

;;-------------------------------------------------------------->
;; cocoa5-open-wordlist-file
;;-------------------------------------------------------------->

(defun cocoa5-open-wordlist-file ()
  "The function opens a wordlist file chosen by the user in cocoa5-mode"
  (interactive)
  (setq cocoa5-wordlist-file 
	(read-file-name "CoCoA-5 word list file: " cocoa5-wordlist-default-file))
  (if cocoa5-wordlist-file  
      (setq cocoa5-wordlist-file cocoa5-wordlist-default-file))
  (find-file-noselect cocoa5-wordlist-file)
)

;;-------------------------------------------------------------->
;; cocoa5-open-file
;;-------------------------------------------------------------->

(setq cocoa5-shell-buffer-list '())

(defun cocoa5-open-file ()
  "The function opens a new file chosen by the user in cocoa5-mode"
  (interactive)
  (setq cocoa5-file 
	(read-file-name "Enter name of a CoCoA-5 file: " cocoa5-dir))
  (setq cocoa5-file-buffer-name (find-file-noselect cocoa5-file))
  (switch-to-buffer cocoa5-file-buffer-name)
  (cocoa5-mode)
  (setq cocoa5-shell-buffer-list 
	(cons cocoa5-file-buffer-name cocoa5-shell-buffer-list))
)

;;-------------------------------------------------------------->
;; cocoa5-make-shell
;;-------------------------------------------------------------->
(message "cocoa5-make-shell")

(defun cocoa5-make-shell ()
  (save-excursion
;;    (setq default-directory cocoa5-shell-launch-dir)
    (set-buffer  (make-comint "cocoa5" 
			      cocoa5-executable nil "-getline"))
    (cocoa5-comint-mode) ;; <-- added line.
;; Load wordlist
    (if cocoa5-auto-load-wordlist
	(find-file-noselect cocoa5-wordlist-default-file 'NOWARN))
  )
)

;;-------------------------------------------------------------->
;; cocoa5-make-server-shell
;;-------------------------------------------------------------->

(defun cocoa5-make-server-shell ()
  (save-excursion
;;    (setq default-directory cocoa5-shell-launch-dir)
    (set-buffer  (make-comint "Cocoa5Server" cocoa5server-executable))
  )
)

;;-------------------------------------------------------------->
;; cocoa5-mode
;;-------------------------------------------------------------->

;;;###autoload
(defun cocoa5-mode ()
  "Major mode for editing a cocoa5 file that sends input to CoCoA-5.
Editing mode with \\[cocoa5-send-line] sending the current line as input.

Special commands:
\\{cocoa5-mode-map}
"
;;  (setq default-directory cocoa5-shell-launch-dir)
  (interactive)
  (kill-all-local-variables)
  (use-local-map cocoa5-mode-map)
  (setq major-mode 'cocoa5-mode
	mode-name "Cocoa5")

  (run-hooks 'cocoa5-mode-hook)
)

;;-------------------------------------------------------------->
;; cocoa5-send-line
;;-------------------------------------------------------------->

(defun cocoa5-send-line ()
  "Send current line as input to CoCoA in the other window.
Note that the cursor can be at any position in the line;
after the input has been sent, the cursor goes to the next non-empty
line in the cocoa file.

Note: if the *cocoa* buffer has no process, CoCoA will be restarted
automatically when this function is called.
"
  (interactive)
  (cocoa5-shell-check-process)
  (beginning-of-line)
  (setq cocoa5-line-beg (point))
  (end-of-line)
  (setq cocoa5-input-line (buffer-substring cocoa5-line-beg (point)))
  (setq cocoa5-source-buffer (current-buffer)) 
  (cocoa5-shell-check-window)
  (save-excursion
;    (switch-to-buffer-other-window "*cocoa5*")
    (pop-to-buffer-cocoa5-comint)
    (goto-char (point-max))
    (insert cocoa5-input-line)
    (comint-send-input)
;    (switch-to-buffer-other-window cocoa5-file-buffer-name)
;    (other-window 1)
    (pop-to-buffer-cocoa5-source)
    )
  (skip-chars-forward "-*\n-*")
)

;;-------------------------------------------------------------->
;; cocoa5-show-version
;;-------------------------------------------------------------->

(defun cocoa5-exec-this (arg)
  "Send arg to Cocoa5
"
  (cocoa5-shell-check-process)
  (cocoa5-shell-check-window)
  (setq cocoa5-source-buffer (current-buffer)) ;  for jumping back to it. Burki
  (save-excursion
;    (switch-to-buffer-other-window "*cocoa5*")
;    (pop-to-buffer "*cocoa5*")
    (pop-to-buffer-cocoa5-comint)
    (goto-char (point-max))
    (insert arg)
    (comint-send-input)
;    (switch-to-buffer-other-window cocoa5-file-buffer-name)
;    (other-window 1)
    (pop-to-buffer-cocoa5-source)
    )
  (skip-chars-forward "-*\n-*")
)

(defun cocoa5-show-version ()
  (interactive)
  (cocoa5-exec-this "Comp(Cocoa5Infos(), \"Version\");")
)

(defun cocoa5-show-mode-version ()
  (interactive)
  (cocoa5-exec-this (concat "-- " cocoa5-mode-version))
)

(defun cocoa5-shell-panic-string ()
  (interactive)
  (cocoa5-exec-this "\"'*/ # \"CoCoA-5 \"+/*NOT*/\"ready\"; -- repeat to get out of pending statements")
)

;;-------------------------------------------------------------->
;; cocoa5-word-man
;;-------------------------------------------------------------->

(defun cocoa5-word-man ()
  "Send current word input to CoCoA-5 'Man' in the *cocoa5* window.
Note that the cursor can be at any position in the word;
after the input has been sent, the cursor goes to the next non-empty
line in the cocoa5 file.
Note: if the *cocoa5* buffer has no process, CoCoA-5 will be restarted
automatically when this function is called.
"
  (interactive)
  (cocoa5-shell-check-process)
  (setq word-regexp "[a-zA-Z][a-zA-Z0-9_]*")
  (setq word-boundary "\\(^\\|[^a-zA-Z_]\\)")
  ;; Now mark the word and save to string.
  (forward-char)
  (re-search-backward (concat word-boundary word-regexp))
  (or (re-search-forward word-regexp (point-max) t)
      (error "No word found to check!"))
  (setq start (match-beginning 0)
	end (point)
	word (buffer-substring start end))
  (cocoa5-exec-this (concat "? " word))
)
;;-------------------------------------------------------------->
;; cocoa5-shell-quit
;;-------------------------------------------------------------->
(message "cocoa5-shell-quit")

(defun cocoa5-shell-quit ()
  "Quit CoCoA-5 after saving all notebook files."
  (interactive)
  (message "saving all notebook files . . .")
  (save-excursion
    (while (not (null cocoa5-shell-buffer-list))
      (setq cocoa5-shell-buffer-tosave (car cocoa5-shell-buffer-list))
      (if (buffer-name cocoa5-shell-buffer-tosave) 
	  (progn 
	    (switch-to-buffer cocoa5-shell-buffer-tosave)
	    (save-buffer)
	    (kill-buffer cocoa5-shell-buffer-tosave)))
      (setq cocoa5-shell-buffer-list (cdr cocoa5-shell-buffer-list))
      ))
  (cocoa5-shell-quit-cocoa5-shell)
  (message "  . . . bye!  ")
)

;;-------------------------------------------------------------->
;; cocoa5-kill-shell-process, cocoa5-kill-server-process
;;-------------------------------------------------------------->

(defun cocoa5-kill-shell-process ()
  (if (not (eq (get-buffer-process "*cocoa5*") nil)) 
      (delete-process "*cocoa5*"))
;;      (process-kill-without-query (get-process "cocoa5")))
)

(defun cocoa5-kill-server-process ()
  (if (not (eq (get-buffer-process "*Cocoa5Server*") nil)) 
      (delete-process "*Cocoa5Server*"))
)

;;-------------------------------------------------------------->
;; cocoa5-quit-emacs
;;-------------------------------------------------------------->

(defun cocoa5-quit-emacs ()
"Quit cocoa5 and then emacs."

(interactive)
;;(process-kill-without-query (get-process "cocoa5"))
(cocoa5-kill-shell-process)
(cocoa5-kill-server-process)
(save-buffers-kill-emacs)
)

;;-------------------------------------------------------------->
;; cocoa5-restart-cocoa5server
;;-------------------------------------------------------------->
(message "cocoa5-restart-cocoa5server")

(defun cocoa5-restart-cocoa5server ()
"(Re)start Cocoa5Server in the buffer named *Cocoa5Server*."

(interactive)
(cocoa5-kill-server-process)
(cocoa5-make-server-shell)
(switch-to-buffer-other-frame "*Cocoa5Server*")
(previous-multiframe-window) ;possibly (other-frame -1)
)

;;-------------------------------------------------------------->
;; cocoa5-restart-shell
;;-------------------------------------------------------------->
(message "cocoa5-restart-shell")

(defun cocoa5-comint-restart-shell ()
"(re)start CoCoA-5 process in the buffer named *cocoa5*."

(interactive)
(cocoa5-kill-shell-process)
(cocoa5-make-shell)
)

(defun cocoa5-restart-shell ()
"(Re)start CoCoA-5 process in the buffer named *cocoa5*."

(interactive)
(cocoa5-kill-shell-process)
(cocoa5-make-shell)
(switch-to-buffer-other-window "*cocoa5*")
(other-window 1)
)

;;-------------------------------------------------------------->
;; cocoa5-send-buffer
;;-------------------------------------------------------------->

(defun cocoa5-send-buffer ()
  "Send current buffer as input to Cocoa5. See \\[cocoa5-send-region] for 
more information. Does not save the buffer, so it's useful for trying 
experimental versions.
See \\[cocoa5-send-file] for an alternative.

Note: if the *cocoa5* buffer has no process, CoCoA-5 will be restarted
automatically when this function is called.
"
  (interactive)
  (cocoa5-shell-check-process)
  (setq kill-ring '())
  (copy-region-as-kill (point-min) (point-max))
  (cocoa5-shell-check-window)
  (save-excursion
    (switch-to-buffer-other-window "*cocoa5*")
    (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
    (goto-char (point-max))
    (yank)
    (comint-send-input)
;  (switch-to-buffer-other-window cocoa5-file-buffer-name)
    (other-window 1)
  )
)

;;-------------------------------------------------------------->
;; cocoa5-send-region
;;-------------------------------------------------------------->

(defun cocoa5-send-region ()
  "Send current region as input to Cocoa5.

Note: if the *cocoa5* buffer has no process, CoCoA-5 will be restarted
automatically when this function is called.
"
  (interactive)
  (cocoa5-shell-check-process)
  (setq kill-ring '())
  (copy-region-as-kill (point) (mark))
  (cocoa5-shell-check-window)
  (save-excursion
    (switch-to-buffer-other-window "*cocoa5*")
    (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
    (goto-char (point-max))
    (yank)
    (comint-send-input)
;    (switch-to-buffer-other-window cocoa5-file-buffer-name)
    (other-window 1)
  )
)

;;-------------------------------------------------------------->
;; cocoa5-source-file
;;-------------------------------------------------------------->

(defun cocoa5-source-file ()
  "Saves current file (no prompt!) and send current file's as input to Cocoa5.

Note: if the *cocoa5* buffer has no process, CoCoA-5 will be restarted
automatically when this function is called.
"
  (interactive)
  (cocoa5-shell-check-process)
  (save-buffer)
  (setq cocoa5-source-buffer (current-buffer)) ;  for jumping back to it. Burki
  (setq cocoa5-source-file buffer-file-name)
  (setq cocoa5-shell-source-command (concat "Source \"" cocoa5-source-file "\";"))
  ;;    (if (cocoa5-shell-running)
  ;;        (cocoa5-shell-kill-job)
  ;;      (cocoa5-make-shell)
  ;;     )
  (cocoa5-shell-check-window)
  (save-excursion
    (pop-to-buffer-cocoa5-comint)
    (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
    (goto-char (point-max))
    (insert cocoa5-shell-source-command)
    (comint-send-input)
    (pop-to-buffer-cocoa5-source)
  )
)

;;-------------------------------------------------------------->
;; cocoa5-shell-check-process
;;-------------------------------------------------------------->
(message "cocoa5-shell-check-process")

(defun cocoa5-shell-check-process ()
  (if (eq (get-buffer-process "*cocoa5*") nil) (cocoa5-make-shell) )
)

(defun cocoa5-shell-check-window ()
  (if (get-buffer-window "*cocoa5*" 0)
      ()
      (progn
	(delete-other-windows)
	(split-window-vertically)
	(other-window -1)
      )
  )
)



;; ;; installation: Append this file to your cocoa5.el file.

;; ;;-------------------------------------------------------------->
;; ;; cocoa5-comint-mode
;; ;; =
;; ;; comint-mode + things for Cocoa5.
;; ;;
;; ;; things = F2 key, ...
;; ;;
;; ;; Burkhard Zimmermann, Friday May 13, 2005.
;; ;;-------------------------------------------------------------->


;; ;; tested with: emacs:              GNU emacs 21.3.1, 
;; ;;              cocoa5-mode-version: "21 December 2004",
;; ;;              Windows XP.

;; (message "Adding F2 to CoCoA-5 mode.")
;; (set 'cocoa5-mode-version (concat cocoa5-mode-version " - [F2]"))

;; ;; F2 = send file to Cocoa5, similarly to C-c C-a.
;; (add-hook 'cocoa5-mode-hook 
;;    (lambda () (define-key cocoa5-mode-map [f2] 'cocoa5-send-file-xxx)
;;               (define-key cocoa5-mode-map [f1] 'cocoa5-word-man))) ; Burki.
;; ; (I put the to the define-key in the hook so that I don't need to modify cocoa5.el at all.)

;; ;; change to the function: cocoa5-make-server-shell
;; (defun cocoa5-make-server-shell ()
;;   (save-excursion
;;     (set-buffer  (make-comint "Cocoa55Server" 
;; 			      (concat cocoa55server-executable)))
;;     (cocoa5-comint-mode) ;; <-- added line.
;;   ))

;; ;;-------------------------------------------------------------->
;; ;; cocoa5-send-file-xxx
;; ;;
;; ;; Similar to cocoa5-send-file. differences:
;; ;;
;; ;; - remembers cocoa5-source-buffer. that's used for jumping back.
;; ;; - gives focus to the comint buffer.
;; ;; - keeps the windows layout. (uses pop-to-buffer. doesn't call cocoa5-shell-check-window)
;; ;;-------------------------------------------------------------->

;; (defun cocoa5-send-file-xxx ()
;;   "Sends the content of the buffer to Cocoa5. Similar to cocoa5-send-file."
;;   (interactive)
;;   (cocoa5-shell-check-process)
;;   (save-buffer) ;; <--- ;; to do: don't save autoamtically. save in a temporal file only. 
;;   (setq cocoa5-shell-input-file buffer-file-name)
;;   (setq cocoa5-shell-input-file-name (concat (concat "<< '" cocoa5-shell-input-file) "';"))
;;   ;; I don't like to have my window layout changed to a horizontal split by:
;;   ;; (cocoa5-shell-check-window)
;;   (save-excursion
;;     (set 'cocoa5-source-buffer (current-buffer)) ; <---  for jumping back to it. ; Burki
;;     (pop-to-buffer "*cocoa5*")
;;     (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
;;     (goto-char (point-max))
;;     (insert cocoa5-shell-input-file-name)
;;     (comint-send-input)
;;   ))


;; ;;-------------------------------------------------------------->
;; ;; cocoa5-comint-mode
;; ;; =
;; ;; comint-mode + things for Cocoa5.
;; ;;-------------------------------------------------------------->

(message "cocoa5-comint-mode")
(defun cocoa5-comint-mode ()
  "Major mode for interacting with Cocoa5. Similar to `comint-mode'.
\\{cocoa5-comint-mode-map}"
  (interactive)
  ;; stolen from maplev.el:
  (comint-mode)
  (setq ;;comint-prompt-regexp (concat "^\\(" maplev-cmaple-prompt "\\)+ *")
          ;; Maple uses "> " where CoCoA-5 uses "" as a prompt.
        ;; comint-use-prompt-regexp-instead-of-fields t
        comint-eol-on-send t ;; ?
        major-mode 'cocoa5-comint-mode
        mode-name  "cocoa5-comint")
  (use-local-map cocoa5-comint-mode-map)
  (cocoa5-comint-setup-menubar)
  (run-hooks 'cocoa5-comint-mode-hook)
  )

;; shouldn't this go into the function cocoa5-comint-mode?
(defvar cocoa5-comint-mode-map nil
  "Keymap used in cocoa5-comint mode.")
(require 'comint)
(unless cocoa5-comint-mode-map
  (let ((map (copy-keymap comint-mode-map)  ))
    (define-key map [f2] 'pop-to-buffer-cocoa5-source-find-error) 
;;    (define-key map [f1] 'cocoa5-word-man) 
    (setq cocoa5-comint-mode-map map)))



;; ; jump to the error line.

;; ; ERROR: parse error in line 5 of device e:\burki\systems\cocoa\test.coc
;; ; -> put the cursor there.
;; ; ERROR: parse error in line 31 of device stdin
;; ; -> take no action.

(defun pop-to-buffer-cocoa5-source-find-error ()
  "If there is an error, set the cursor at the error line."
  (interactive)
  (let ((found nil)
        (emesg nil) 
        (eline nil)
        (efile nil))
  (progn
;;    (message "debug cocoa5-find-error.")
;;    (switch-to-buffer "*cocoa5*") ; useless, since we are already there. 
    (pop-to-buffer-cocoa5-comint)
    (goto-char comint-last-input-start)
    ; search for an error message.
    (if (re-search-forward "^ERROR: \\(.*\\) in line \\([0-9]+\\) of device \\(.*\\)$" nil t)
	(progn 
	   (setq found t)
	   (setq emesg (buffer-substring (match-beginning 1) (match-end 1)))
	   (setq eline (buffer-substring (match-beginning 2) (match-end 2)))
	   (setq efile (buffer-substring (match-beginning 3) (match-end 3)))
	))
        
    (goto-char (point-max)) ; go to the end of the *cocoa5* buffer. 
;;    (message "debug cocoa5-find-error: found=%s, efile=%s, eline=%s, emesg=%s" found efile eline emesg)
    ; messages such as "ERROR: parse error in line 31 of device stdin" 
    ; cannot be used for jumping to the error line.
    ; in such a case, proceed as if no error was found.
    (if (and found (string-equal efile "stdin"))
      (progn
	(message "error in stdin: %s" emesg)
	(setq found nil)
      ))
    ; messages about files that do not exist are useless.
    ; should never happen.
    ; in such a case, proceed as if no error was found.
    (if (and found (not (file-exists-p efile)))
      (progn
	(message "%s, line %s: %s. Emacs can't open the file efile." efile eline emesg efile)
	(setq found nil)
      ))
    (if found
      (progn 
	 (message "%s, line %s: %s" efile eline emesg)

         (pop-to-buffer-cocoa5-source) ;; <--- bad. ignores the efile. 
;;	 (find-file efile)            ;; <--- bad: doesn't behave like pop-to-buffer.

	 (goto-line (string-to-int eline))
      )
      (pop-to-buffer-cocoa5-source) ; no error found. still, go back to the source buffer.
      )
   ))) 

(defun pop-to-buffer-cocoa5-source ()
  "Switch to the cocoa source buffer, if it exists."
  (interactive)
  (try-pop-to-buffer cocoa5-source-buffer))

(defun pop-to-buffer-cocoa5-comint ()
  "Switch to the *cocoa5* comint buffer, if it exists."
  (interactive)
  (try-pop-to-buffer "*cocoa5*"))

(defun try-pop-to-buffer (buffer)
  "Switch to buffer, if it exists."
  (let ((buf (get-buffer buffer)))
    (if buf
; switches buffer, but keeps the window layout:
; never changes the number and the sizes of the visible windows in the current frame.
	(pop-to-buffer buf)  
        ;; would destroy the window layout: (switch-to-buffer buf)
      (message "No buffer \"%s\"." buffer))))


; Optional feature: tab completion.
;; ANNA: achieved in electric-cocoa5-tab
;(add-hook 'cocoa5-mode-hook 
;   (lambda () (local-set-key (kbd "<tab>") 'dabbrev-expand)))

;; (add-hook 'cocoa5-comint-mode-hook 
;;    (lambda () (local-set-key (kbd "<tab>") 'dabbrev-expand)))

(defun cocoa5-syntax () 
  (progn 
  ;; Copied from the function cocoa5-mode. 
  ;; to do: avoid to duplicate code. 
  (setq local-abbrev-table cocoa5-abbrev-table)
  (set-syntax-table cocoa5-mode-syntax-table)
  (make-local-variable 'indent-line-function)
  (setq indent-line-function 'cocoa5-indent-line)
  (make-local-variable 'case-fold-search)
  (setq case-fold-search t)

  (make-local-variable 'comment-start)
  (setq comment-start "--")
  (make-local-variable 'comment-end)
  (setq comment-end "")
  (make-local-variable 'comment-column)
  (setq comment-column 32)
  (make-local-variable 'comment-start-skip)
  (setq comment-start-skip  "/\\*+ *\\|// *")
  (make-local-variable 'comment-indent-function)
  (setq comment-indent-function 'cocoa5-indent-comment)
  (make-local-variable 'parse-sexp-ignore-comments)
  (setq parse-sexp-ignore-comments nil)

;; Font lock support
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(cocoa5-font-lock-keywords nil nil))
;; Imenu support
  (make-local-variable 'imenu-generic-expression)
  (setq imenu-generic-expression cocoa5-imenu-generic-expression)
;; Load wordlist
  (if cocoa5-auto-load-wordlist
	 (find-file-noselect cocoa5-wordlist-default-file 'NOWARN))
;; Menu
  (cocoa5-mode-setup-menubar)
  ))

; Optional feature: syntax highlighting. doesn't work fully. 
;;(add-hook 'cocoa5-comint-mode-hook 'cocoa5-syntax);; loses bindings in the menu!
(add-hook 'cocoa5-mode-hook 'cocoa5-syntax)

;(set 'pop-up-windows nil)

(defun cocoa5-exec-this-and-stay-in-comint (arg)
  "Send arg to Cocoa5
"
;  (cocoa5-shell-check-process)
;;  (cocoa5-shell-check-window) ; <---
  (save-excursion
;    (switch-to-buffer-other-window "*cocoa5*")
    (pop-to-buffer-cocoa5-comint)
;    (if (not (eq (process-status "cocoa5") "run")) (cocoa5-make-shell))
    (goto-char (point-max))
;    (insert "debug ")
    (insert arg)
    (comint-send-input)
;    (switch-to-buffer-other-window cocoa5-file-buffer-name)
;    (other-window 1)

    ;; scroll to the beginning of the help
    (goto-char comint-last-input-start)
    ; todo: scroll so that the line with the cursor is the top line. 
    (recenter 0)

;    (insert "aasdf")
    )
;  (skip-chars-forward "-*\n-*")
)

;; ;(global-set-key [f1] 'cocoa5-word-man)

;; ;; don't split windows:
;; ; (set 'pop-up-windows nil)

;; ;; don't create new frames:
;; ; (set 'pop-up-frames nil)
