//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/CanonicalHom.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/FreeModule.H"
#include "CoCoA/TmpGOperations.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/ModuleTermOrdering.H"
#include "CoCoA/QuotientRing.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingFp.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H"
#include "CoCoA/submodule.H"
#include "CoCoA/symbol.H"

using namespace CoCoA;

#include <iostream>
using std::endl;
#include <vector>
using std::vector;

//----------------------------------------------------------------------
// Test for ideal/module operations (same examples as cocoa5.ts)
// functions: [GBasis], NF, IsElem, IsContained
// environments: DMP (Fp, Q), DMPI (Q), DMPII
//----------------------------------------------------------------------

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


//[Id = "cocoa5-01", Descr = "ReducedGBasis5x Module posto"];
void test1()
{
  SparsePolyRing P(NewPolyRing(RingQ(),3));
  RingElem x = indet(P,0),  y = indet(P,1),  z = indet(P,2);
  //Use Q[x,y,z],PosTo;
  FreeModule FM = NewFreeModule(P, NewWDegPosTO(ordering(PPM(P)),2));
  const vector<ModuleElem>& e = gens(FM);
  vector<ModuleElem> g;
  g.push_back(x*e[0] + z*e[1]);
  g.push_back(z*e[0] + y*e[1]);
  FGModule M = NewSubmodule(FM, g);
  //  vector<ModuleElem> GB = M.myTidyGens();
  vector<ModuleElem> GB;
  ComputeGBasis(GB, gens(M));
  GlobalOutput() << "GB = " << GB << endl;
  // [Vector(z, y), Vector(x, z), Vector(0, xy - z^2)];
}


//[Id = "cocoa5-02", Descr = "ReducedGBasis5x Module ToPos"];
void test2()
{
  SparsePolyRing P(NewPolyRing(RingQ(),3));
  RingElem x = indet(P,0),  y = indet(P,1),  z = indet(P,2);
  //Use Q[x,y,z],PosTo;
  FreeModule FM = NewFreeModule(P, NewWDegTOPos(ordering(PPM(P)),2));
  const vector<ModuleElem>& e = gens(FM);
  vector<ModuleElem> g;
  g.push_back(x*e[0] + z*e[1]); // [x,z]
  g.push_back(z*e[0] + y*e[1]); // [z,y]
  FGModule M = NewSubmodule(FM, g);
  //  vector<ModuleElem> GB = M.myTidyGens();
  vector<ModuleElem> GB;
  ComputeGBasis(GB, gens(M));
  GlobalOutput() << "GB = " << GB << endl;
  // [Vector(z, y), Vector(x, z)];
}


//Test := Record[Id = "cocoa5-03", Descr = "GBasis5x param"];
void test3()
{
  //Use Z/(32003)[a, x,y], Ord(M);
  PolyRing Fpa = NewPolyRing(NewRingFp(32003),symbols("a"));
  ring K = NewFractionField(Fpa);
  PolyRing P(NewPolyRing(K,2));
  RingElem x = indet(P,0),  y = indet(P,1);
  RingElem a = CanonicalHom(K,P)(CanonicalHom(Fpa,K)(indet(Fpa,0)));
  //    I := Ideal((a-1)*x+(a^2+a)*y, (a+1)*x + y);
  ideal I = ideal((a-1)*x+(power(a,2)+a)*y, (a+1)*x + y);
  vector<RingElem> GB = TidyGens(I);
  TEST_ASSERT("cocoa5-3" && GB[0]==x && GB[1]==y);
}


//[Id = "cocoa5-11", Descr = "Intersection5x 2 param"];
void test11()
{
  SparsePolyRing R = NewPolyRing(NewZmod(32003),symbols("a"));
  FractionField K = NewFractionField(R);
  SparsePolyRing P(NewPolyRing(K,2));
  RingHom im = CanonicalHom(K,P);
  RingHom pi = CanonicalHom(R,K);
  RingElem a = im(pi(indet(R,0))); // a is in P
  RingElem x = indet(P,0);
  RingElem y = indet(P,1);
// II := Ideal(x-y);
// I := (a-1) * x * II;
// J := (a+1) * y * II;
  ideal I(P), J(P), X(P);
  I = ideal((a-1) * x * (x-y));
  J = ideal((a+1) * y * (x-y));
// X := Intersection5x(I,J, Record(NumParams=1));
  X = intersect(I, J);
// X = x*y*II;
  TEST_ASSERT("cocoa5-11" && X == ideal(x*y*(x-y)));
}


//[Id = "cocoa5-30", Descr = "Intersection5x multi-param"];
void test30()
{
// II := Ideal(x-y);
// I := (a+b-1) * x * II;
// J := (a-b^2+1) * y * II;
// X := Intersection5x(I,J, Record(NumParams=2));
// X=x*y*II;
  SparsePolyRing R(NewPolyRing(NewZmod(32003),symbols("a","b")));
  FractionField K(NewFractionField(R));
  SparsePolyRing P(NewPolyRing(K,2));
  RingHom im = CanonicalHom(K,P);
  RingHom pi = CanonicalHom(R,K);
  RingElem a = im(pi(indet(R,0))); // a is in P
  RingElem b = im(pi(indet(R,1))); // b is in P
  RingElem x = indet(P,0);
  RingElem y = indet(P,1);
  ideal I(P), J(P), X(P);
  I = ideal((a+b-1) * x * (x-y));
  J = ideal((a-b*b+1) * y * (x-y));
  X = intersect(I,J);
  TEST_ASSERT("cocoa5-30" && X == ideal(x*y*(x-y)));
}


void program()
{
  GlobalManager CoCoAFoundations;
  
  test1();
  test2();
  test3();
  test11();
  test30();
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
