//   Copyright (c)  2008  Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/QuotientRing.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingDistrMPolyInlFpPP.H"
#include "CoCoA/RingDistrMPolyInlPP.H"
#include "CoCoA/RingFp.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H"
#include "CoCoA/symbol.H"

using namespace CoCoA;

#include <iostream>
using std::endl;
#include <vector>
using std::vector;

//----------------------------------------------------------------------
// Test for monomial ideal operations
// functions: GBasis, intersect // todo NF, IsElem, IsContained
// environments: DMP (Fp, Q), DMPI (Q), DMPII
//----------------------------------------------------------------------

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


void TestSparsePolyRing(SparsePolyRing P)
{
  //  GlobalOutput() << "TESTING: " << P << endl << endl;

  RingElem x = indet(P,0);
  RingElem y = indet(P,1);
  RingElem z = indet(P,2);
  ideal I(P), J(P);

  I = ideal(x*y, y*z);
  J = ideal(x*x, x*y*y);
  //  GlobalOutput() << I << endl;
  //  GlobalOutput() << J << endl;
  J = intersect(I, J);
  //  GlobalOutput() << J << endl;
  //  GlobalOutput() << TidyGens(J) << endl;
  TEST_ASSERT(J == ideal(x*y*y, x*x*y));
  const vector<RingElem>& g = TidyGens(J);
  TEST_ASSERT(g.size() == 2);  
}


void program()
{
  GlobalManager CoCoAFoundations;

  const QuotientRing Fp = NewZmod(101);
  const SparsePolyRing Qxyz = NewPolyRing(RingQ(), symbols("x", "y", "z"));
  const SparsePolyRing Qx = NewPolyRing_DMPI(RingQ(), 4);
  const SparsePolyRing Fpy = NewPolyRing_DMPI(Fp, 3);
  const SparsePolyRing FpyII = NewPolyRing_DMPII(Fp, 4);

  TestSparsePolyRing(Qxyz);
  TestSparsePolyRing(Fpy);
  TestSparsePolyRing(FpyII);
  TestSparsePolyRing(Qx);
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
