//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/OpenMathXML.H"
#include "CoCoA/PolyRing.H"
#include "CoCoA/QuotientRing.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingFp.H"
#include "CoCoA/RingZ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/ZZ.H"
#include "CoCoA/io.H"

using namespace CoCoA;
#include <iostream>
using std::cout;
using std::endl;
#include <utility>
using std::pair;
using std::make_pair;

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


void program()
{
  GlobalManager CoCoAFoundations(UseNonNegResidues);

  OpenMathOutput OMOut(new OpenMathOutputXML(cout));

  const ring Z = RingZ();

  RingElem n(Z);
  cout << "OpenMath for RingZ is: " << endl;
  OMOut << Z;
  cout << endl << endl;

  cout << "OpenMath for RingElem(Z, 0) is " << endl;
  OMOut << n;
  cout << endl << endl;

  n = -12345;
  cout << "OpenMath for RingElem(Z, " << n << ") is " << endl;
  OMOut << n;
  cout << endl << endl;

  n = power(n, 100);
  cout << "OpenMath for RingElem(Z, " << n << ") is " << endl;
  OMOut << n;
  cout << endl << endl;

  const ring Z13 = NewZmod(13);
  cout << "OpenMath for Z13 is: " << endl;
  OMOut << Z13;
  cout << endl << endl;

  RingElem r(Z13);
  cout << "OpenMath for RingElem(Z13, " << r << ") is " << endl;
  OMOut << r;
  cout << endl << endl;
  r = 12345;
  cout << "OpenMath for RingElem(Z13, " << r << ") is " << endl;
  OMOut << r;
  cout << endl << endl;

  ring Q = NewFractionField(Z);
//   std::vector<std::pair<std::string,std::size_t> > VarNames;
//   VarNames.push_back(make_pair<std::string,std::size_t>("x",3));
//   PolyRing P = NewPolyRing(Q, NewPPMonoid(NewDegRevLexOrdering(3),VarNames));
  PolyRing P = NewPolyRing(Q, 3);
  cout << "OpenMath for " << P << " is" << endl;
  OMOut << P;
  RingElem x = indet(P, 0);
  RingElem f = (((2*x+4)*x+5)*x+4)*x+2;
  cout << "OpenMath for " << f << " is " << endl;
  OMOut << f;
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
