//   Copyright (c)  2009  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/error.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/io.H"
#include "CoCoA/QQ.H"

#include <iostream>
using std::cout;
using std::endl;

using namespace CoCoA;

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)

void program()
{
  // This test does virtually nothing, but is a handy template if you want
  // to create your own test code: just copy this file and add your code
  // after the line below -- remember that you must not use CoCoALib functions
  // without first creating a GlobalManager.
  GlobalManager CoCoAFoundations;

  int a=13;
  int b=21;

  GlobalOutput() << 101 << " = " << QQ(101) << endl;
  GlobalOutput() << power(101, 9) << " = " << QQ(power(101,9)) << endl;

  // Test the ctors.
  QQ frac(a,b);
  TEST_ASSERT(frac == QQ(a,ZZ(b)));
  TEST_ASSERT(frac == QQ(ZZ(a),b));
  TEST_ASSERT(frac == QQ(ZZ(a),ZZ(b)));

  // Generators are negative
  TEST_ASSERT(frac == QQ(-a,-b));
  TEST_ASSERT(frac == QQ(-a,ZZ(-b)));
  TEST_ASSERT(frac == QQ(ZZ(-a),-b));
  TEST_ASSERT(frac == QQ(ZZ(-a),ZZ(-b)));

  // Generators are not coprime, but positive.
  TEST_ASSERT(frac == QQ(a*b, b*b));
  TEST_ASSERT(frac == QQ(a*b, ZZ(b*b)));
  TEST_ASSERT(frac == QQ(ZZ(a*b), b*b));
  TEST_ASSERT(frac == QQ(ZZ(a*b), ZZ(b*b)));

  // Generators are not coprime, and negative.
  TEST_ASSERT(frac == QQ(-a*b, -b*b));
  TEST_ASSERT(frac == QQ(-a*b, ZZ(-b*b)));
  TEST_ASSERT(frac == QQ(ZZ(-a*b), -b*b));
  TEST_ASSERT(frac == QQ(ZZ(-a*b), ZZ(-b*b)));

  // Now check that a/b - a/b gives zero.
  TEST_ASSERT(IsZero(frac-frac));
  TEST_ASSERT(IsZero(frac + QQ(-a,b)));
  TEST_ASSERT(IsZero(frac + QQ(a,-b)));
  TEST_ASSERT(IsZero(frac + QQ(-a,ZZ(b))));
  TEST_ASSERT(IsZero(frac + QQ(a,ZZ(-b))));
  TEST_ASSERT(IsZero(frac + QQ(ZZ(-a),b)));
  TEST_ASSERT(IsZero(frac + QQ(ZZ(a),-b)));
  TEST_ASSERT(IsZero(frac + QQ(ZZ(-a),ZZ(b))));
  TEST_ASSERT(IsZero(frac + QQ(ZZ(a),ZZ(-b))));

  // Non trivial GCD between the ctor args.
  TEST_ASSERT(IsZero(frac + QQ(-a*b,b*b)));
  TEST_ASSERT(IsZero(frac + QQ(a*b,-b*b)));
  TEST_ASSERT(IsZero(frac + QQ(-a*b,ZZ(b*b))));
  TEST_ASSERT(IsZero(frac + QQ(a*b,ZZ(-b*b))));
  TEST_ASSERT(IsZero(frac + QQ(ZZ(-a*b),b*b)));
  TEST_ASSERT(IsZero(frac + QQ(ZZ(a*b),-b*b)));
  TEST_ASSERT(IsZero(frac + QQ(ZZ(-a*b),ZZ(b*b))));
  TEST_ASSERT(IsZero(frac + QQ(ZZ(a*b),ZZ(-b*b))));


  QQ frac2;
  TEST_ASSERT(IsZero(frac2));
  frac2 += frac;
  TEST_ASSERT(frac2 == frac);
  frac2 -= frac;
  TEST_ASSERT(IsZero(frac2));
  frac2 = frac;
  frac2 *= frac;
  TEST_ASSERT(frac2 == QQ(a*a, b*b));
  frac2 /= frac2;
  TEST_ASSERT(IsOne(frac2));

  frac2 += ZZ(99);
  TEST_ASSERT(frac2 == 100);
  frac2 -= ZZ(-100);
  TEST_ASSERT(frac2 == 200);
  frac2 *= ZZ(-5);
  TEST_ASSERT(frac2 == -1000);
  frac2 /= ZZ(1000);
  TEST_ASSERT(IsMinusOne(frac2));

  frac2 += 11;
  TEST_ASSERT(frac2 == 10);
  frac2 -= -6;
  TEST_ASSERT(frac2 == 16);
  frac2 *= -5;
  TEST_ASSERT(frac2 == -80);
  frac2 /= 9;
  TEST_ASSERT(frac2 == QQ(80,-9));

  TEST_ASSERT(QQ(-71,9) == ++frac2);
  TEST_ASSERT(frac2++ == QQ(71,-9));
  TEST_ASSERT(frac2 == QQ(62,-9));
  TEST_ASSERT(--frac2 == QQ(-71,9));
  TEST_ASSERT(QQ(71,-9) == frac2--);
  TEST_ASSERT(frac2 == QQ(-80,9));

  swap(frac, frac2);
  TEST_ASSERT(frac == QQ(-80,9));
  TEST_ASSERT(frac2 == QQ(a,b));

  TEST_ASSERT(num(frac) == -80);
  TEST_ASSERT(den(frac) == 9);
  TEST_ASSERT(num(abs(frac)) == 80);
  TEST_ASSERT(den(abs(frac)) == 9);
  TEST_ASSERT(num(-frac) == 80);
  TEST_ASSERT(den(-frac) == 9);

  TEST_ASSERT(frac+frac2 == QQ(9*a-80*b,9*b));
  TEST_ASSERT(frac-frac2 == QQ(-9*a-80*b,9*b));
  TEST_ASSERT(frac*frac2 == QQ(-80*a,9*b));
  TEST_ASSERT(frac/frac2 == QQ(-80*b,9*a));

  TEST_ASSERT(frac+ZZ(10) == QQ(10,9));
  TEST_ASSERT(frac-ZZ(10) == QQ(-170,9));
  TEST_ASSERT(frac*ZZ(10) == QQ(-800,9));
  TEST_ASSERT(frac/ZZ(10) == QQ(-8,9));

  TEST_ASSERT(ZZ(-10)+frac == QQ(-170,9));
  TEST_ASSERT(ZZ(-10)-frac == QQ(-10,9));
  TEST_ASSERT(ZZ(-10)*frac == QQ(800,9));
  TEST_ASSERT(ZZ(-10)/frac == QQ(9,8));

  TEST_ASSERT(frac+10 == QQ(10,9));
  TEST_ASSERT(frac-10 == QQ(-170,9));
  TEST_ASSERT(frac*10 == QQ(-800,9));
  TEST_ASSERT(frac/10 == QQ(-8,9));

  TEST_ASSERT(10+frac == QQ(10,9));
  TEST_ASSERT(10-frac == QQ(170,9));
  TEST_ASSERT(10*frac == QQ(-800,9));
  TEST_ASSERT(10/frac == QQ(-9,8));

  QQ frac3 = power(frac2, 17);
  TEST_ASSERT(frac3 == QQ(power(a,17), power(b,17)));

  frac3 = power(frac2, ZZ(255));
  TEST_ASSERT(frac3 == QQ(power(a,255), power(b,255)));

  TEST_ASSERT(!IsZero(frac));
  TEST_ASSERT(!IsOne(frac));
  TEST_ASSERT(IsOne(frac/frac));
  TEST_ASSERT(!IsMinusOne(frac/frac));
  TEST_ASSERT(IsMinusOne(frac*((-1)/frac)));
  TEST_ASSERT(sign(frac) == -1 && sign(frac2) == 1);

  // Comparisons
  TEST_ASSERT(cmp(frac, frac) == 0);
  TEST_ASSERT(cmp(frac, -frac) < 0);
  TEST_ASSERT(cmp(-frac, frac) > 0);

  TEST_ASSERT(cmp(frac, -ZZ(9)) > 0);
  TEST_ASSERT(cmp(frac, -ZZ(8)) < 0);
  TEST_ASSERT(cmp(ZZ(9), -frac) > 0);
  TEST_ASSERT(cmp(ZZ(8), -frac) < 0);

  TEST_ASSERT(cmp(frac, -9) > 0);
  TEST_ASSERT(cmp(frac, -8) < 0);
  TEST_ASSERT(cmp(9, -frac) > 0);
  TEST_ASSERT(cmp(8, -frac) < 0);

  TEST_ASSERT(frac == frac);
  TEST_ASSERT(!(frac != frac));
  TEST_ASSERT(!(frac == frac2));
  TEST_ASSERT(frac != frac2);
  TEST_ASSERT(!(frac < frac));
  TEST_ASSERT(frac < frac2);
  TEST_ASSERT(frac <= frac);
  TEST_ASSERT(frac <= frac2);
  TEST_ASSERT(!(frac > frac));
  TEST_ASSERT(frac2 > frac);
  TEST_ASSERT(frac >= frac);
  TEST_ASSERT(frac2 >= frac);

  TEST_ASSERT(!(frac == ZZ(-9)));
  TEST_ASSERT(frac != ZZ(-9));
  TEST_ASSERT(frac > ZZ(-9));
  TEST_ASSERT(frac >= ZZ(-9));
  TEST_ASSERT(frac < ZZ(-8));
  TEST_ASSERT(frac <= ZZ(-8));

  TEST_ASSERT(!(ZZ(-9) == frac));
  TEST_ASSERT(ZZ(-9) != frac);
  TEST_ASSERT(ZZ(-8) > frac);
  TEST_ASSERT(ZZ(-8) >= frac);
  TEST_ASSERT(-ZZ(9) < frac);
  TEST_ASSERT(-ZZ(9) <= frac);

  TEST_ASSERT(!(frac == -9));
  TEST_ASSERT(frac != -9);
  TEST_ASSERT(frac > -9);
  TEST_ASSERT(frac >= -9);
  TEST_ASSERT(frac < -8);
  TEST_ASSERT(frac <= -8);

  TEST_ASSERT(!(-9 == frac));
  TEST_ASSERT(-9 != frac);
  TEST_ASSERT(-8 > frac);
  TEST_ASSERT(-8 >= frac);
  TEST_ASSERT(-9 < frac);
  TEST_ASSERT(-9 <= frac);

  TEST_ASSERT(round(frac) == -9);
  TEST_ASSERT(round(QQ(1,2)) == 1);
  TEST_ASSERT(round(QQ(-1,2)) == 0);
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
