//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/CanonicalHom.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/PolyRing.H"
#include "CoCoA/QuotientRing.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingDistrMPolyInlFpPP.H"
#include "CoCoA/RingDistrMPolyInlPP.H"
#include "CoCoA/RingFp.H"
#include "CoCoA/RingHom.H"
#include "CoCoA/RingZ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H"
#include "CoCoA/ring.H"

using namespace CoCoA;

#include <iostream>
using std::cout;
using std::endl;
#include <vector>
using std::vector;

//----------------------------------------------------------------------
// Test for RingHom
// Zxy->FpxII, FpxII->Fpy, FpxII->FpxII
//----------------------------------------------------------------------

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)

void program()
{
  GlobalManager CoCoAFoundations;

  const QuotientRing Fp = NewZmod(101);
  const PolyRing Zxy = NewPolyRing(RingZ(), 2);
  const PolyRing Fpy = NewPolyRing(Fp, 2);
  const PolyRing FpxI = NewPolyRing_DMPI(Fp, 2);
  const PolyRing FpxII = NewPolyRing_DMPII(Fp, 2);

//   RingHom ZToFpxI = ZEmbeddingHom(FpxI);
//   RingHom ZToFpxII = ZEmbeddingHom(FpxII);
  RingHom ZToFpxI = CanonicalHom(RingZ(), FpxI);
  RingHom ZToFpxII = CanonicalHom(RingZ(), FpxII);

  const vector<RingElem>& x = indets(FpxII);
  vector<RingElem> imx;
  imx.push_back(x[0]*x[0]);
  imx.push_back(x[1]*x[1]);

  const vector<RingElem>& z = indets(FpxI);
  vector<RingElem> imx1;
  imx1.push_back(z[0]*z[0]);
  imx1.push_back(z[1]*z[1]);

  const vector<RingElem>& y = indets(Fpy);
  vector<RingElem> imx2;
  imx2.push_back(y[0]*y[0]);
  imx2.push_back(y[1]*y[1]);

  RingHom ZxyToFpxII = PolyRingHom(Zxy, FpxII, ZToFpxII, imx);
  RingHom FpxIIToFpxII = PolyRingHom(FpxII, FpxII, CoeffEmbeddingHom(FpxII), imx);
  RingHom FpxIIToFpy = PolyRingHom(FpxII, Fpy, CoeffEmbeddingHom(Fpy), imx2);

  RingHom ZxyToFpxI = PolyRingHom(Zxy, FpxI, ZToFpxI, imx1);
//   RingHom FpxIToFpxI = PolyRingHom(FpxI, FpxI, CoeffEmbeddingHom(FpxI), imx1);
//   RingHom FpxIToFpy = PolyRingHom(FpxI, Fpy, CoeffEmbeddingHom(Fpy), imx2);
  RingHom FpxIToFpxI = PolyAlgebraHom(FpxI, FpxI, imx1);
  RingHom FpxIToFpy = PolyAlgebraHom(FpxI, Fpy, imx2);

  cout << "Simple test on polynomial ring hom" << endl
       << Zxy << "  -->  " << FpxII << endl
       << "sending " << indet(Zxy,0) << "  to  " << imx[0] << "  and" << endl
       << "sending " << indet(Zxy,1) << "  to  " << imx[1] << endl;

  RingElem f = indet(Zxy,0) + 2*indet(Zxy,1);
  RingElem g = ZxyToFpxII(f);
  RingElem h = ZxyToFpxI(f);

  cout << f << " in " << owner(f) << "  maps to " << g << " in " << owner(g) << endl;
  cout << g << " in " << owner(g) << "  maps to " << FpxIIToFpxII(g) << " in " << owner(FpxIIToFpxII(g)) << endl;
  cout << g << " in " << owner(g) << "  maps to " << FpxIIToFpy(g) << " in " << owner(FpxIIToFpy(g)) << endl;

  cout << f << " in " << owner(f) << "  maps to " << h << " in " << owner(h) << endl;
  cout << h << " in " << owner(h) << "  maps to " << FpxIToFpxI(h) << " in " << owner(FpxIToFpxI(h)) << endl;
  cout << h << " in " << owner(h) << "  maps to " << FpxIToFpy(h) << " in " << owner(FpxIToFpy(h)) << endl;

  vector<RingElem> imx3;
  imx3.push_back(RingElem(Fp,13));
  imx3.push_back(RingElem(Fp,31));

  RingHom EvalFpxI = PolyAlgebraHom(FpxI, Fp, imx3);
  RingHom EvalFpxII = PolyAlgebraHom(FpxII, Fp, imx3);
  TEST_ASSERT(EvalFpxII(g) == EvalFpxI(h));
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
