//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/PPMonoidEv.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingTwinFloat.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H"
#include "CoCoA/ring.H"
#include "CoCoA/symbol.H"

using namespace CoCoA;

#include <cstddef>
using std::size_t;
#include <iostream>
using std::cout;
using std::endl;
#include <vector>
using std::vector;

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


void solve(size_t PrecBits)
{
  const ring RR = NewRingTwinFloat(PrecBits);

  const PolyRing P = NewPolyRing(RR, SymbolRange("x", 0, 2), NewStdDegLexOrdering(3));

  const RingElem x = indet(P, 0);
  const RingElem y = indet(P, 1);
  const RingElem z = indet(P, 2);
  const RingElem f = power(x,41)-power(z,40)*(x-z);
//  RingElem g = power(x,5)*y-2*power(x,3)-3*x+1;
  const RingElem g = power(x,6)- 9*power(x,5)*z+x*power(z,5)+11*power(z,6)-power(y,6);

  vector<RingElem> gens;
  gens.push_back(f);
  gens.push_back(g);
  ideal I = ideal(gens);

  vector<RingElem> GI = CoCoA::gens(I);
  vector<RingElem> GBasis = TidyGens(I); // might throw an exception
  std::cout<<"DONE "<<GBasis.size()<<endl;
  TEST_ASSERT(PrecBits >= 121);
  TEST_ASSERT(GBasis.size() == 64);
  TEST_ASSERT(GBasis[0] == g);
  for (size_t i=0; i < GBasis.size(); ++i)
  {
    GlobalOutput() << PP(BeginIter(GBasis[i])) << "  ";
  }
  GlobalOutput() << endl;
}


void program()
{
  GlobalManager CoCoAFoundations;

  for (size_t PrecBits = 32;; ++PrecBits)
  {
    std::cout<<"prec="<<PrecBits<<endl;
    try
    {
      solve(PrecBits);
      if (PrecBits > 199)      break;
    }
    catch (const RingTwinFloat::InsufficientPrecision&) {}
  }
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
