//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/FreeModule.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/QuotientRing.H"
#include "CoCoA/RingTwinFloat.H"
#include "CoCoA/RingFp.H"
#include "CoCoA/RingZ.H"
#include "CoCoA/ZZ.H"
#include "CoCoA/io.H"
#include "CoCoA/submodule.H"

#include <iostream>
using std::cout;
using std::endl;
#include <vector>
using std::vector;

using namespace CoCoA;

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


// This trial is still very rudimentary..

void trial(ring R)
{
  const size_t dim = 4;
  const FreeModule F = NewFreeModule(R, dim);
  const vector<ModuleElem>& Fgens = gens(F);
  TEST_ASSERT(Fgens.size() == dim);
// This does not yet compile
//   for (size_t i=0; i < dim; ++i)
//     for (size_t j=0; j < dim; ++j)
//       TEST_ASSERT(Fgens[i][j] == (i==j));
  TEST_ASSERT(IsFGModule(F));

  const ModuleElem& v0 = Fgens[0];
  const ModuleElem& v1 = Fgens[1];
  const ModuleElem& v2 = Fgens[2];
  const ModuleElem& v3 = Fgens[3];
  vector<ModuleElem> Mgens;
  Mgens.push_back(1*v0 + 2*v1 + 3*v2 + 4*v3);
  Mgens.push_back(4*v0 + 3*v1 + 2*v2 + 1*v3);

  const FGModule M = NewSubmodule(F, Mgens);
  // M is submodule generated by elems of Mgens
  TEST_ASSERT(gens(M).size() == 2);
}


void program()
{
  GlobalManager CoCoAFoundations;

  trial(RingZ());
  trial(NewRingTwinFloat(32));
  trial(NewZmod(2));
  trial(NewZmod(1048576)); // ring has zero divisors
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
